@@ -8,19 +8,20 @@
                Home page: http://www.omegat.org/
                Support center: http://groups.yahoo.com/group/OmegaT/
 
- This program is free software; you can redistribute it and/or modify
+ This file is part of OmegaT.
+
+ OmegaT is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
+ OmegaT is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
 package org.omegat.core.segmentation;
@@ -39,27 +40,37 @@ import org.omegat.util.OStrings;
  */
 public final class LanguageCodes {
     // Language Codes
-    public static final String ENGLISH_CODE = "English";
-    public static final String JAPANESE_CODE = "Japanese";
-    public static final String RUSSIAN_CODE = "Russian";
-    public static final String GERMAN_CODE = "German";
     public static final String CATALAN_CODE = "Catalan";
+    public static final String GERMAN_CODE = "German";
+    public static final String ENGLISH_CODE = "English";
     public static final String SPANISH_CODE = "Spanish";
-    public static final String POLISH_CODE = "Polish";
+    public static final String FINNISH_CODE = "Finnish";
+    public static final String FRENCH_CODE = "French";
+    public static final String ITALIAN_CODE = "Italian";
+    public static final String JAPANESE_CODE = "Japanese";
     public static final String DUTCH_CODE = "Dutch";
+    public static final String POLISH_CODE = "Polish";
+    public static final String RUSSIAN_CODE = "Russian";
+    public static final String SWEDISH_CODE = "Swedish";
+    public static final String CHINESE_CODE = "Chinese";
     public static final String DEFAULT_CODE = "Default";
     public static final String F_TEXT_CODE = "Text";
     public static final String F_HTML_CODE = "HTML";
 
     // Language Keys from Resource Bundle
-    public static final String ENGLISH_KEY = "CORE_SRX_RULES_LANG_ENGLISH";
-    public static final String JAPANESE_KEY = "CORE_SRX_RULES_LANG_JAPANESE";
-    public static final String RUSSIAN_KEY = "CORE_SRX_RULES_LANG_RUSSIAN";
-    public static final String GERMAN_KEY = "CORE_SRX_RULES_LANG_GERMAN";
     public static final String CATALAN_KEY = "CORE_SRX_RULES_LANG_CATALAN";
+    public static final String GERMAN_KEY = "CORE_SRX_RULES_LANG_GERMAN";
+    public static final String ENGLISH_KEY = "CORE_SRX_RULES_LANG_ENGLISH";
     public static final String SPANISH_KEY = "CORE_SRX_RULES_LANG_SPANISH";
-    public static final String POLISH_KEY = "CORE_SRX_RULES_LANG_POLISH";
+    public static final String FINNISH_KEY = "CORE_SRX_RULES_LANG_FINNISH";
+    public static final String FRENCH_KEY = "CORE_SRX_RULES_LANG_FRENCH";
+    public static final String ITALIAN_KEY = "CORE_SRX_RULES_LANG_ITALIAN";
+    public static final String JAPANESE_KEY = "CORE_SRX_RULES_LANG_JAPANESE";
     public static final String DUTCH_KEY = "CORE_SRX_RULES_LANG_DUTCH";
+    public static final String POLISH_KEY = "CORE_SRX_RULES_LANG_POLISH";
+    public static final String RUSSIAN_KEY = "CORE_SRX_RULES_LANG_RUSSIAN";
+    public static final String SWEDISH_KEY = "CORE_SRX_RULES_LANG_SWEDISH";
+    public static final String CHINESE_KEY = "CORE_SRX_RULES_LANG_CHINESE";
     public static final String DEFAULT_KEY = "CORE_SRX_RULES_LANG_DEFAULT";
     public static final String F_TEXT_KEY = "CORE_SRX_RULES_FORMATTING_TEXT";
     public static final String F_HTML_KEY = "CORE_SRX_RULES_FORMATTING_HTML";
@@ -68,14 +79,19 @@ public final class LanguageCodes {
     private static Map<String, String> codeKeyHash = new HashMap<String, String>();
 
     static {
-        codeKeyHash.put(ENGLISH_CODE, ENGLISH_KEY);
-        codeKeyHash.put(JAPANESE_CODE, JAPANESE_KEY);
-        codeKeyHash.put(RUSSIAN_CODE, RUSSIAN_KEY);
-        codeKeyHash.put(GERMAN_CODE, GERMAN_KEY);
         codeKeyHash.put(CATALAN_CODE, CATALAN_KEY);
+        codeKeyHash.put(GERMAN_CODE, GERMAN_KEY);
+        codeKeyHash.put(ENGLISH_CODE, ENGLISH_KEY);
         codeKeyHash.put(SPANISH_CODE, SPANISH_KEY);
-        codeKeyHash.put(POLISH_CODE, POLISH_KEY);
+        codeKeyHash.put(FINNISH_CODE, FINNISH_KEY);
+        codeKeyHash.put(FRENCH_CODE, FRENCH_KEY);
+        codeKeyHash.put(ITALIAN_CODE, ITALIAN_KEY);
+        codeKeyHash.put(JAPANESE_CODE, JAPANESE_KEY);
         codeKeyHash.put(DUTCH_CODE, DUTCH_KEY);
+        codeKeyHash.put(POLISH_CODE, POLISH_KEY);
+        codeKeyHash.put(RUSSIAN_CODE, RUSSIAN_KEY);
+        codeKeyHash.put(SWEDISH_CODE, SWEDISH_KEY);
+        codeKeyHash.put(CHINESE_CODE, CHINESE_KEY);
         codeKeyHash.put(DEFAULT_CODE, DEFAULT_KEY);
         codeKeyHash.put(F_TEXT_CODE, F_TEXT_KEY);
         codeKeyHash.put(F_HTML_CODE, F_HTML_KEY);
