@@ -7,35 +7,42 @@
                Home page: http://www.omegat.org/
                Support center: http://groups.yahoo.com/group/OmegaT/
 
- This program is free software; you can redistribute it and/or modify
+ This file is part of OmegaT.
+
+ OmegaT is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
+ OmegaT is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
 package org.omegat.core.segmentation;
 
 import java.io.Serializable;
+import java.util.ArrayList;
 import java.util.List;
 import java.util.regex.Pattern;
 import java.util.regex.PatternSyntaxException;
 
+import org.omegat.util.StringUtil;
+
 /**
  * A class representing the language rules and their mapping to the segmentation
  * rules for each particular language.
  * 
  * @author Maxym Mykhalchuk
  */
-public class MapRule implements Serializable {
+public class MapRule implements Serializable, Cloneable {
+
+    private static final long serialVersionUID = -5868132953113679291L;
+
     /** creates a new empty MapRule */
     public MapRule() {
     }
@@ -53,9 +60,7 @@ public class MapRule implements Serializable {
     /** Returns Language Name (to display it in a dialog). */
     public String getLanguage() {
         String res = LanguageCodes.getLanguageName(languageCode);
-        if (res == null || res.length() == 0)
-            res = languageCode;
-        return res;
+        return StringUtil.isEmpty(res) ? languageCode : res;
     }
 
     /** Sets Language Name */
@@ -95,6 +100,17 @@ public class MapRule implements Serializable {
         this.pattern = Pattern.compile(pattern, Pattern.CASE_INSENSITIVE);
     }
 
+    public MapRule clone() {
+        MapRule result = new MapRule();
+        result.languageCode = languageCode;
+        result.pattern = pattern;
+        result.rules = new ArrayList<Rule>(rules.size());
+        for (Rule rule : rules) {
+            result.rules.add(rule.clone());
+        }
+        return result;
+    }
+
     /** List of rules (of class {@link Rule}) for the language */
     private List<Rule> rules;
 
@@ -110,14 +126,13 @@ public class MapRule implements Serializable {
 
     /** Indicates whether some other MapRule is "equal to" this one. */
     public boolean equals(Object obj) {
-        if (obj == null)
+        if (obj == null || !(obj instanceof MapRule)) {
             return false;
-        else {
-            MapRule that = (MapRule) obj;
-            return this.getPattern().equals(that.getPattern())
-                    && this.getLanguage().equals(that.getLanguage())
-                    && this.getRules().equals(that.getRules());
         }
+        MapRule that = (MapRule) obj;
+        return this.getPattern().equals(that.getPattern())
+                && this.getLanguage().equals(that.getLanguage())
+                && this.getRules().equals(that.getRules());
     }
 
     /** Returns a hash code value for the object. */
