@@ -8,27 +8,24 @@
                Home page: http://www.omegat.org/
                Support center: http://groups.yahoo.com/group/OmegaT/
 
- This program is free software; you can redistribute it and/or modify
+ This file is part of OmegaT.
+
+ OmegaT is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
+ OmegaT is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
 package org.omegat.core.segmentation;
 
-import gen.core.segmentation.Languagemap;
-import gen.core.segmentation.Languagerule;
-import gen.core.segmentation.Srx;
-
 import java.beans.ExceptionListener;
 import java.beans.XMLDecoder;
 import java.beans.XMLEncoder;
@@ -42,14 +39,16 @@ import java.net.URL;
 import java.util.ArrayList;
 import java.util.List;
 
-import javax.swing.JOptionPane;
 import javax.xml.bind.JAXBContext;
 import javax.xml.bind.JAXBException;
 
 import org.omegat.util.Language;
 import org.omegat.util.Log;
 import org.omegat.util.OStrings;
-import org.omegat.util.StaticUtils;
+
+import gen.core.segmentation.Languagemap;
+import gen.core.segmentation.Languagerule;
+import gen.core.segmentation.Srx;
 
 /**
  * The class with all the segmentation data possible -- rules, languages, etc.
@@ -59,9 +58,9 @@ import org.omegat.util.StaticUtils;
  */
 public class SRX implements Serializable, Cloneable {
 
-    private static SRX srx = null;
-    private static final String CONF_SENTSEG = "segmentation.conf";
-    private static final File configFile = new File(StaticUtils.getConfigDir() + CONF_SENTSEG);
+    private static final long serialVersionUID = 2182125877925944613L;
+
+    public static final String CONF_SENTSEG = "segmentation.conf";
 
     /** Context for JAXB rules processing. */
     protected static final JAXBContext SRX_JAXB_CONTEXT;
@@ -83,31 +82,14 @@ public class SRX implements Serializable, Cloneable {
         }
     }
 
-    /**
-     * SRX factory method.
-     * <p>
-     * For now, just returns the only SRX manager object.
-     */
-    public static SRX getSRX() {
-        if (srx == null) {
-            srx = load();
-        }
-        return srx;
-    }
-
-    /**
-     * Reloads SRX rules from disk.
-     */
-    public static void reload() {
-        srx = load();
-    }
-
     /**
      * Initializes SRX rules to defaults.
      */
-    public static void init() {
-        srx = new SRX();
-        srx.initDefaults();
+    private void init() {
+        this.mappingRules = new ArrayList<MapRule>();
+        this.includeEndingTags=true;
+        this.segmentSubflows=true;
+        initDefaults();
     }
 
     /**
@@ -118,21 +100,32 @@ public class SRX implements Serializable, Cloneable {
     public SRX() {
     }
 
+    public SRX clone() {
+        SRX result = new SRX();
+        result.mappingRules = new ArrayList<MapRule>(mappingRules.size());
+        for (MapRule rule : mappingRules) {
+            result.mappingRules.add(rule.clone());
+        }
+        return result;
+    }
+
     /**
-     * Saves segmentation rules.
+     * Saves segmentation rules into specified file.
      */
-    public void save() {
+    public static void saveTo(SRX srx, File outFile) throws IOException {
+        if (srx == null) {
+            outFile.delete();
+            return;
+        }
         try {
-            setVersion(CURRENT_VERSION);
-            XMLEncoder xmlenc = new XMLEncoder(new FileOutputStream(configFile));
-            xmlenc.writeObject(this);
+            srx.setVersion(CURRENT_VERSION);
+            XMLEncoder xmlenc = new XMLEncoder(new FileOutputStream(outFile));
+            xmlenc.writeObject(srx);
             xmlenc.close();
         } catch (IOException ioe) {
             Log.logErrorRB("CORE_SRX_ERROR_SAVING_SEGMENTATION_CONFIG");
             Log.log(ioe);
-            JOptionPane.showMessageDialog(null,
-                    OStrings.getString("CORE_SRX_ERROR_SAVING_SEGMENTATION_CONFIG") + "\n" + ioe,
-                    OStrings.getString("ERROR_TITLE"), JOptionPane.ERROR_MESSAGE);
+            throw ioe;
         }
     }
 
@@ -144,7 +137,10 @@ public class SRX implements Serializable, Cloneable {
      * is older than that of the current OmegaT, and tries to merge the two sets
      * of rules.
      */
-    private static SRX load() {
+    public static SRX loadSRX(File configFile) {
+        if (!configFile.exists()) {
+            return null;
+        }
         SRX res;
         try {
 
@@ -154,13 +150,13 @@ public class SRX implements Serializable, Cloneable {
             xmldec.close();
 
             if (myel.isExceptionOccured()) {
-                StringBuffer sb = new StringBuffer();
+                StringBuilder sb = new StringBuilder();
                 for (Exception ex : myel.getExceptionsList()) {
                     sb.append("    ");
                     sb.append(ex);
                     sb.append("\n");
                 }
-                Log.logErrorRB("CORE_SRX_EXC_LOADING_SEG_RULES", new Object[] { sb.toString() });
+                Log.logErrorRB("CORE_SRX_EXC_LOADING_SEG_RULES", sb.toString());
                 res = new SRX();
                 res.initDefaults();
                 return res;
@@ -185,8 +181,18 @@ public class SRX implements Serializable, Cloneable {
         }
         return res;
     }
+    
+    /**
+     * Does a config file already exists for the project at the given location?
+     * @param configDir the project directory for storage of settings file
+     */
+    public static boolean projectConfigFileExists(String configDir) {
+        File configFile = new File(configDir + CONF_SENTSEG);
+        return configFile.exists();
+    }
 
-    /** Merges two sets of segmnetation rules together. */
+
+    /** Merges two sets of segmentation rules together. */
     private static SRX merge(SRX current, SRX defaults) {
         current = upgrade(current, defaults);
 
@@ -322,7 +328,7 @@ public class SRX implements Serializable, Cloneable {
     private void initDefaults() {
         try {
             List<MapRule> newMap = new ArrayList<MapRule>();
-            URL rulesUrl = this.getClass().getResource("defaultRules.xml");
+            URL rulesUrl = this.getClass().getResource("defaultRules.srx");
             Srx data = (Srx) SRX_JAXB_CONTEXT.createUnmarshaller().unmarshal(rulesUrl);
 
             for (Languagerule rules : data.getBody().getLanguagerules().getLanguagerule()) {
@@ -351,14 +357,20 @@ public class SRX implements Serializable, Cloneable {
         }
     }
 
+    public static SRX getDefault() {
+        SRX srx = new SRX();
+        srx.init();
+        return srx;
+    }
+
     /**
      * Finds the rules for a certain language.
      * <p>
-     * Usually (if the user didn't skrew up the setup) there're a default
+     * Usually (if the user didn't screw up the setup) there're a default
      * segmentation rules, so it's a good idea to rely on this method always
      * returning at least some rules.
      * <p>
-     * Or in case of a completely screwd setup -- an empty list without any
+     * Or in case of a completely screwed setup -- an empty list without any
      * rules.
      */
     public List<Rule> lookupRulesForLanguage(Language srclang) {
