@@ -7,19 +7,20 @@
                Home page: http://www.omegat.org/
                Support center: http://groups.yahoo.com/group/OmegaT/
 
- This program is free software; you can redistribute it and/or modify
+ This file is part of OmegaT.
+
+ OmegaT is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
+ OmegaT is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
 package org.omegat.core.segmentation.datamodels;
@@ -41,6 +42,7 @@ import org.omegat.util.OStrings;
  * 
  * @author Maxym Mykhalchuk
  */
+@SuppressWarnings("serial")
 public class MappingRulesModel extends AbstractTableModel {
     private SRX srx;
 
@@ -120,18 +122,14 @@ public class MappingRulesModel extends AbstractTableModel {
 
     /** Moves a mapping rule up an order. */
     public void moveRowUp(int row) {
-        MapRule maprulePrev = srx.getMappingRules().get(row - 1);
-        MapRule maprule = srx.getMappingRules().get(row);
-        srx.getMappingRules().remove(row - 1);
+        MapRule maprulePrev = srx.getMappingRules().remove(row - 1);
         srx.getMappingRules().add(row, maprulePrev);
         fireTableRowsUpdated(row - 1, row);
     }
 
     /** Moves a mapping rule down an order. */
     public void moveRowDown(int row) {
-        MapRule mapruleNext = srx.getMappingRules().get(row + 1);
-        MapRule maprule = srx.getMappingRules().get(row);
-        srx.getMappingRules().remove(row + 1);
+        MapRule mapruleNext = srx.getMappingRules().remove(row + 1);
         srx.getMappingRules().add(row, mapruleNext);
         fireTableRowsUpdated(row, row + 1);
     }
