@@ -7,19 +7,20 @@
                Home page: http://www.omegat.org/
                Support center: http://groups.yahoo.com/group/OmegaT/
 
- This program is free software; you can redistribute it and/or modify
+ This file is part of OmegaT.
+
+ OmegaT is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
+ OmegaT is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
 package org.omegat.core.segmentation.datamodels;
@@ -39,6 +40,7 @@ import org.omegat.util.OStrings;
  * 
  * @author Maxym Mykhalchuk
  */
+@SuppressWarnings("serial")
 public class SegmentationRulesModel extends AbstractTableModel {
     private List<Rule> rules;
 
@@ -53,7 +55,7 @@ public class SegmentationRulesModel extends AbstractTableModel {
         Rule rule = rules.get(rowIndex);
         switch (columnIndex) {
         case 0:
-            return new Boolean(rule.isBreakRule());
+            return rule.isBreakRule();
         case 1:
             return rule.getBeforebreak();
         case 2:
@@ -129,18 +131,14 @@ public class SegmentationRulesModel extends AbstractTableModel {
 
     /** Moves a segmentation rule up an order. */
     public void moveRowUp(int row) {
-        Rule rulePrev = rules.get(row - 1);
-        Rule rule = rules.get(row);
-        rules.remove(row - 1);
+        Rule rulePrev = rules.remove(row - 1);
         rules.add(row, rulePrev);
         fireTableRowsUpdated(row - 1, row);
     }
 
     /** Moves a segmentation rule down an order. */
     public void moveRowDown(int row) {
-        Rule ruleNext = rules.get(row + 1);
-        Rule rule = rules.get(row);
-        rules.remove(row + 1);
+        Rule ruleNext = rules.remove(row + 1);
         rules.add(row, ruleNext);
         fireTableRowsUpdated(row, row + 1);
     }
