@@ -4,37 +4,42 @@
           glossaries, and translation leveraging into updated projects.
 
  Copyright (C) 2007 Zoltan Bartko 
-               2009 Didier Briel
+               2009-2011 Didier Briel
                Home page: http://www.omegat.org/
                Support center: http://groups.yahoo.com/group/OmegaT/
 
- This program is free software; you can redistribute it and/or modify
+ This file is part of OmegaT.
+
+ OmegaT is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
+ OmegaT is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
 package org.omegat.core.spellchecker;
 
 import java.io.File;
+import java.io.FileNotFoundException;
 import java.io.IOException;
 import java.net.MalformedURLException;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.List;
 import java.util.Locale;
 import java.util.regex.Matcher;
 
 import org.omegat.util.OConsts;
+import org.omegat.util.OStrings;
 import org.omegat.util.PatternConsts;
+import org.omegat.util.Preferences;
 import org.omegat.util.StaticUtils;
 
 /**
@@ -46,7 +51,7 @@ import org.omegat.util.StaticUtils;
 public class DictionaryManager {
 
     /** the directory string */
-    private File dir;
+    private final File dir;
 
     /**
      * Creates a new instance of DictionaryManager.
@@ -55,8 +60,8 @@ public class DictionaryManager {
      *            : the directory where the spell checking dictionary files
      *            (*.(aff|dic) are available locally
      */
-    public DictionaryManager(String dirName) {
-        dir = new File(dirName);
+    public DictionaryManager(File dir) {
+        this.dir = dir;
     }
 
     /**
@@ -118,29 +123,30 @@ public class DictionaryManager {
 
         // match them
         if (affixFiles != null && dictionaryFiles != null) {
-            for (int i = 0; i < affixFiles.length; i++) {
+            for (String affixFile : affixFiles) {
                 boolean match = false;
 
                 // get the affix file name
-                String affixName = getFileNameOnly(affixFiles[i]);
-                if (affixName == null || affixName.equals(""))
+                String affixName = getFileNameOnly(affixFile);
+                if (affixName == null || affixName.isEmpty()) {
                     continue;
-
+                }
                 // cycle through the dictionary names
-                for (int j = 0; j < dictionaryFiles.length; j++) {
+                for (String dictionaryFile : dictionaryFiles) {
                     // get the dic file name
-                    String dicName = getFileNameOnly(dictionaryFiles[j]);
-                    if (dicName == null || dicName.equals(""))
+                    String dicName = getFileNameOnly(dictionaryFile);
+                    if (dicName == null || dicName.isEmpty()) {
                         continue;
-
+                    }
                     if (affixName.equals(dicName)) {
                         match = true;
                         break;
                     }
                 }
 
-                if (match)
+                if (match) {
                     result.add(affixName);
+                }
             }
         }
 
@@ -155,8 +161,9 @@ public class DictionaryManager {
      * @returns true upon success, otherwise false
      */
     public boolean uninstallDictionary(String lang) {
-        if (lang == null || lang.equals(""))
+        if (lang == null || lang.isEmpty()) {
             return false;
+        }
 
         String base = getDirectory() + File.separator + lang;
 
@@ -167,10 +174,7 @@ public class DictionaryManager {
 
         File dicFile = new File(base + OConsts.SC_DICTIONARY_EXTENSION);
 
-        if (!dicFile.delete())
-            return false;
-
-        return true;
+        return dicFile.delete();
     }
 
     /**
@@ -207,7 +211,8 @@ public class DictionaryManager {
         List<String> result = new ArrayList<String>();
 
         // download the file
-        String htmlfile = StaticUtils.downloadFileToString(OConsts.REMOTE_SC_DICTIONARY_LIST_LOCATION);
+        String htmlfile = StaticUtils.downloadFileToString
+                (Preferences.getPreference(Preferences.SPELLCHECKER_DICTIONARY_URL));
 
         // build a list of available language codes
         Matcher matcher = PatternConsts.DICTIONARY_ZIP.matcher(htmlfile);
@@ -233,29 +238,28 @@ public class DictionaryManager {
      */
     public void installRemoteDictionary(String langCode) throws MalformedURLException, IOException {
         // download the package in question to the disk to a temporary location
-        String from = OConsts.REMOTE_SC_DICTIONARY_LIST_LOCATION + "/" + langCode + ".zip";
+        String from = Preferences.getPreference(Preferences.SPELLCHECKER_DICTIONARY_URL) +
+                      "/" + langCode + ".zip";
 
         // TODO: replace this with something meaningful
         File tempFile = File.createTempFile(langCode, ".zip");
 
-        String to = tempFile.getAbsolutePath();
-
-        StaticUtils.downloadFileToDisk(from, to);
+        StaticUtils.downloadFileToDisk(from, tempFile.getAbsolutePath());
 
         // Dirty hack for the French dictionary. Since it is named
         // fr_FR_1-3-2.zip, we remove the "_1-3-2" portion
         // [ 2138846 ] French dictionary cannot be downloaded and installed
-        int pos;
-        if ((pos = langCode.indexOf("_1-3-2", 0)) != -1) {
+        int pos = langCode.indexOf("_1-3-2", 0);
+        if (pos != -1) {
             langCode = langCode.substring(0, pos);
         }
 
-        List<String> filenames = new ArrayList<String>();
-
-        filenames.add(langCode + OConsts.SC_AFFIX_EXTENSION);
-        filenames.add(langCode + OConsts.SC_DICTIONARY_EXTENSION);
-
-        dir.mkdirs();
-        StaticUtils.extractFileFromJar(to, filenames, dir.getAbsolutePath());
+        try {
+            StaticUtils.extractFileFromJar(tempFile,
+                    Arrays.asList(langCode + OConsts.SC_AFFIX_EXTENSION, langCode + OConsts.SC_DICTIONARY_EXTENSION),
+                    dir.getAbsolutePath());
+        } catch (FileNotFoundException ex) {
+            throw new RuntimeException(OStrings.getString("GUI_SPELLCHECKER_ERROR_ON_INSTALL"), ex);
+        }
     }
 }
