@@ -4,23 +4,25 @@
           glossaries, and translation leveraging into updated projects.
 
  Copyright (C) 2007 Zoltan Bartko, Alex Buloichik
-               2009 Didier Briel               
+               2009 Didier Briel
+               2015 Aaron Madlon-Kay
                Home page: http://www.omegat.org/               
                Support center: http://groups.yahoo.com/group/OmegaT/
 
- This program is free software; you can redistribute it and/or modify
+ This file is part of OmegaT.
+
+ OmegaT is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
+ OmegaT is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
 package org.omegat.core.spellchecker;
@@ -32,9 +34,11 @@ import java.io.FileInputStream;
 import java.io.FileNotFoundException;
 import java.io.FileOutputStream;
 import java.io.IOException;
+import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.io.OutputStreamWriter;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collections;
 import java.util.HashSet;
 import java.util.List;
@@ -48,6 +52,7 @@ import org.omegat.core.events.IProjectEventListener;
 import org.omegat.util.Log;
 import org.omegat.util.OConsts;
 import org.omegat.util.Preferences;
+import org.omegat.util.StaticUtils;
 
 /**
  * Common spell checker interface for use any spellchecker providers.
@@ -55,6 +60,7 @@ import org.omegat.util.Preferences;
  * @author Zoltan Bartko (bartkozoltan at bartkozoltan dot com)
  * @author Alex Buloichik (alex73mail@gmail.com)
  * @author Didier Briel
+ * @author Aaron Madlon-Kay
  */
 public class SpellChecker implements ISpellChecker {
     /** The spell checking provider. */
@@ -93,6 +99,8 @@ public class SpellChecker implements ISpellChecker {
                 case CLOSE:
                     destroy();
                     break;
+                default:
+                    // Nothing
                 }
                 resetCache();
             }
@@ -115,56 +123,82 @@ public class SpellChecker implements ISpellChecker {
         // initialize the spell checker - get the data from the preferences
         String language = Core.getProject().getProjectProperties().getTargetLanguage().getLocaleCode();
 
-        String dictionaryDir = Preferences.getPreference(Preferences.SPELLCHECKER_DICTIONARY_DIRECTORY);
+        String dictionaryDir = Preferences.getPreferenceDefault(
+                Preferences.SPELLCHECKER_DICTIONARY_DIRECTORY,
+                new File(StaticUtils.getConfigDir(), OConsts.SPELLING_DICT_DIR).getPath());
 
-        if (dictionaryDir != null) {
-            String affixName = dictionaryDir + File.separator + language + OConsts.SC_AFFIX_EXTENSION;
+        installBundledDictionary(dictionaryDir, language);
+        
+        File affixName = new File(dictionaryDir, language + OConsts.SC_AFFIX_EXTENSION);
+        File dictionaryName = new File(dictionaryDir, language + OConsts.SC_DICTIONARY_EXTENSION);
 
-            String dictionaryName = dictionaryDir + File.separator + language
-                    + OConsts.SC_DICTIONARY_EXTENSION;
+        // find out the internal project directory
+        String projectDir = Core.getProject().getProjectProperties().getProjectInternal();
 
-            // find out the internal project directory
-            String projectDir = Core.getProject().getProjectProperties().getProjectInternal();
+        // load the ignore list
+        ignoreFileName = projectDir + OConsts.IGNORED_WORD_LIST_FILE_NAME;
 
-            // load the ignore list
-            ignoreFileName = projectDir + OConsts.IGNORED_WORD_LIST_FILE_NAME;
+        // Since we read from disk, we clean the list first
+        ignoreList = new ArrayList<String>();
+        fillWordList(ignoreFileName, ignoreList);
 
-            // Since we read from disk, we clean the list first
-            ignoreList = new ArrayList<String>();
-            fillWordList(ignoreFileName, ignoreList);
+        // now the correct words
+        learnedFileName = projectDir + OConsts.LEARNED_WORD_LIST_FILE_NAME;
 
-            // now the correct words
-            learnedFileName = projectDir + OConsts.LEARNED_WORD_LIST_FILE_NAME;
+        // Since we read from disk, we clean the list first
+        learnedList = new ArrayList<String>();
+        fillWordList(learnedFileName, learnedList);
 
-            // Since we read from disk, we clean the list first
-            learnedList = new ArrayList<String>();
-            fillWordList(learnedFileName, learnedList);
-
-            checker = null;
-            if (new File(dictionaryName).exists()) {
-                try {
-                    checker = new SpellCheckerHunspell(language, dictionaryName, affixName);
-                } catch (Exception ex) {
-                    Log.log("Error loading hunspell: " + ex.getMessage());
-                } catch (Error err) {
-                    Log.log("Error loading hunspell: " + err.getMessage());
-                }
-                if (checker == null) {
-                    try {
-                        checker = new SpellCheckerJMySpell(language, dictionaryName, affixName);
-                    } catch (Exception ex) {
-                        Log.log("Error loading jmyspell: " + ex.getMessage());
-                    }
-                }
+        checker = null;
+        if (dictionaryName.isFile()) {
+            try {
+                checker = new SpellCheckerHunspell(language, dictionaryName.getPath(), affixName.getPath());
+            } catch (Exception ex) {
+                Log.log("Error loading hunspell: " + ex.getMessage());
+            } catch (Error err) {
+                Log.log("Error loading hunspell: " + err.getMessage());
             }
             if (checker == null) {
-                checker = new SpellCheckerDummy();
-                Log.log("No spell checker loaded");
-            }
-            for (String w : learnedList) {
-                checker.learnWord(w);
+                try {
+                    checker = new SpellCheckerJMySpell(language, dictionaryName.getPath(), affixName.getPath());
+                } catch (Exception ex) {
+                    Log.log("Error loading jmyspell: " + ex.getMessage());
+                }
             }
         }
+        if (checker == null) {
+            checker = new SpellCheckerDummy();
+            Log.log("No spell checker loaded");
+        }
+        for (String w : learnedList) {
+            checker.learnWord(w);
+        }
+    }
+    
+    /**
+     * If there is a Hunspell dictionary for the current target language bundled inside
+     * this OmegaT distribution, install it if necessary. 
+     */
+    private void installBundledDictionary(String dictionaryDir, String language) {
+        InputStream bundledDict = getClass().getResourceAsStream(language + ".zip");
+        if (bundledDict == null) {
+            // Relevant dictionary not present.
+            return;
+        }
+        
+        File affix = new File(dictionaryDir, language + OConsts.SC_AFFIX_EXTENSION);
+        File dict = new File(dictionaryDir, language + OConsts.SC_DICTIONARY_EXTENSION);
+        if (affix.isFile() && dict.isFile()) {
+            // Dictionary already installed.
+            return;
+        }
+        
+        try {
+            StaticUtils.extractFileFromJar(bundledDict, Arrays.asList(affix.getName(), dict.getName()), dictionaryDir);
+            bundledDict.close();            
+        } catch (IOException e) {
+            Log.log(e);
+        }
     }
 
     /**
@@ -249,6 +283,13 @@ public class SpellChecker implements ISpellChecker {
      * Otherwise false.
      */
     public boolean isCorrect(String word) {
+        //check if spellchecker is already initialized. If not, skip checking
+        //to prevent nullPointerErrors.
+        if (checker==null) 
+            return true;
+
+        word = normalize(word);
+        
         // check in cache first
         synchronized (this) {
             if (incorrectWordsCache.contains(word)) {
@@ -283,16 +324,17 @@ public class SpellChecker implements ISpellChecker {
      */
     public List<String> suggest(String word) {
         if (isCorrect(word)) {
-            return Collections.EMPTY_LIST;
+            return Collections.emptyList();
         }
 
-        return checker.suggest(word);
+        return checker.suggest(normalize(word));
     }
 
     /**
      * Add a word to the list of ignored words
      */
     public void ignoreWord(String word) {
+        word = normalize(word);
         if (!ignoreList.contains(word)) {
             ignoreList.add(word);
             synchronized (this) {
@@ -306,6 +348,7 @@ public class SpellChecker implements ISpellChecker {
      * Add a word to the list of correct words
      */
     public void learnWord(String word) {
+        word = normalize(word);
         if (!learnedList.contains(word)) {
             learnedList.add(word);
             checker.learnWord(word);
@@ -315,4 +358,13 @@ public class SpellChecker implements ISpellChecker {
             }
         }
     }
+    
+    /**
+     * Normalize the orthography of the word by replacing alternative characters
+     * with "canonical" ones.
+     */
+    private static String normalize(String word) {
+        // U+2019 RIGHT SINGLE QUOTATION MARK to U+0027 APOSTROPHE
+        return word.replace('\u2019', '\'');
+    }
 }
