@@ -5,33 +5,40 @@
 
  Copyright (C) 2000-2006 Keith Godfrey and Maxym Mykhalchuk
                2008 Alex Buloichik
+               2012 Didier Briel
+               2015 Aaron Madlon-Kay
                Home page: http://www.omegat.org/
                Support center: http://groups.yahoo.com/group/OmegaT/
 
- This program is free software; you can redistribute it and/or modify
+ This file is part of OmegaT.
+
+ OmegaT is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
+ OmegaT is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
 package org.omegat.core.threads;
 
+import java.beans.PropertyChangeEvent;
+import java.beans.PropertyChangeListener;
 import java.text.DateFormat;
 import java.util.Date;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 
 import org.omegat.core.Core;
+import org.omegat.core.KnownException;
 import org.omegat.core.data.IProject;
+import org.omegat.util.Preferences;
 
 /**
  * An independent stream to save project, created in order not to freese UI
@@ -39,17 +46,35 @@ import org.omegat.core.data.IProject;
  * 
  * @author Keith Godfrey
  * @author Alex Buloichik (alex73mail@gmail.com)
+ * @author Didier Briel
+ * @author Aaron Madlon-Kay
  */
 public class SaveThread extends Thread implements IAutoSave {
     private static final Logger LOGGER = Logger.getLogger(SaveThread.class.getName());
 
-    private static final int SAVE_DURATION = 10 * 60 * 1000; // 10 minutes;
-
+    /** The length the thread should wait in milliseconds */
+    private int waitDuration;
     private boolean needToSaveNow;
     private boolean enabled;
 
     public SaveThread() {
         setName("Save thread");
+        setWaitDuration(Preferences.getPreferenceDefault(Preferences.AUTO_SAVE_INTERVAL, Preferences.AUTO_SAVE_DEFAULT));
+        Preferences.addPropertyChangeListener(new PropertyChangeListener() {
+            @Override
+            public void propertyChange(PropertyChangeEvent evt) {
+                if (evt.getPropertyName().equals(Preferences.AUTO_SAVE_INTERVAL)) {
+                    setWaitDuration((Integer) evt.getNewValue());
+                    synchronized (SaveThread.this) {
+                        SaveThread.this.notify();
+                    }
+                }
+            }
+        });
+    }
+
+    private void setWaitDuration(int seconds) {
+        waitDuration = seconds * 1000;
     }
 
     public synchronized void disable() {
@@ -64,6 +89,7 @@ public class SaveThread extends Thread implements IAutoSave {
         notify();
     }
 
+    @Override
     public void run() {
         try {
             while (true) {
@@ -72,16 +98,20 @@ public class SaveThread extends Thread implements IAutoSave {
                     // clear this flag also.
                     needToSaveNow = true;
                     // sleep
-                    wait(SAVE_DURATION);
+                    wait(waitDuration);
                 }
                 if (needToSaveNow && enabled) {
                     // Wait finished by time and autosaving enabled.
                     IProject dataEngine = Core.getProject();
                     LOGGER.fine("Start project save from SaveThread");
-                    dataEngine.saveProject();
+                    try {
+                        dataEngine.saveProject();
+                        Core.getMainWindow().showStatusMessageRB("ST_PROJECT_AUTOSAVED",
+                                DateFormat.getTimeInstance(DateFormat.SHORT).format(new Date()));
+                    } catch (KnownException ex) {
+                        Core.getMainWindow().showStatusMessageRB(ex.getMessage(), ex.getParams());
+                    }
                     LOGGER.fine("Finish project save from SaveThread");
-                    Core.getMainWindow().showStatusMessageRB("ST_PROJECT_AUTOSAVED",
-                            DateFormat.getTimeInstance(DateFormat.SHORT).format(new Date()));
                 }
             }
         } catch (InterruptedException ex) {
