@@ -5,24 +5,26 @@
 
  Copyright (C) 2000-2006 Keith Godfrey and Maxym Mykhalchuk
                2006 Martin Wunderlich
-               2009 Alex Buloichik
-               2011 Didier Briel
+               2011 Alex Buloichik, Didier Briel,
+               2012 Guido Leenders
+               2015 Aaron Madlon-Kay
                Home page: http://www.omegat.org/
                Support center: http://groups.yahoo.com/group/OmegaT/
 
- This program is free software; you can redistribute it and/or modify
+ This file is part of OmegaT.
+
+ OmegaT is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
+ OmegaT is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
 package org.omegat.filters2;
@@ -37,14 +39,16 @@ import java.io.IOException;
 import java.io.InputStreamReader;
 import java.io.OutputStreamWriter;
 import java.io.UnsupportedEncodingException;
+import java.nio.charset.Charset;
 import java.util.Map;
 
+import org.omegat.util.EncodingDetector;
 import org.omegat.util.NullBufferedWriter;
 import org.omegat.util.OStrings;
 
 /**
  * The base class for all filters (aka file handlers). Each filter should extend this class or one of its
- * decendants.
+ * descendants.
  * <p>
  * The process how the filter works is the following:
  * <ol>
@@ -60,6 +64,8 @@ import org.omegat.util.OStrings;
  * @author Martin Wunderlich
  * @author Alex Buloichik (alex73mail@gmail.com)
  * @author Didier Briel
+ * @author Guido Leenders
+ * @author Aaron Madlon-Kay
  */
 public abstract class AbstractFilter implements IFilter {
 
@@ -85,11 +91,77 @@ public abstract class AbstractFilter implements IFilter {
     public static final String TFP_TARGET_COUNTRY_CODE = "${targetCountryCode}";
     /** Old spelling of the variable for country "YY" only */
     public static final String TFP_TARGET_COUTRY_CODE = "${targetCoutryCode}";
+    /** System time at generation time in various patterns. */
+    public static final String TFP_TIMESTAMP_LA = "${timestamp-a}";
+    public static final String TFP_TIMESTAMP_LD = "${timestamp-d}";
+    public static final String TFP_TIMESTAMP_LDD = "${timestamp-dd}";
+    public static final String TFP_TIMESTAMP_LH = "${timestamp-h}";
+    public static final String TFP_TIMESTAMP_LHH = "${timestamp-hh}";
+    public static final String TFP_TIMESTAMP_LM = "${timestamp-m}";
+    public static final String TFP_TIMESTAMP_LMM = "${timestamp-mm}";
+    public static final String TFP_TIMESTAMP_LS = "${timestamp-s}";
+    public static final String TFP_TIMESTAMP_LSS = "${timestamp-ss}";
+    public static final String TFP_TIMESTAMP_LYYYY = "${timestamp-yyyy}";
+    public static final String TFP_TIMESTAMP_UD = "${timestamp-D}";
+    public static final String TFP_TIMESTAMP_UEEE = "${timestamp-EEE}";
+    public static final String TFP_TIMESTAMP_UEEEE = "${timestamp-EEEE}";
+    public static final String TFP_TIMESTAMP_UH = "${timestamp-H}";
+    public static final String TFP_TIMESTAMP_UHH = "${timestamp-HH}";
+    public static final String TFP_TIMESTAMP_UM = "${timestamp-M}";
+    public static final String TFP_TIMESTAMP_UMM = "${timestamp-MM}";
+    public static final String TFP_TIMESTAMP_UMMM = "${timestamp-MMM}";
+    /** Workstation properties. */
+    public static final String TFP_SYSTEM_OS_NAME = "${system-os-name}";
+    public static final String TFP_SYSTEM_OS_VERSION = "${system-os-version}";
+    public static final String TFP_SYSTEM_OS_ARCH = "${system-os-arch}";
+    public static final String TFP_SYSTEM_USER_NAME = "${system-user-name}";
+    public static final String TFP_SYSTEM_HOST_NAME = "${system-host-name}";
+    /** File properties. */
+    public static final String TFP_FILE_SOURCE_ENCODING = "${file-source-encoding}";
+    public static final String TFP_FILE_TARGET_ENCODING = "${file-target-encoding}";
+    public static final String TFP_FILE_FILTER_NAME = "${file-filter-name}";
+    /** Microsoft. */
+    public static final String TFP_TARGET_LOCALE_LCID = "${targetLocaleLCID}";
+
+    protected String inEncodingLastParsedFile;
 
     /** All target filename patterns. */
-    public static final String[] TARGET_FILENAME_PATTERNS = new String[] { TFP_FILENAME, TFP_NAMEONLY,
-            TFP_EXTENSION, TFP_TARGET_LOCALE, TFP_TARGET_LANGUAGE, TFP_TARGET_LANG_CODE,
-            TFP_TARGET_COUNTRY_CODE };
+    public static final String[] TARGET_FILENAME_PATTERNS = new String[] { 
+                TFP_FILENAME,
+                TFP_NAMEONLY,
+                TFP_EXTENSION,
+                TFP_TARGET_LOCALE,
+                TFP_TARGET_LOCALE_LCID,
+                TFP_TARGET_LANGUAGE,
+                TFP_TARGET_LANG_CODE,
+                TFP_TARGET_COUNTRY_CODE,
+                TFP_TIMESTAMP_LA,
+                TFP_TIMESTAMP_LD,
+                TFP_TIMESTAMP_LDD,
+                TFP_TIMESTAMP_LH,
+                TFP_TIMESTAMP_LHH,
+                TFP_TIMESTAMP_LM,
+                TFP_TIMESTAMP_LMM,
+                TFP_TIMESTAMP_LS,
+                TFP_TIMESTAMP_LSS,
+                TFP_TIMESTAMP_LYYYY,
+                TFP_TIMESTAMP_UD,
+                TFP_TIMESTAMP_UEEE,
+                TFP_TIMESTAMP_UEEEE,
+                TFP_TIMESTAMP_UH,
+                TFP_TIMESTAMP_UHH,
+                TFP_TIMESTAMP_UM,
+                TFP_TIMESTAMP_UMM,
+                TFP_TIMESTAMP_UMMM,
+                TFP_SYSTEM_OS_NAME,
+                TFP_SYSTEM_OS_VERSION,
+                TFP_SYSTEM_OS_ARCH,
+                TFP_SYSTEM_USER_NAME,
+                TFP_SYSTEM_HOST_NAME,
+                TFP_FILE_SOURCE_ENCODING,
+                TFP_FILE_TARGET_ENCODING,
+                TFP_FILE_FILTER_NAME
+        };
 
     /** Callback for parse. */
     protected IParseCallback entryParseCallback;
@@ -288,27 +360,31 @@ public abstract class AbstractFilter implements IFilter {
     }
 
     /**
-     * Processes a single file given a reader and a writer. Generally this method should read strings from the
-     * input reader and write them to the output reader. In order to let OmegaT know what strings are
-     * translatable and to get their translation, filter should call {@link #processEntry(String)} method.
+     * Processes a single file given a reader and a writer. Generally this
+     * method should read strings from the input reader and write them to the
+     * output reader. In order to let OmegaT know what strings are translatable
+     * and to get their translation, filter should call
+     * {@link #processEntry(String)} method.
      * <p>
-     * Note that outFile is never null, even when the project is loading. (in this case it writes no nowhere,
-     * but anyway you may use it...)
+     * Note that outFile is never null, even when the project is loading. (in
+     * this case it writes no nowhere, but anyway you may use it...)
      * <p>
-     * If you need more control over processed files, override {@link #processFile(File,String,File,String)}
-     * instead.
+     * If you need more control over processed files, override
+     * {@link #processFile(File, File, FilterContext)} instead.
      * 
      * @param inFile
-     *            Reader of the source file. It's the result of calling {@link #createReader(File,String)}.
+     *            Reader of the source file. It's the result of calling
+     *            {@link #createReader(File,String)}.
      * @param outFile
-     *            Writer of the target file on compilation (the result of calling
-     *            {@link #createWriter(File, String)}), or a fictive writer to /dev/null.
+     *            Writer of the target file on compilation (the result of
+     *            calling {@link #createWriter(File, String)}), or a fictive
+     *            writer to /dev/null.
      * @throws TranslationException
      *             Should be thrown when processed file has any format defects.
      * @throws IOException
      *             In case of any I/O error.
      */
-    protected abstract void processFile(BufferedReader inFile, BufferedWriter outFile) throws IOException,
+    protected abstract void processFile(BufferedReader inFile, BufferedWriter outFile, FilterContext fc) throws IOException,
             TranslationException;
 
     /**
@@ -342,23 +418,24 @@ public abstract class AbstractFilter implements IFilter {
      *             In case of any I/O error.
      * @throws TranslationException
      *             Should be thrown when processed file has any format defects.
-     * 
-     * @author Martin Wunderlich
      */
     protected void processFile(File inFile, File outFile, FilterContext fc) throws IOException,
             TranslationException {
-        BufferedReader reader = createReader(inFile, fc.getInEncoding());
+        String encoding = getInputEncoding(fc, inFile);
+        BufferedReader reader = createReader(inFile, encoding);
+        inEncodingLastParsedFile = encoding == null ? Charset.defaultCharset().name() : encoding;
         try {
             BufferedWriter writer;
 
             if (outFile != null) {
-                writer = createWriter(outFile, fc.getOutEncoding());
+                String outEncoding = getOutputEncoding(fc);
+                writer = createWriter(outFile, outEncoding);
             } else {
                 writer = new NullBufferedWriter();
             }
 
             try {
-                processFile(reader, writer);
+                processFile(reader, writer, fc);
             } finally {
                 writer.close();
             }
@@ -366,9 +443,48 @@ public abstract class AbstractFilter implements IFilter {
             reader.close();
         }
     }
+    
+    /**
+     * Get the input encoding. If it's not set in the FilterContext (setting is "&lt;auto&gt;")
+     * and the filter allows ({@link #isSourceEncodingVariable()}), try to detect it. The result may be null.
+     * @param fc
+     * @param inFile
+     * @return
+     * @throws IOException
+     */
+    protected String getInputEncoding(FilterContext fc, File inFile) throws IOException {
+        String encoding = fc.getInEncoding();
+        if (encoding == null && isSourceEncodingVariable()) {
+            encoding = EncodingDetector.detectEncoding(inFile);
+        }
+        return encoding;
+    }
+    
+    /**
+     * Get the output encoding. If it's not set in the FilterContext (setting is "&lt;auto&gt;")
+     * and the filter allows ({@link #isTargetEncodingVariable()}):
+     * <ul><li>Reuse the input encoding if it's Unicode
+     * <li>If the input was not Unicode, fall back to UTF-8.
+     * </ul>
+     * The result may be null.
+     * @param fc
+     * @return
+     */
+    protected String getOutputEncoding(FilterContext fc) {
+        String encoding = fc.getOutEncoding();
+        if (encoding == null && isTargetEncodingVariable()) {
+            // Use input encoding if it's Unicode; otherwise default to UTF-8
+            if (inEncodingLastParsedFile != null && inEncodingLastParsedFile.toLowerCase().startsWith("utf-")) {
+                encoding = inEncodingLastParsedFile;
+            } else {
+                encoding = "UTF-8";
+            }
+        }
+        return encoding;
+    }
 
-    public void parseFile(File inFile, Map<String, String> config, FilterContext fc, IParseCallback callback)
-            throws Exception {
+    public final void parseFile(File inFile, Map<String, String> config, FilterContext fc,
+            IParseCallback callback) throws Exception {
         entryParseCallback = callback;
         entryTranslateCallback = null;
         entryAlignCallback = null;
@@ -376,13 +492,17 @@ public abstract class AbstractFilter implements IFilter {
 
         try {
             processFile(inFile, null, fc);
+            if (requirePrevNextFields()) {
+                // parsing - need to link prev/next
+                entryParseCallback.linkPrevNextSegments();
+            }
         } finally {
             entryParseCallback = null;
             processOptions = null;
         }
     }
 
-    public void alignFile(File inFile, File outFile, Map<String, String> config, FilterContext fc,
+    public final void alignFile(File inFile, File outFile, Map<String, String> config, FilterContext fc,
             IAlignCallback callback) throws Exception {
         entryParseCallback = null;
         entryTranslateCallback = null;
@@ -393,7 +513,7 @@ public abstract class AbstractFilter implements IFilter {
         BufferedReader readerOut = createReader(outFile, fc.getOutEncoding());
 
         try {
-            alignFile(readerIn, readerOut);
+            alignFile(readerIn, readerOut, fc);
         } finally {
             readerIn.close();
             readerOut.close();
@@ -408,18 +528,32 @@ public abstract class AbstractFilter implements IFilter {
      * @param translatedFile
      *            translated file
      */
-    protected void alignFile(BufferedReader sourceFile, BufferedReader translatedFile) throws Exception {
+    protected void alignFile(BufferedReader sourceFile, BufferedReader translatedFile, FilterContext fc) throws Exception {
     }
 
-    public void translateFile(File inFile, File outFile, Map<String, String> config, FilterContext fc,
-            ITranslateCallback callback) throws Exception {
+    /**
+     * Method can be overrided for return true. It means what two-pass parsing and translating will be
+     * processed and prev/next segments will be linked.
+     */
+    protected boolean requirePrevNextFields() {
+        return false;
+    }
+
+    public final void translateFile(File inFile, File outFile, Map<String, String> config, FilterContext fc,
+       ITranslateCallback callback) throws Exception {
         entryParseCallback = null;
         entryTranslateCallback = callback;
         entryAlignCallback = null;
         processOptions = config;
 
         try {
+            entryTranslateCallback.setPass(1);
             processFile(inFile, outFile, fc);
+            if (requirePrevNextFields()) {
+                entryTranslateCallback.linkPrevNextSegments();
+                entryTranslateCallback.setPass(2);
+                processFile(inFile, outFile, fc);
+            }
         } finally {
             entryTranslateCallback = null;
             processOptions = null;
@@ -438,11 +572,26 @@ public abstract class AbstractFilter implements IFilter {
      * @return Translation of the source string. If there's no translation, returns the source string itself.
      */
     protected final String processEntry(String entry) {
+        return processEntry(entry, null);
+    }
+    /**
+     * Call this method to:
+     * <ul>
+     * <li>Instruct OmegaT what source strings are translatable.
+     * <li>Get the translation of each source string.
+     * </ul>
+     * 
+     * @param entry
+     *            Translatable source string
+     * @param comment comment on the source string in the source file (if available)
+     * @return Translation of the source string. If there's no translation, returns the source string itself.
+     */
+    protected final String processEntry(String entry, String comment) {
         if (entryParseCallback != null) {
-            entryParseCallback.addEntry(null, entry, null, false, null, this);
+            entryParseCallback.addEntry(null, entry, null, false, comment, null, this, null);
             return entry;
         } else {
-            String translation = entryTranslateCallback.getTranslation(null, entry);
+            String translation = entryTranslateCallback.getTranslation(null, entry, null);
             return translation != null ? translation : entry;
         }
     }
@@ -457,4 +606,9 @@ public abstract class AbstractFilter implements IFilter {
         this.entryParseCallback = parseCallback;
         this.entryTranslateCallback = translateCallback;
     }
+
+    public String getInEncodingLastParsedFile() {
+        return inEncodingLastParsedFile;
+    }
+
 }
