@@ -4,22 +4,24 @@
           glossaries, and translation leveraging into updated projects.
 
  Copyright (C) 2000-2006 Keith Godfrey and Maxym Mykhalchuk
+               2015 Aaron Madlon-Kay
                Home page: http://www.omegat.org/
                Support center: http://groups.yahoo.com/group/OmegaT/
 
- This program is free software; you can redistribute it and/or modify
+ This file is part of OmegaT.
+
+ OmegaT is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
+ OmegaT is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
 package org.omegat.filters2;
@@ -31,7 +33,9 @@ package org.omegat.filters2;
  * Note that a filter may also throw IOException in case of any I/O errors.
  * 
  * @author Maxym Mykhalchuk
+ * @author Aaron Madlon-Kay
  */
+@SuppressWarnings("serial")
 public class TranslationException extends Exception {
     /**
      * Constructs an instance of <code>TranslationException</code> with the
@@ -56,4 +60,15 @@ public class TranslationException extends Exception {
     public TranslationException(String msg, Throwable cause) {
         super(msg, cause);
     }
+
+    /**
+     * Constructs an instance of <code>TranslationException</code> with the
+     * specified cause.
+     * 
+     * @param cause
+     *            cause the cause
+     */
+    public TranslationException(Throwable cause) {
+        super(cause);
+    }
 }
