@@ -4,23 +4,24 @@
           glossaries, and translation leveraging into updated projects.
 
  Copyright (C) 2000-2006 Keith Godfrey and Maxym Mykhalchuk
- Copyright (C) 2006 by Didier Briel
+ Copyright (C) 2006-2011 by Didier Briel
                Home page: http://www.omegat.org/
                Support center: http://groups.yahoo.com/group/OmegaT/
 
- This program is free software; you can redistribute it and/or modify
+ This file is part of OmegaT.
+
+ OmegaT is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
+ OmegaT is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
 package org.omegat.filters2.hhc;
@@ -49,6 +50,7 @@ class HHCFilterVisitor extends FilterVisitor {
     /** Do we collect the translatable text now. */
     boolean text = false;
 
+    @Override
     public void visitTag(Tag tag) {
         if (isParagraphTag(tag) && text)
             endup();
@@ -64,4 +66,21 @@ class HHCFilterVisitor extends FilterVisitor {
         return tagname.equals("HTML") || tagname.equals("HEAD") || tagname.equals("BODY");
     }
 
+    /**
+     * If the attribute of the tag is not empty, it translates it as a separate
+     * segment.
+     *
+     * @param tag
+     *            the tag object
+     * @param key
+     *            the name of the attribute
+     */
+    @Override
+    protected void maybeTranslateAttribute(Tag tag, String key) {
+        String attr = tag.getAttribute(key);
+        if (attr != null) {
+            String trans = filter.privateProcessEntry(entitiesToChars(attr), null);
+            tag.setAttribute(key, charsToEntities(trans));
+        }
+    }
 }
