@@ -5,22 +5,24 @@
 
  Copyright (C) 2000-2006 Keith Godfrey and Maxym Mykhalchuk
                2007-2008 Martin Fleurke
+               2012 Didier Briel
                Home page: http://www.omegat.org/
                Support center: http://groups.yahoo.com/group/OmegaT/
 
- This program is free software; you can redistribute it and/or modify
+ This file is part of OmegaT.
+
+ OmegaT is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
+ OmegaT is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
 package org.omegat.filters2.html2;
@@ -43,16 +45,18 @@ import org.omegat.filters2.Instance;
 import org.omegat.filters2.TranslationException;
 import org.omegat.util.Log;
 import org.omegat.util.OStrings;
+import org.omegat.util.StringUtil;
 
 /**
  * A filter to translate HTML and XHTML files.
  * <p>
  * Some useful discussion why HTML filter should behave like it does, happened
- * on a <a href="http://sourceforge.net/support/tracker.php?aid=1364265">bug
- * report</a> devoted to compressing space.
+ * on a <a href="https://sourceforge.net/p/omegat/bugs/108/">bug report</a>
+ * devoted to compressing space.
  * 
  * @author Maxym Mykhalchuk
  * @author Martin Fleurke
+ * @author Didier Briel
  */
 public class HTMLFilter2 extends AbstractFilter {
     /** Creates a new instance of HTMLFilter2 */
@@ -77,12 +81,24 @@ public class HTMLFilter2 extends AbstractFilter {
      */
     private HashMap<String, String> skipMetaAttributes;
 
+    /**
+     * A map of attribute-name and attribute value pairs that, if exist in a
+     * tag, indicate that this tag should not be translated
+     */
+    private HashMap<String, String> ignoreTagsAttributes;
+
+
+    @Override
+    protected boolean requirePrevNextFields() {
+        return true;
+    }
     /**
      * Customized version of creating input reader for HTML files, aware of
      * encoding by using <code>EncodingAwareReader</code> class.
      * 
      * @see HTMLReader
      */
+    @Override
     public BufferedReader createReader(File infile, String encoding) throws UnsupportedEncodingException,
             IOException {
         HTMLReader hreader = new HTMLReader(infile.getAbsolutePath(), encoding);
@@ -96,6 +112,7 @@ public class HTMLFilter2 extends AbstractFilter {
      * 
      * @see HTMLWriter
      */
+    @Override
     public BufferedWriter createWriter(File outfile, String encoding) throws UnsupportedEncodingException,
             IOException {
         HTMLWriter hwriter;
@@ -109,11 +126,12 @@ public class HTMLFilter2 extends AbstractFilter {
         return new BufferedWriter(hwriter);
     }
 
-    public void processFile(BufferedReader infile, BufferedWriter outfile) throws IOException,
+    @Override
+    public void processFile(BufferedReader infile, BufferedWriter outfile, org.omegat.filters2.FilterContext fc) throws IOException,
             TranslationException {
-        StringBuffer all = null;
+        StringBuilder all = null;
         try {
-            all = new StringBuffer();
+            all = new StringBuilder();
             char cbuf[] = new char[1000];
             int len = -1;
             while ((len = infile.read(cbuf)) > 0)
@@ -129,7 +147,7 @@ public class HTMLFilter2 extends AbstractFilter {
 
         // Prepare matcher
         String skipRegExp = options.getSkipRegExp();
-        if (skipRegExp != null && skipRegExp.length() > 0) {
+        if (!StringUtil.isEmpty(skipRegExp)) {
             try {
                 this.skipRegExpPattern = Pattern.compile(skipRegExp, Pattern.CASE_INSENSITIVE);
             } catch (PatternSyntaxException e) {
@@ -146,6 +164,15 @@ public class HTMLFilter2 extends AbstractFilter {
             skipMetaAttributes.put(keyvalue, "");
         }
 
+        // Prepare set of attributes that indicate not to translate a tag
+        String ignoreTagString = options.getIgnoreTags();
+        ignoreTagsAttributes = new HashMap<String, String>();
+        String[] ignoreTagsAttributesStringarray = ignoreTagString.split(",");
+        for (int i = 0; i < ignoreTagsAttributesStringarray.length; i++) {
+            String keyvalue = ignoreTagsAttributesStringarray[i].trim().toUpperCase();
+            ignoreTagsAttributes.put(keyvalue, "");
+        }
+
         Parser parser = new Parser();
         try {
             parser.setInputHTML(all.toString());
@@ -160,17 +187,17 @@ public class HTMLFilter2 extends AbstractFilter {
     // ////////////////////////////////////////////////////////////////////////
 
     /** Package-internal processEntry to give it to FilterVisitor */
-    String privateProcessEntry(String entry) {
+    public String privateProcessEntry(String entry, String comment) {
         if (skipRegExpPattern != null) {
             if (skipRegExpPattern.matcher(entry).matches()) {
                 // System.out.println("Skipping \""+entry+"\"");
                 return entry;
             } else {
                 // System.out.println("Using: \""+entry+"\"");
-                return super.processEntry(entry);
+                return super.processEntry(entry, comment);
             }
         }
-        return super.processEntry(entry);
+        return super.processEntry(entry, comment);
     }
 
     // ////////////////////////////////////////////////////////////////////////
@@ -201,6 +228,7 @@ public class HTMLFilter2 extends AbstractFilter {
      * declaration, it will be used disregarding any value you set in this
      * dialog.
      */
+    @Override
     public String getHint() {
         return OStrings.getString("HTML_NOTE");
     }
@@ -210,6 +238,7 @@ public class HTMLFilter2 extends AbstractFilter {
      * 
      * @return True, because (X)HTML filter has options.
      */
+    @Override
     public boolean hasOptions() {
         return true;
     }
@@ -222,6 +251,7 @@ public class HTMLFilter2 extends AbstractFilter {
      * @return Updated filter options if user confirmed the changes, and current
      *         options otherwise.
      */
+    @Override
     public Map<String, String> changeOptions(Dialog parent, Map<String, String> config) {
         try {
             EditOptionsDialog dialog = new EditOptionsDialog(parent, config);
@@ -250,4 +280,14 @@ public class HTMLFilter2 extends AbstractFilter {
     public boolean checkDoSkipMetaTag(String key, String value) {
         return skipMetaAttributes.containsKey(key.toUpperCase() + "=" + value.toUpperCase());
     }
+
+    public boolean checkIgnoreTags(String key, String value) {
+        return ignoreTagsAttributes.containsKey(key.toUpperCase() + "=" + value.toUpperCase());
+    }
+
+    @Override
+    public String getInEncodingLastParsedFile() {
+        return sourceEncoding;
+    }
+
 }
