@@ -8,22 +8,25 @@
                2008 Martin Fleurke
                2009 Arno Peters
                2011 Didier Briel
+               2014 Adiel Mittmann
+               2017 Didier Briel
                Home page: http://www.omegat.org/
                Support center: http://groups.yahoo.com/group/OmegaT/
 
- This program is free software; you can redistribute it and/or modify
+ This file is part of OmegaT.
+
+ OmegaT is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
+ OmegaT is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
 package org.omegat.filters2.latex;
@@ -40,6 +43,8 @@ import java.util.regex.Pattern;
 
 import org.omegat.filters2.AbstractFilter;
 import org.omegat.filters2.Instance;
+import org.omegat.util.LinebreakPreservingReader;
+import org.omegat.util.Log;
 import org.omegat.util.OStrings;
 
 /**
@@ -51,6 +56,7 @@ import org.omegat.util.OStrings;
  * @author Martin Fleurke
  * @author Arno Peters
  * @author Didier Briel
+ * @author Adiel Mittmann
  */
 public class LatexFilter extends AbstractFilter {
 
@@ -69,8 +75,14 @@ public class LatexFilter extends AbstractFilter {
     public boolean isTargetEncodingVariable() {
         return true;
     }
+    
+    @Override
+    protected boolean requirePrevNextFields() {
+        return true;
+    }
 
-    public void processFile(BufferedReader in, BufferedWriter out) throws IOException {
+    @Override
+    public void processFile(BufferedReader in, BufferedWriter out, org.omegat.filters2.FilterContext fc) throws IOException {
         // BOM (byte order mark) bugfix
         in.mark(1);
         int ch = in.read();
@@ -126,9 +138,11 @@ public class LatexFilter extends AbstractFilter {
      * @throws java.io.IOException
      */
     private void processLatexFile(BufferedReader in, Writer out) throws IOException {
-        StringBuffer par = new StringBuffer();
+        LinebreakPreservingReader lpin = new LinebreakPreservingReader(in);
+
+        StringBuilder par = new StringBuilder();
         String s;
-        StringBuffer comment = new StringBuffer();
+        StringBuilder comment = new StringBuilder();
 
         LinkedList<String> commands = new LinkedList<String>();
 
@@ -137,18 +151,25 @@ public class LatexFilter extends AbstractFilter {
          * blanks
          */
         String state;
-        while ((s = in.readLine()) != null) {
-            String[] c = s.split("");
+        while ((s = lpin.readLine()) != null) {
+            // String[] c = s.split(""); In Java 8, that line gave a first empty element, so it was replaced with the 
+            // following lines, and idx below was started at 0 instead of 1
+            String[] c;
+            if (!s.isEmpty()){
+              c = s.split("(?!^)");
+            } else {
+              c = new String[0];
+            }
             state = "N";
 
-            int idx = 1;
+            int idx = 0;
             while (idx < c.length) {
                 String cidx = c[idx];
                 int cat = findStringCategory(cidx);
 
                 if (cat == 0) {
                     /* parse control sequence */
-                    StringBuffer cmd = new StringBuffer();
+                    StringBuilder cmd = new StringBuilder();
                     cmd.append(cidx);
                     idx++;
                     while (idx < c.length) {
@@ -202,18 +223,20 @@ public class LatexFilter extends AbstractFilter {
 
             /* at the end of the line */
             if (state.equals("N")) {
+                String endOfLine = lpin.getLinebreak();
                 /* \par */
                 if (par.length() > 0) {
                     out.write(processParagraph(commands, par.toString()));
-                    out.write("\n\n");
+                    out.write(endOfLine);
+                    out.write(endOfLine);
                     par.setLength(0);
                 }
                 // System.out.println(commands);
                 commands.clear();
                 if (comment.length() > 0) { // If there is a comment, write it
-                     out.write(comment.toString());
-                     out.write("\n");
-                     comment.setLength(0);
+                    out.write(comment.toString());
+                    out.write(endOfLine);
+                    comment.setLength(0);
                 }
             } else if (state.equals("M")) {
                 par.append(" ");
@@ -228,7 +251,7 @@ public class LatexFilter extends AbstractFilter {
 
     private String substituteUnicode(String par) {
         par = par.replaceAll("\\\\\\\\", "<br0>");
-        par = par.replaceAll("\\{?\\\\ss\\}?", "ß");
+        par = par.replaceAll("\\{?\\\\ss\\}?", "\u00df");
         par = par.replaceAll("\\{?\\\\glqq\\}?(\\{\\})?", "\u301f");
         par = par.replaceAll("\\{?\\\\grqq\\}?(\\{\\})?", "\u301d");
         par = par.replaceAll("\\{?\\\\glq\\}?(\\{\\})?", "\u201a");
@@ -400,25 +423,33 @@ public class LatexFilter extends AbstractFilter {
         for (Iterator<String> it = commands.iterator(); it.hasNext();) {
             String command = it.next();
 
-            if (command.equals("\\\\") || command.equals("\\{") || command.equals("\\["))
+            if (command.equals("\\\\") || command.equals("\\{") || command.equals("\\[") || command.equals("\\|")) {                
                 // continue;
                 command = "\\" + command;
+            }
 
             StringBuffer sb = new StringBuffer();
             String find = "\\" + command;
 
-            Pattern p = Pattern.compile(find);
-            Matcher m = p.matcher(par);
-            while (m.find()) {
-                String replace = "<u" + String.valueOf(counter) + ">";
-                String[] subst = { reHarden(m.group(0)), reHarden(replace) };
-                substituted.addFirst(subst);
-                m.appendReplacement(sb, replace);
-                counter++;
+            try {
+                Pattern p = Pattern.compile(find);
+                Matcher m = p.matcher(par);
+                while (m.find()) {
+                    String replace = "<u" + String.valueOf(counter) + ">";
+                    String[] subst = { reHarden(m.group(0)), reHarden(replace) };
+                    substituted.addFirst(subst);
+                    m.appendReplacement(sb, replace);
+                    counter++;
+                }
+                m.appendTail(sb);
+
+                par = sb.toString();
+             } catch (java.util.regex.PatternSyntaxException e) {
+               //TODO: understand the exceptions
+               Log.log("LaTeX PatternSyntaxException: " + e.getMessage());
+               Log.log(command);
             }
-            m.appendTail(sb);
 
-            par = sb.toString();
         }
         return par;
     }
