@@ -11,45 +11,48 @@
                2009 Didier Briel, Arno Peters, Alex Buloichik
                2010 Alex Buloichik
                2011 Alex Buloichik, Didier Briel
+               2012 Guido Leenders, Thomas Cordonnier
+               2013 Alex Buloichik
 
                Home page: http://www.omegat.org/
                Support center: http://groups.yahoo.com/group/OmegaT/
 
- This program is free software; you can redistribute it and/or modify
+ This file is part of OmegaT.
+
+ OmegaT is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
+ OmegaT is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
 package org.omegat.filters2.master;
 
-import gen.core.filters.Files;
-import gen.core.filters.Filter;
-import gen.core.filters.Filter.Option;
-import gen.core.filters.Filters;
-
 import java.io.File;
 import java.io.IOException;
 import java.nio.charset.Charset;
+import java.text.SimpleDateFormat;
 import java.util.ArrayList;
+import java.util.Calendar;
 import java.util.List;
 import java.util.Map;
 import java.util.TreeMap;
+import java.util.regex.Pattern;
 
 import javax.swing.JOptionPane;
 import javax.xml.bind.JAXBContext;
 import javax.xml.bind.Marshaller;
 import javax.xml.bind.Unmarshaller;
 
+import org.apache.commons.io.FileUtils;
+import org.omegat.core.Core;
 import org.omegat.filters2.AbstractFilter;
 import org.omegat.filters2.FilterContext;
 import org.omegat.filters2.IAlignCallback;
@@ -58,11 +61,15 @@ import org.omegat.filters2.IParseCallback;
 import org.omegat.filters2.ITranslateCallback;
 import org.omegat.filters2.Instance;
 import org.omegat.filters2.TranslationException;
-import org.omegat.util.LFileCopy;
 import org.omegat.util.Language;
 import org.omegat.util.Log;
 import org.omegat.util.OStrings;
-import org.omegat.util.StaticUtils;
+import org.omegat.util.StringUtil;
+
+import gen.core.filters.Files;
+import gen.core.filters.Filter;
+import gen.core.filters.Filter.Option;
+import gen.core.filters.Filters;
 
 /**
  * A master class that registers and handles all the filters. Singleton - there can be only one instance of
@@ -75,6 +82,8 @@ import org.omegat.util.StaticUtils;
  * @author Martin Fleurke
  * @author Arno Peters
  * @author Alex Buloichik (alex73mail@gmail.com)
+ * @author Guido Leenders
+ * @author Thomas Cordonnier
  */
 public class FilterMaster {
     /** name of the filter configuration file */
@@ -85,7 +94,7 @@ public class FilterMaster {
     /**
      * There was no version of file filters support (1.4.5 Beta 1 -- 1.6.0 RC12).
      */
-    public static String INITIAL_VERSION = new String();
+    public static String INITIAL_VERSION = "";
     /** File filters support of 1.6.0 RC12a: now upgrading the configuration. */
     public static String OT160RC12a_VERSION = "1.6 RC12a";
     public static String OT160FINAL_VERSION = "1.6.0";
@@ -94,21 +103,17 @@ public class FilterMaster {
     /** Currently file filters support version. */
     public static String CURRENT_VERSION = "2.0";
 
-    /** FilterMaster instance. */
-    private static FilterMaster master = null;
-
-    /** Config file. */
-    private File configFile = new File(StaticUtils.getConfigDir() + FILE_FILTERS);
-
     /** Filters config stored in XML file. */
-    private Filters config;
+    private final Filters config;
 
     /** Classes of all filters. */
-    private List<Class<IFilter>> filtersClasses;
+    private static List<Class<?>> filtersClasses;
 
     static {
         try {
             CONFIG_CTX = JAXBContext.newInstance(Filters.class);
+            filtersClasses = new ArrayList<Class<?>>();
+            filtersClasses.addAll((List<Class<?>>) PluginUtils.getFilterClasses());
         } catch (Exception ex) {
             throw new ExceptionInInitializerError(ex);
         }
@@ -117,24 +122,22 @@ public class FilterMaster {
     /**
      * Create a new FilterMaster.
      */
-    private FilterMaster() {
-        filtersClasses = new ArrayList<Class<IFilter>>();
-        filtersClasses.addAll((List)PluginUtils.getFilterClasses());
+    public FilterMaster(Filters config) {
+        this.config = config;
+    }
 
-        loadConfig();
-
-        addNewFiltersToConfig(config);
-
-        saveConfig();
+    public Filters getConfig() {
+        return config;
     }
 
     /**
      * Adds new filters(which was not exist in config yet) into config.
      */
-    private void addNewFiltersToConfig(final Filters conf) {
-        for (Class<IFilter> fclass : filtersClasses) {
+    private static boolean addNewFiltersToConfig(final Filters conf) {
+        boolean result = false;
+        for (Class<?> fclass : filtersClasses) {
             boolean found = false;
-            for (Filter fc : conf.getFilter()) {
+            for (Filter fc : conf.getFilters()) {
                 if (fclass.getName().equals(fc.getClassName())) {
                     // filter already exist in config
                     found = true;
@@ -143,18 +146,11 @@ public class FilterMaster {
             }
             if (!found) {
                 // filter not found in config
-                conf.getFilter().add(getDefaultSettingsFromFilter(fclass.getName()));
+                conf.getFilters().add(getDefaultSettingsFromFilter(fclass.getName()));
+                result = true;
             }
         }
-    }
-
-    /**
-     * Returns the only instance of this class.
-     */
-    public static FilterMaster getInstance() {
-        if (master == null)
-            master = new FilterMaster();
-        return master;
+        return result;
     }
 
     /**
@@ -164,11 +160,11 @@ public class FilterMaster {
      *            filter's class name
      * @return filter instance
      */
-    public IFilter getFilterInstance(final String classname) {
-        for (Class<IFilter> f : filtersClasses) {
+    public static IFilter getFilterInstance(final String classname) {
+        for (Class<?> f : filtersClasses) {
             if (f.getName().equals(classname)) {
                 try {
-                    return f.newInstance();
+                    return (IFilter) f.newInstance();
                 } catch (Exception ex) {
                     Log.log(ex);
                 }
@@ -183,27 +179,29 @@ public class FilterMaster {
      * @param filename
      *            The name of the source file to load.
      * @return Whether the file was handled by one of OmegaT filters.
-     * @see #translateFile(String, String, String)
+     * @see #translateFile(String, String, String, FilterContext,
+     *      ITranslateCallback)
      */
-    public boolean loadFile(String filename, FilterContext fc, IParseCallback parseCallback)
+    public IFilter loadFile(String filename, FilterContext fc, IParseCallback parseCallback)
             throws IOException, TranslationException {
+        IFilter filterObject = null;
         try {
-            LookupInformation lookup = lookupFilter(filename, fc);
-            if (lookup == null)
-                return false;
-
+            LookupInformation lookup = lookupFilter(new File(filename), fc);
+            if (lookup == null) {
+                return null;
+            }
             File inFile = new File(filename);
             fc.setInEncoding(lookup.outFilesInfo.getSourceEncoding());
             fc.setOutEncoding(lookup.outFilesInfo.getTargetEncoding());
 
-            IFilter filterObject = lookup.filterObject;
+            filterObject = lookup.filterObject;
 
             filterObject.parseFile(inFile, lookup.config, fc, parseCallback);
         } catch (Exception ioe) {
             ioe.printStackTrace();
             throw new IOException(filename + "\n" + ioe);
         }
-        return true;
+        return filterObject;
     }
 
     /**
@@ -227,25 +225,21 @@ public class FilterMaster {
      */
     public void translateFile(String sourcedir, String filename, String targetdir, FilterContext fc,
             ITranslateCallback translateCallback) throws IOException, TranslationException {
-        LookupInformation lookup = lookupFilter(sourcedir + File.separator + filename, fc);
+        LookupInformation lookup = lookupFilter(new File(sourcedir, filename), fc);
         if (lookup == null) {
             // The file is not supported by any of the filters.
             // Copying it
-            LFileCopy.copy(sourcedir + File.separator + filename, targetdir + File.separator + filename);
+            FileUtils.copyFile(new File(sourcedir, filename), new File(targetdir, filename));
             return;
         }
 
-        File inFile = new File(sourcedir + File.separator + filename);
+        File inFile = new File(sourcedir, filename).getCanonicalFile();
+        File outFile = new File(targetdir, getTargetForSource(filename, lookup, fc.getTargetLang())).getCanonicalFile();
 
-        String name = inFile.getName();
-        String path = filename.substring(0, filename.length() - name.length());
-
-        File outFile = new File(targetdir
-                + File.separator
-                + path
-                + File.separator
-                + constructTargetFilename(lookup.outFilesInfo.getSourceFilenameMask(), name,
-                        lookup.outFilesInfo.getTargetFilenamePattern(), fc.getTargetLang()));
+        if (inFile.equals(outFile)) {
+            throw new TranslationException(
+                    StringUtil.format(OStrings.getString("FILTERMASTER_ERROR_SRC_TRG_SAME_FILE"), inFile.getPath()));
+        }
 
         fc.setInEncoding(lookup.outFilesInfo.getSourceEncoding());
         fc.setOutEncoding(lookup.outFilesInfo.getTargetEncoding());
@@ -261,24 +255,15 @@ public class FilterMaster {
     public void alignFile(String sourceDir, String fileName, String targetdir, FilterContext fc,
             IAlignCallback alignCallback) throws Exception {
 
-        LookupInformation lookup = lookupFilter(sourceDir + File.separator + fileName, fc);
+        LookupInformation lookup = lookupFilter(new File(sourceDir, fileName), fc);
         if (lookup == null) {
             // The file is not supported by any of the filters.
             // Skip it
             return;
         }
 
-        File inFile = new File(sourceDir + File.separator + fileName);
-
-        String name = inFile.getName();
-        String path = fileName.substring(0, fileName.length() - name.length());
-
-        File outFile = new File(targetdir
-                + File.separator
-                + path
-                + File.separator
-                + constructTargetFilename(lookup.outFilesInfo.getSourceFilenameMask(), name,
-                        lookup.outFilesInfo.getTargetFilenamePattern(), fc.getTargetLang()));
+        File inFile = new File(sourceDir, fileName);
+        File outFile = new File(targetdir, getTargetForSource(fileName, lookup, fc.getTargetLang()));
 
         if (!outFile.exists()) {
             // out file not exist - skip
@@ -296,7 +281,7 @@ public class FilterMaster {
         }
     }
 
-    class LookupInformation {
+    static class LookupInformation {
         public final Files outFilesInfo;
         public final IFilter filterObject;
         public final Map<String, String> config;
@@ -318,43 +303,32 @@ public class FilterMaster {
      * <li>Creates a reader (use <code>OneFilter.getReader()</code> to get it)
      * <li>Checks whether the filter supports the file.
      * </ul>
-     * It <b>does not</b> check whether the filter supports the inFile, i.e. it doesn't call
-     * <code>isFileSupported</code>
      * 
-     * 
-     * @param filename
-     *            The source filename.
-     * @return The filter to handle the inFile.
+     * @param inFile The full path to the source file
+     * @return The corresponding LookupInformation
      */
-    private LookupInformation lookupFilter(String filename, FilterContext fc) throws TranslationException,
+    private LookupInformation lookupFilter(File inFile, FilterContext fc) throws TranslationException,
             IOException {
-        File inFile = new File(filename);
-        String name = inFile.getName();
-        String path = inFile.getParent();
-        if (path == null)
-            path = "";
-
-        for (Filter f : config.getFilter()) {
+        for (Filter f : config.getFilters()) {
             if (!f.isEnabled()) {
                 continue;
             }
             for (Files ff : f.getFiles()) {
-                if (matchesMask(name, ff.getSourceFilenameMask())) {
-                    IFilter filterObject;
-                    filterObject = getFilterInstance(f.getClassName());
-
-                    if (filterObject != null) {
-                        fc.setInEncoding(ff.getSourceEncoding());
-                        fc.setOutEncoding(ff.getTargetEncoding());
-                        // only for exist filters
-                        Map<String, String> config = forFilter(f.getOption());
-                        if (!filterObject.isFileSupported(inFile, config, fc)) {
-                            break;
-                        }
-
-                        return new LookupInformation(filterObject, ff, config);
-                    }
+                if (!matchesMask(inFile.getName(), ff.getSourceFilenameMask())) {
+                    continue;
                 }
+                IFilter filterObject = getFilterInstance(f.getClassName());
+                if (filterObject == null) {
+                    continue;
+                }
+                fc.setInEncoding(ff.getSourceEncoding());
+                fc.setOutEncoding(ff.getTargetEncoding());
+                // only for exist filters
+                Map<String, String> config = forFilter(f.getOption());
+                if (!filterObject.isFileSupported(inFile, config, fc)) {
+                    break;
+                }
+                return new LookupInformation(filterObject, ff, config);
             }
         }
         return null;
@@ -370,9 +344,10 @@ public class FilterMaster {
      */
     public static List<String> getSupportedEncodings() {
         if (supportedEncodings == null) {
-            supportedEncodings = new ArrayList<String>();
-            supportedEncodings.add(AbstractFilter.ENCODING_AUTO_HUMAN);
-            supportedEncodings.addAll(Charset.availableCharsets().keySet());
+            ArrayList<String> list = new ArrayList<String>();
+            list.add(AbstractFilter.ENCODING_AUTO_HUMAN);
+            list.addAll(Charset.availableCharsets().keySet());
+            supportedEncodings = list;
         }
         return supportedEncodings;
     }
@@ -390,7 +365,7 @@ public class FilterMaster {
      * <li>Saves the configuration
      * </ul>
      */
-    public Filters createDefaultFiltersConfig() {
+    public static Filters createDefaultFiltersConfig() {
         Filters c = new Filters();
         addNewFiltersToConfig(c);
         return c;
@@ -400,25 +375,37 @@ public class FilterMaster {
      * Loads information about the filters from an XML file. If there's an error loading a file, it calls
      * <code>setupDefaultFilters</code>.
      */
-    public void loadConfig() {
+    public static Filters loadConfig(String configDir) {
+        File configFile = new File(configDir, FILE_FILTERS);
         if (!configFile.exists()) {
-            config = new Filters();
-            return;
+            return null;
         }
+        Filters result;
         try {
             Unmarshaller unm = CONFIG_CTX.createUnmarshaller();
-            config = (Filters) unm.unmarshal(configFile);
+            result = (Filters) unm.unmarshal(configFile);
         } catch (Exception e) {
             Log.logErrorRB("FILTERMASTER_ERROR_LOADING_FILTERS_CONFIG");
             Log.log(e);
-            config = new Filters();
+            result = new Filters();
         }
+
+        if (addNewFiltersToConfig(result)) {
+            saveConfig(result, configDir);
+        }
+
+        return result;
     }
 
     /**
      * Saves information about the filters to an XML file.
      */
-    public void saveConfig() {
+    public static void saveConfig(Filters config, String configDir) {
+        File configFile = new File(configDir, FILE_FILTERS);
+        if (config == null) {
+            configFile.delete();
+            return;
+        }
         try {
             Marshaller m = CONFIG_CTX.createMarshaller();
             m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, true);
@@ -426,7 +413,7 @@ public class FilterMaster {
         } catch (Exception e) {
             Log.logErrorRB("FILTERMASTER_ERROR_SAVING_FILTERS_CONFIG");
             Log.log(e);
-            JOptionPane.showMessageDialog(null,
+            JOptionPane.showMessageDialog(Core.getMainWindow().getApplicationFrame(),
                     OStrings.getString("FILTERMASTER_ERROR_SAVING_FILTERS_CONFIG") + "\n" + e,
                     OStrings.getString("ERROR_TITLE"), JOptionPane.ERROR_MESSAGE);
         }
@@ -452,6 +439,45 @@ public class FilterMaster {
         return filename.matches("(?iu)" + mask);
     }
 
+    /**
+     * Return current system time in the specified date format.
+     *
+     * @param dateFormat
+     *            Date format for java.text.SimpleDateFormat.
+     */
+    public static String now(String dateFormat) {
+        Calendar cal = Calendar.getInstance();
+        SimpleDateFormat sdf = new SimpleDateFormat(dateFormat);
+        return sdf.format(cal.getTime());
+    }
+
+    /**
+     * Calculate the target path corresponding to the given source file.
+     * @param sourceDir Path to the project's <code>source</code> dir
+     * @param srcRelPath Relative path under <code>sourceDir</code> of the source file
+     * @param fc Filter context
+     * @return The relative path under <code>target</code> of the corresponding target file
+     * @throws IOException
+     * @throws TranslationException
+     */
+    public String getTargetForSource(String sourceDir, String srcRelPath, FilterContext fc) throws IOException, TranslationException {
+        File srcFile = new File(sourceDir, srcRelPath);
+        if (!srcFile.isFile()) {
+            throw new IllegalArgumentException("The sourceDir and srcRelPath arguments must together point to an existing file.");
+        }
+        LookupInformation lookup = lookupFilter(srcFile, fc);
+        return getTargetForSource(srcRelPath, lookup, fc.getTargetLang());
+    }
+    
+    private static String getTargetForSource(String srcRelPath, LookupInformation lookup, Language targetLang) {
+        File srcRelFile = new File(srcRelPath);
+        return new File(srcRelFile.getParent(),
+                constructTargetFilename(lookup.outFilesInfo.getSourceFilenameMask(), srcRelFile.getName(),
+                        lookup.outFilesInfo.getTargetFilenamePattern(), targetLang,
+                        lookup.outFilesInfo.getSourceEncoding(), lookup.outFilesInfo.getTargetEncoding(),
+                        lookup.filterObject.getFileFormatName())).getPath();
+    }
+    
     /**
      * Construct a target filename according to pattern from a file's name. Filename should be "name.ext",
      * without path.
@@ -464,10 +490,13 @@ public class FilterMaster {
      * <li><code>${filename}</code> - full filename of the input file, both name and extension (default)
      * <li><code>${nameOnly}</code> - only the name of the input file without extension part
      * <li><code>${extension}</code> - the extension of the input file
+     * <li><code>${nameOnly-1}</code> - only the name of the input file with first extension
+     * <li><code>${extension-1}</code> - the extensions, without the first one
      * <li><code>${targetLocale}</code> - target locale code (of a form "xx_YY")
      * <li><code>${targetLanguage}</code> - the target language and country code together (of a form "XX-YY")
      * <li><code>${targetLanguageCode}</code> - the target language only ("XX")
      * <li><code>${targetCountryCode}</code> - the target country only ("YY")
+     * <li><code>${1}, ${2}, ...</code> - variables captured by jokers (* or ?)
      * </ul>
      * <p>
      * Most file filters will use default "<code>${filename}</code>, that leads to the name of translated file
@@ -483,6 +512,8 @@ public class FilterMaster {
      * <ul>
      * <li><code>${nameOnly}</code> will be equal to "thisisfile"
      * <li>and <code>${extension}</code> - "ext1.ext2"
+     * <li><code>${nameOnly-1}</code> will be equal to "thisisfile.ext1"
+     * <li>and <code>${extension-1}</code> - "ext2"
      * </ul>
      * 
      * @param filename
@@ -491,8 +522,8 @@ public class FilterMaster {
      *            Pattern, according to which we change the filename
      * @return The changed filename
      */
-    private String constructTargetFilename(String sourceMask, String filename, String pattern,
-            Language targetLang) {
+    private static String constructTargetFilename(String sourceMask, String filename, String pattern,
+            Language targetLang, String sourceEncoding, String targetEncoding, String filterFormatName) {
         int lastStarPos = sourceMask.lastIndexOf('*');
         int dot = 0;
         if (lastStarPos >= 0) {
@@ -526,29 +557,96 @@ public class FilterMaster {
         res = res.replace(AbstractFilter.TFP_TARGET_COUNTRY_CODE, targetLang.getCountryCode());
         // Replace also old variable spelling
         res = res.replace(AbstractFilter.TFP_TARGET_COUTRY_CODE, targetLang.getCountryCode());
+        res = res.replace(AbstractFilter.TFP_TARGET_LOCALE_LCID, targetLang.getLocaleLCID());
+        //
+        // System generation time
+        //
+        res = res.replace(AbstractFilter.TFP_TIMESTAMP_LA, now("a"));
+        res = res.replace(AbstractFilter.TFP_TIMESTAMP_LD, now("d"));
+        res = res.replace(AbstractFilter.TFP_TIMESTAMP_LDD, now("dd"));
+        res = res.replace(AbstractFilter.TFP_TIMESTAMP_LH, now("h"));
+        res = res.replace(AbstractFilter.TFP_TIMESTAMP_LHH, now("hh"));
+        res = res.replace(AbstractFilter.TFP_TIMESTAMP_LM, now("m"));
+        res = res.replace(AbstractFilter.TFP_TIMESTAMP_LMM, now("mm"));
+        res = res.replace(AbstractFilter.TFP_TIMESTAMP_LS, now("s"));
+        res = res.replace(AbstractFilter.TFP_TIMESTAMP_LSS, now("ss"));
+        res = res.replace(AbstractFilter.TFP_TIMESTAMP_LYYYY, now("yyyy"));
+        res = res.replace(AbstractFilter.TFP_TIMESTAMP_UD, now("D"));
+        res = res.replace(AbstractFilter.TFP_TIMESTAMP_UEEE, now("EEE"));
+        res = res.replace(AbstractFilter.TFP_TIMESTAMP_UEEEE, now("EEEE"));
+        res = res.replace(AbstractFilter.TFP_TIMESTAMP_UH, now("H"));
+        res = res.replace(AbstractFilter.TFP_TIMESTAMP_UHH, now("HH"));
+        res = res.replace(AbstractFilter.TFP_TIMESTAMP_UM, now("M"));
+        res = res.replace(AbstractFilter.TFP_TIMESTAMP_UMM, now("MM"));
+        res = res.replace(AbstractFilter.TFP_TIMESTAMP_UMMM, now("MMM"));
+        //
+        // Workstation properties
+        //
+        res = res.replace(AbstractFilter.TFP_SYSTEM_OS_NAME, System.getProperty("os.name"));
+        res = res.replace(AbstractFilter.TFP_SYSTEM_OS_VERSION, System.getProperty("os.arch"));
+        res = res.replace(AbstractFilter.TFP_SYSTEM_OS_ARCH, System.getProperty("os.version"));
+        res = res.replace(AbstractFilter.TFP_SYSTEM_USER_NAME, System.getProperty("user.name"));
+        String hostName = null;
+        try {
+            hostName = java.net.InetAddress.getLocalHost().getHostName();
+        } catch (java.net.UnknownHostException uhe) {
+            hostName = "";
+        }
+        res = res.replace(AbstractFilter.TFP_SYSTEM_HOST_NAME, hostName);
+        //
+        // File properties.
+        //
+        String sourceEncodingText = "auto";
+        if (sourceEncoding != null) {
+            sourceEncodingText = sourceEncoding;
+        }
+        res = res.replace(AbstractFilter.TFP_FILE_SOURCE_ENCODING, sourceEncodingText);
+        //
+        String targetEncodingText = "auto";
+        if (targetEncoding != null) {
+            targetEncodingText = targetEncoding;
+        }
+        res = res.replace(AbstractFilter.TFP_FILE_TARGET_ENCODING, targetEncodingText);
+        //
+        res = res.replace(AbstractFilter.TFP_FILE_FILTER_NAME, filterFormatName);
+        //
+        
+        String sourceMaskPattern = sourceMask.replaceAll("\\?","(.)").replaceAll("\\*","(.*?)");
+        java.util.regex.Matcher sourceMatcher = Pattern.compile(sourceMaskPattern).matcher(filename);
+        if (sourceMatcher.find()) {
+            for (int i = 1; i <= sourceMatcher.groupCount(); i++) {
+                res = res.replaceAll("\\$\\{" + i + "\\}", sourceMatcher.group(i));
+            }
+        }
+        
+        String[] splitName = filename.split("\\.");
+        StringBuilder nameOnlyBuf = new StringBuilder (splitName[0]);
+        StringBuilder extensionBuf = new StringBuilder (splitName[splitName.length - 1]);
+        for (int i = 0; i < splitName.length; i++) {
+            res = res.replaceAll ("\\$\\{nameOnly-" + i + "\\}", nameOnlyBuf.toString());
+            res = res.replaceAll ("\\$\\{extension-" + i + "\\}", extensionBuf.toString());
+            if (i + 1 < splitName.length) {
+                nameOnlyBuf.append (".").append(splitName[i + 1]);
+                extensionBuf.insert(0, splitName[splitName.length - i - 2] + '.');
+            }
+        }
 
         return res;
     }
 
-    /**
-     * Set new config. Used by filter's editor.
-     * 
-     * @param config
-     *            new config
-     */
-    public void setConfig(final Filters config) {
-        this.config = config;
-    }
-
     /**
      * Clone config for editing
      * 
      * @return new config instance
      */
-    public Filters cloneConfig() {
+    public static Filters cloneConfig(Filters orig) {
         Filters c = new Filters();
-        for (Filter f : config.getFilter()) {
-            c.getFilter().add(cloneFilter(f));
+        c.setRemoveTags(orig.isRemoveTags());
+        c.setRemoveSpacesNonseg(orig.isRemoveSpacesNonseg());
+        c.setPreserveSpaces(orig.isPreserveSpaces());
+        c.setIgnoreFileContext(orig.isIgnoreFileContext());
+        for (Filter f : orig.getFilters()) {
+            c.getFilters().add(cloneFilter(f));
         }
         return c;
     }
@@ -560,7 +658,7 @@ public class FilterMaster {
      *            one filter's config
      * @return new config instance
      */
-    public Filter cloneFilter(Filter filter) {
+    public static Filter cloneFilter(Filter filter) {
         Filter f = new Filter();
         f.setClassName(filter.getClassName());
         f.setEnabled(filter.isEnabled());
@@ -583,7 +681,7 @@ public class FilterMaster {
      *            new filter's instance config
      * @return new config instance
      */
-    public Files cloneFiles(Files files) {
+    private static Files cloneFiles(Files files) {
         Files ff = new Files();
         ff.setSourceEncoding(files.getSourceEncoding());
         ff.setSourceFilenameMask(files.getSourceFilenameMask());
@@ -599,7 +697,7 @@ public class FilterMaster {
      *            filter's classname
      * @return default filter's config
      */
-    public Filter getDefaultSettingsFromFilter(final String filterClassname) {
+    public static Filter getDefaultSettingsFromFilter(final String filterClassname) {
         IFilter f = getFilterInstance(filterClassname);
         Filter fc = new Filter();
         fc.setClassName(f.getClass().getName());
