@@ -4,22 +4,24 @@
           glossaries, and translation leveraging into updated projects.
 
  Copyright (C) 2000-2006 Keith Godfrey and Maxym Mykhalchuk
+               2014 Didier Briel
                Home page: http://www.omegat.org/
                Support center: http://groups.yahoo.com/group/OmegaT/
 
- This program is free software; you can redistribute it and/or modify
+ This file is part of OmegaT.
+
+ OmegaT is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
+ OmegaT is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
 package org.omegat.filters2.text;
@@ -31,10 +33,13 @@ import java.io.IOException;
 import java.io.Writer;
 import java.util.Map;
 
+import org.omegat.core.Core;
 import org.omegat.filters2.AbstractFilter;
+import org.omegat.filters2.FilterContext;
 import org.omegat.filters2.Instance;
 import org.omegat.util.LinebreakPreservingReader;
 import org.omegat.util.Log;
+import org.omegat.util.OConsts;
 import org.omegat.util.OStrings;
 
 /**
@@ -42,6 +47,7 @@ import org.omegat.util.OStrings;
  * 
  * @author Keith Godfrey
  * @author Maxym Mykhalchuk
+ * @author Didier Briel
  */
 public class TextFilter extends AbstractFilter {
 
@@ -50,7 +56,7 @@ public class TextFilter extends AbstractFilter {
      */
     public static final String SEGMENT_BREAKS = "BREAKS";
     /**
-     * Defult. Text filter should segmentOn text into paragraphs on empty lines.
+     * Default. Text filter should segmentOn text into paragraphs on empty lines.
      */
     public static final String SEGMENT_EMPTYLINES = "EMPTYLINES";
     /**
@@ -59,44 +65,94 @@ public class TextFilter extends AbstractFilter {
     public static final String SEGMENT_NEVER = "NEVER";
 
     public static final String OPTION_SEGMENT_ON = "segmentOn";
+    
+    /**
+     * Length at which a line break should occur in target documents
+     */
+    public static final String OPTION_LINE_LENGTH = "lineLength";
+    
+    /**
+     * Maximum line length in target documents
+     */
+    public static final String OPTION_MAX_LINE_LENGTH = "maxLineLength";
 
+    
+    /**
+     * Register plugin into OmegaT.
+     */
+    public static void loadPlugins() {
+        Core.registerFilterClass(TextFilter.class);
+    }
+
+    public static void unloadPlugins() {
+    }
+
+    @Override
     public String getFileFormatName() {
         return OStrings.getString("TEXTFILTER_FILTER_NAME");
     }
 
+    @Override
     public Instance[] getDefaultInstances() {
-        return new Instance[] { new Instance("*.txt"), new Instance("*.txt1", "ISO-8859-1", "ISO-8859-1"),
-                new Instance("*.txt2", "ISO-8859-2", "ISO-8859-2"), new Instance("*.utf8", "UTF-8", "UTF-8") };
+        return new Instance[] { new Instance("*.txt"), new Instance("*.txt1", OConsts.ISO88591, OConsts.ISO88591),
+                new Instance("*.txt2", OConsts.ISO88592, OConsts.ISO88592), new Instance("*.utf8", OConsts.UTF8, OConsts.UTF8) };
     }
 
+    @Override
     public boolean isSourceEncodingVariable() {
         return true;
     }
 
+    @Override
     public boolean isTargetEncodingVariable() {
         return true;
     }
+    
+    @Override
+    protected boolean requirePrevNextFields() {
+        return true;
+    }
 
-    public void processFile(BufferedReader in, BufferedWriter out) throws IOException {
+    @Override
+    public void processFile(BufferedReader in, BufferedWriter out, FilterContext fc) throws IOException {
         // BOM (byte order mark) bugfix
         in.mark(1);
         int ch = in.read();
         if (ch != 0xFEFF)
             in.reset();
 
+        int lineLength, maxLineLength;
+        try {
+            lineLength = Integer.parseInt(processOptions.get(TextFilter.OPTION_LINE_LENGTH));
+        } catch (Exception ex) {
+            lineLength = 0;
+        }
+        try {
+            maxLineLength = Integer.parseInt(processOptions.get(TextFilter.OPTION_MAX_LINE_LENGTH));
+        } catch (Exception ex) {
+            maxLineLength = 0;
+        }
+        Writer output;
+        if (lineLength != 0 && maxLineLength != 0) {
+            output = new LineLengthLimitWriter(out, lineLength, maxLineLength, fc.getTargetTokenizer());
+        } else {
+            output = out;
+        }
+
         String segmentOn = processOptions.get(TextFilter.OPTION_SEGMENT_ON);
         if (SEGMENT_BREAKS.equals(segmentOn)) {
-            processSegLineBreaks(in, out);
+            processSegLineBreaks(in, output);
         } else if (SEGMENT_NEVER.equals(segmentOn)) {
-            processNonSeg(in, out);
+            processNonSeg(in, output);
         } else {
-            processSegEmptyLines(in, out);
+            processSegEmptyLines(in, output);
         }
+        output.close();
     }
 
     /** Process the file without segmenting it. */
     private void processNonSeg(BufferedReader in, Writer out) throws IOException {
-        StringBuffer segment = new StringBuffer();
+        StringBuilder segment = new StringBuilder();
         char[] buf = new char[4096];
         int len;
         while ((len = in.read(buf)) >= 0)
@@ -110,7 +166,7 @@ public class TextFilter extends AbstractFilter {
         String nontrans = "";
         String s;
         while ((s = lpin.readLine()) != null) {
-            if (s.trim().length() == 0) {
+            if (s.trim().isEmpty()) {
                 nontrans += s + lpin.getLinebreak();
                 continue;
             }
@@ -124,19 +180,21 @@ public class TextFilter extends AbstractFilter {
 
             nontrans += lpin.getLinebreak();
         }
+        lpin.close();
 
-        if (nontrans.length() != 0)
+        if (!nontrans.isEmpty()) {
             out.write(nontrans);
+        }
     }
 
     /** Processes the file segmenting on line breaks. */
     private void processSegEmptyLines(BufferedReader in, Writer out) throws IOException {
         LinebreakPreservingReader lpin = new LinebreakPreservingReader(in);
-        StringBuffer nontrans = new StringBuffer();
-        StringBuffer trans = new StringBuffer();
+        StringBuilder nontrans = new StringBuilder();
+        StringBuilder trans = new StringBuilder();
         String s;
         while ((s = lpin.readLine()) != null) {
-            if (s.length() == 0) {
+            if (s.isEmpty()) {
                 out.write(nontrans.toString());
                 nontrans.setLength(0);
 
@@ -144,7 +202,7 @@ public class TextFilter extends AbstractFilter {
                 trans.setLength(0);
                 nontrans.append(lpin.getLinebreak());
             } else {
-                if (s.trim().length() == 0 && trans.length() == 0) {
+                if (s.trim().isEmpty() && trans.length() == 0) {
                     nontrans.append(s);
                     nontrans.append(lpin.getLinebreak());
                 } else {
@@ -153,11 +211,13 @@ public class TextFilter extends AbstractFilter {
                 }
             }
         }
-
-        if (nontrans.length() >= 0)
+        lpin.close();
+        if (nontrans.length() >= 0) {
             out.write(nontrans.toString());
-        if (trans.length() >= 0)
+        }
+        if (trans.length() >= 0) {
             out.write(processEntry(trans.toString()));
+        }
     }
 
     @Override
@@ -181,6 +241,7 @@ public class TextFilter extends AbstractFilter {
      * 
      * @return True, because Text filter has options.
      */
+    @Override
     public boolean hasOptions() {
         return true;
     }
