@@ -6,22 +6,25 @@
  Copyright (C) 2000-2006 Keith Godfrey and Maxym Mykhalchuk
                2007 Didier Briel
                2010 Antonio Vilei
+               2012 Didier Briel
+               2013 Alex Buloichik, Didier Briel
                Home page: http://www.omegat.org/
                Support center: http://groups.yahoo.com/group/OmegaT/
 
- This program is free software; you can redistribute it and/or modify
+ This file is part of OmegaT.
+
+ OmegaT is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
+ OmegaT is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
 package org.omegat.filters3;
@@ -29,8 +32,15 @@ package org.omegat.filters3;
 import java.util.ArrayList;
 import java.util.List;
 
+import org.omegat.core.Core;
+import org.omegat.core.data.ProtectedPart;
 import org.omegat.filters2.TranslationException;
-import org.omegat.util.PatternConsts;
+import org.omegat.filters3.xml.Handler;
+import org.omegat.filters3.xml.XMLContentBasedTag;
+import org.omegat.filters3.xml.XMLDialect;
+import org.omegat.filters3.xml.XMLText;
+import org.omegat.util.StringUtil;
+import org.omegat.util.TagUtil;
 
 /**
  * Translatable entry. Holds a list of source tags and text, translated text and
@@ -38,15 +48,23 @@ import org.omegat.util.PatternConsts;
  * 
  * @author Maxym Mykhalchuk
  * @author Didier Briel
+ * @author Alex Buloichik (alex73mail@gmail.com)
  */
 public class Entry {
+    final XMLDialect xmlDialect;
+    final Handler handler;
+
+    public Entry(XMLDialect xmlDialect, Handler handler) {
+        this.xmlDialect = xmlDialect;
+        this.handler = handler;
+    }
+
     /**
      * Cleans up this entry.
      */
     public void clear() {
         tagsDetected = false;
         elements.clear();
-        translation = new String();
         translatedEntry = null;
         textInstance = null;
     }
@@ -92,7 +110,7 @@ public class Entry {
     /**
      * Detects the first and the last translatable tags and assigns all tags in
      * translatable region the shortcuts. Basically calls {@link #detectTags()}
-     * and {@link #enumerateTags()} if tags were not detected, i.e.
+     * and {@link #enumerateTags(int, int)} if tags were not detected, i.e.
      * {@link #tagsDetected} is false. in the paragraph "last translatable".
      */
     private void detectAndEnumerateTags() {
@@ -102,10 +120,14 @@ public class Entry {
             }
             detectTags();
             tagsDetected = true;
-            enumerateTags();
+            enumerateTags(getFirstGood(), getLastGood());
         }
     }
 
+    public void resetTagDetected() {
+        tagsDetected = false;
+    }
+
     /**
      * Aggregate tags. The current OpenXML filter finds too many tags, usually
      * causing what users call the "tag soup". Tags aggregation can help
@@ -121,7 +143,7 @@ public class Entry {
             if (elem instanceof Tag) {
                 // Add this tag to the aggregated tag
                 if (aggregated == null) {
-                    aggregated = new AggregatedTag("tag", null, Tag.TYPE_ALONE, new Attributes());
+                    aggregated = new AggregatedTag("tag", null, Tag.Type.ALONE, new Attributes());
                 }
                 aggregated.add((Tag) elem);
             } else {
@@ -162,6 +184,15 @@ public class Entry {
             Element elem = get(i);
             if ((elem instanceof Text) && ((Text) elem).isMeaningful()) {
                 textStart = i;
+                break;
+            }
+            if (elem instanceof XMLContentBasedTag) {
+                textStart = i;
+            }
+        }
+        for (int i = 0; i < size(); i++) {
+            Element elem = get(i);
+            if ((elem instanceof Text) && ((Text) elem).isMeaningful()) {
                 textInstance = (Text) elem;
                 break;
             }
@@ -183,6 +214,50 @@ public class Entry {
             }
         }
 
+        // if content-based tag is inside text, then expand text into paired content-based tag
+        for (int i = textStart; i <= textEnd; i++) {
+            Element elem = get(i);
+            if (elem instanceof XMLContentBasedTag) {
+                XMLContentBasedTag tag = (XMLContentBasedTag) elem;
+                if (tag.getTag().equals("bpt") || tag.getTag().equals("ept")) {
+                    // find id of paired tag
+                    String id = StringUtil.nvl(tag.getAttribute("rid"), tag.getAttribute("id"),
+                            tag.getAttribute("i"));
+                    if (id == null) {
+                        continue;
+                    }
+                    // find paired tag before
+                    for (int j = textStart - 1; j >= 0; j--) {
+                        if (get(j) instanceof XMLContentBasedTag) {
+                            XMLContentBasedTag tag2 = (XMLContentBasedTag) get(j);
+                            if (tag2.getTag().equals("bpt") || tag2.getTag().equals("ept")) {
+                                // find id of paired tag
+                                String id2 = StringUtil.nvl(tag2.getAttribute("rid"),
+                                        tag2.getAttribute("id"), tag2.getAttribute("i"));
+                                if (id.equals(id2)) {
+                                    textStart = j;
+                                }
+                            }
+                        }
+                    }
+                    // find paired tag after
+                    for (int j = textEnd + 1; j < size(); j++) {
+                        if (get(j) instanceof XMLContentBasedTag) {
+                            XMLContentBasedTag tag2 = (XMLContentBasedTag) get(j);
+                            if (tag2.getTag().equals("bpt") || tag2.getTag().equals("ept")) {
+                                // find id of paired tag
+                                String id2 = StringUtil.nvl(tag2.getAttribute("rid"),
+                                        tag2.getAttribute("id"), tag2.getAttribute("i"));
+                                if (id.equals(id2)) {
+                                    textEnd = j;
+                                }
+                            }
+                        }
+                    }
+                }
+            }
+        }
+
         // //////////////////////////////////////////////////////////////////////
         // "first good"
         // detecting the first starting tag that has its ending in the paragraph
@@ -193,7 +268,7 @@ public class Entry {
                 continue;
 
             Tag good = (Tag) goodElem;
-            if (Tag.TYPE_BEGIN != good.getType())
+            if (Tag.Type.BEGIN != good.getType())
                 continue;
 
             // trying to test
@@ -203,9 +278,9 @@ public class Entry {
                 if (candElement instanceof Tag) {
                     Tag cand = (Tag) candElement;
                     if (cand.getTag().equals(good.getTag())) {
-                        if (Tag.TYPE_BEGIN == cand.getType())
+                        if (Tag.Type.BEGIN == cand.getType())
                             recursion++;
-                        else if (Tag.TYPE_END == cand.getType()) {
+                        else if (Tag.Type.END == cand.getType()) {
                             recursion--;
                             if (recursion == 0) {
                                 if (i > textStart)
@@ -233,7 +308,7 @@ public class Entry {
                 continue;
 
             Tag good = (Tag) goodElem;
-            if (Tag.TYPE_END != good.getType())
+            if (Tag.Type.END != good.getType())
                 continue;
 
             // trying to test
@@ -243,9 +318,9 @@ public class Entry {
                 if (candElement instanceof Tag) {
                     Tag cand = (Tag) candElement;
                     if (cand.getTag().equals(good.getTag())) {
-                        if (Tag.TYPE_END == cand.getType())
+                        if (Tag.Type.END == cand.getType())
                             recursion++;
-                        else if (Tag.TYPE_BEGIN == cand.getType()) {
+                        else if (Tag.Type.BEGIN == cand.getType()) {
                             recursion--;
                             if (recursion == 0) {
                                 if (i < textEnd)
@@ -262,32 +337,87 @@ public class Entry {
         }
         if (!found)
             lastGood = textEnd;
+
+        boolean removeTags;
+        if (handler.getContext().isRemoveAllTags()) { // If Remove Tags is on,
+            removeTags = true;                        // Remove leading and trailing tags must be on
+        } else {
+            removeTags = Core.getFilterMaster().getConfig().isRemoveTags();
+        }
+        // tags was already removed - restore they if need
+        if (!removeTags) {
+            for (int i = firstGood - 1; i >= 0; i--) {
+                Element elem = get(i);
+                if (elem instanceof Tag) {
+                    if (handler.isParagraphTag((Tag) elem)) {
+                        break;
+                    }
+                    firstGood = i;
+                }
+            }
+            for (int i = lastGood + 1; i < size(); i++) {
+                Element elem = get(i);
+                if (elem instanceof Tag) {
+                    if (handler.isParagraphTag((Tag) elem)) {
+                        break;
+                    }
+                    lastGood = i;
+                }
+            }
+        }
+
+        boolean removeSpacesAround = Core.getFilterMaster().getConfig().isRemoveSpacesNonseg();
+        // spaces was already removed - restore they if need
+        if (!removeSpacesAround) {
+            for (int i = firstGood - 1; i >= 0; i--) {
+                Element elem = get(i);
+                if (elem instanceof Tag) {
+                    if (handler.isParagraphTag((Tag) elem)) {
+                        break;
+                    }
+                }
+                if ((elem instanceof Text) && !((Text) elem).isMeaningful()) {
+                    firstGood = i;
+                }
+            }
+            for (int i = lastGood + 1; i < size(); i++) {
+                Element elem = get(i);
+                if (elem instanceof Tag) {
+                    if (handler.isParagraphTag((Tag) elem)) {
+                        break;
+                    }
+                }
+                if ((elem instanceof Text) && !((Text) elem).isMeaningful()) {
+                    lastGood = i;
+                }
+            }
+        }
     }
 
     /**
      * Enumerates tags to be properly shortcut.
      */
-    private void enumerateTags() {
+    private void enumerateTags(int firstGood, int lastGood) {
         int n = 0;
-        for (int i = getFirstGood(); i <= getLastGood(); i++) {
+        for (int i = firstGood; i <= lastGood; i++) {
             Element elem = get(i);
             if (elem instanceof Tag) {
                 Tag tag = (Tag) elem;
-                if (Tag.TYPE_ALONE == tag.getType() || Tag.TYPE_BEGIN == tag.getType()) {
+                if (Tag.Type.ALONE == tag.getType() || Tag.Type.BEGIN == tag.getType()) {
                     tag.setIndex(n);
                     n++;
-                } else if (Tag.TYPE_END == tag.getType()) {
+                } else if (Tag.Type.END == tag.getType()) {
                     tag.setIndex(-1); // indication of an error
                     // trying to lookup for appropriate starting tag
                     int recursion = 1;
-                    for (int j = i - 1; j >= getFirstGood(); j--) {
+                    for (int j = i - 1; j >= firstGood; j--) {
                         Element otherElem = get(j);
                         if (otherElem instanceof Tag) {
                             Tag other = (Tag) otherElem;
                             if (other.getTag().equals(tag.getTag())) {
-                                if (Tag.TYPE_END == other.getType())
+                                if (Tag.Type.END == other.getType())
                                     recursion++;
-                                else if (Tag.TYPE_BEGIN == other.getType()) {
+                                else if (Tag.Type.BEGIN == other.getType()) {
                                     recursion--;
                                     if (recursion == 0) {
                                         tag.setIndex(other.getIndex());
@@ -315,23 +445,27 @@ public class Entry {
      * 
      * @param tagsAggregation
      *            Whether tags of this entry can be aggregated.
+     * @param xmlDialect
+     *            dialect for processing shortcuts
+     * @param shortcutDetails
+     *            shortcuts details
      */
-    public String sourceToShortcut(boolean tagsAggregation) {
-        StringBuffer buf = new StringBuffer();
-
+    public String sourceToShortcut(boolean tagsAggregation, XMLDialect xmlDialect, List<ProtectedPart> protectedParts) {
         if (tagsAggregation != this.tagsAggregationEnabled) {
             this.tagsAggregationEnabled = tagsAggregation;
             // Each change to tags aggregation setting resets detected tags
             tagsDetected = false;
         }
 
-        for (int i = getFirstGood(); i <= getLastGood(); i++)
-            buf.append(get(i).toShortcut());
-        return buf.toString();
+        if (getFirstGood() <= getLastGood()) {
+            return xmlDialect.constructShortcuts(elements.subList(getFirstGood(), getLastGood() + 1), protectedParts);
+        } else {
+            return "";
+        }
     }
 
-    private String sourceToShortcut() {
-        return sourceToShortcut(tagsAggregationEnabled);
+    private String sourceToShortcut(XMLDialect xmlDialect, List<ProtectedPart> protectedParts) {
+        return sourceToShortcut(tagsAggregationEnabled, xmlDialect, protectedParts);
     }
 
     /**
@@ -341,7 +475,7 @@ public class Entry {
      *       text&lt;ept i="0"&gt;&amp;lt;/b0&amp;gt;&lt;/ept&gt;</code>.
      */
     public String sourceToTMX() {
-        StringBuffer buf = new StringBuffer();
+        StringBuilder buf = new StringBuilder();
         for (int i = 0; i < size(); i++)
             buf.append(get(i).toTMX());
         return buf.toString();
@@ -354,7 +488,7 @@ public class Entry {
      * <code>Here's &lt;b&gt;bold text&lt;/b&gt;</code>.
      */
     public String sourceToOriginal() {
-        StringBuffer buf = new StringBuffer();
+        StringBuilder buf = new StringBuilder();
         for (int i = 0; i < size(); i++)
             buf.append(get(i).toOriginal());
         return buf.toString();
@@ -364,9 +498,6 @@ public class Entry {
     // Dealing with translation
     // //////////////////////////////////////////////////////////////////////////
 
-    /** Translation in shortcut form. */
-    private String translation = new String();
-
     Entry translatedEntry = null;
 
     /**
@@ -395,29 +526,29 @@ public class Entry {
      * @throws TranslationException
      *             -- if any tag is missing or tags are ordered incorrectly.
      */
-    public void setTranslation(String translation) throws TranslationException {
-        if (translation.length() > 0 && !sourceToShortcut().equals(translation)) {
-            checkAndRecoverTags(translation);
-            this.translation = translation;
+    public void setTranslation(String translation, XMLDialect xmlDialect, List<ProtectedPart> protectedParts)
+            throws TranslationException {
+        if (!sourceToShortcut(xmlDialect, protectedParts).equals(translation)) {
+            checkAndRecoverTags(translation, protectedParts);
         }
     }
 
     /**
      * Before setting translation checks whether the translation contains all
      * the same tags in weakly correct order. See
-     * {@link #setTranslation(String)} for details.
+     * {@link #setTranslation(String, XMLDialect, List)} for details.
      */
-    private void checkAndRecoverTags(String translation) throws TranslationException {
-        translatedEntry = new Entry();
+    private void checkAndRecoverTags(String translation, List<ProtectedPart> protectedParts) throws TranslationException {
+        translatedEntry = new Entry(xmlDialect, handler);
 
         // /////////////////////////////////////////////////////////////////////
         // recovering tags
-        List<ShortTag> shortTags = listShortTags(translation);
+        List<TagUtil.Tag> shortTags = TagUtil.buildTagList(translation,
+                protectedParts.toArray(new ProtectedPart[protectedParts.size()]));
         int pos = 0;
-        for (ShortTag shortTag : shortTags) {
+        for (TagUtil.Tag shortTag : shortTags) {
             if (pos < shortTag.pos) {
-                translatedEntry.add(getTextInstance()
-                        .createInstance(translation.substring(pos, shortTag.pos)));
+                translatedEntry.add(createTextInstance(translation.substring(pos, shortTag.pos)));
                 pos = shortTag.pos;
             }
             for (int j = getFirstGood(); j <= getLastGood(); j++) {
@@ -435,14 +566,23 @@ public class Entry {
             // warning.
         }
         if (pos < translation.length())
-            translatedEntry.add(getTextInstance().createInstance(translation.substring(pos)));
+            translatedEntry.add(createTextInstance(translation.substring(pos)));
 
         // /////////////////////////////////////////////////////////////////////
         // checking tags
         // TODO: implement checking
     }
 
-    class ShortTag {
+    private Text createTextInstance(String str) {
+        Text text = getTextInstance();
+        if (text != null) {
+            return text.createInstance(str);
+        } else {
+            return new XMLText(str, false);
+        }
+    }
+
+    static class ShortTag {
         String tag;
         int pos;
 
@@ -452,65 +592,15 @@ public class Entry {
         }
     }
 
-    /**
-     * Lists all OmegaT-style tags within the supplied string. Everything that
-     * looks like <code>&lt;xx0&gt;</code>, <code>&lt;yy1/&gt;</code> or
-     * <code>&lt;/zz2&gt;</code> is considered to probably be a tag.
-     * 
-     * @return List of {@link #ShortTag} classes.
-     */
-    private List<ShortTag> listShortTags(String str) {
-        // The code is nearly the same as in buildTagList in StaticUtils.java
-        final int STATE_NORMAL = 1;
-        final int STATE_COLLECT_TAG = 2;
-
-        int state = STATE_NORMAL;
-
-        List<ShortTag> res = new ArrayList<ShortTag>(str.length() / 4);
-        StringBuffer tag = new StringBuffer(str.length());
-        for (int i = 0; i < str.length(); i++) {
-            char c = str.charAt(i);
-            if (c == '<') // Possible start of a tag
-            {
-                tag.setLength(0);
-                tag.append(c);
-                state = STATE_COLLECT_TAG;
-            } else if (c == '>') // Possible end of a tag
-            {
-                // checking if the tag looks like OmegaT tag,
-                // not 100% correct, but is the best what I can think of now
-                tag.append(c);
-                if (PatternConsts.OMEGAT_TAG.matcher(tag).matches()) {
-                    res.add(new ShortTag(tag.toString(), 1 + i - tag.length()));
-                    tag.setLength(0);
-                    state = STATE_NORMAL;
-                }
-            } else if (state == STATE_COLLECT_TAG)
-                tag.append(c);
-        }
-        return res;
-    }
-
-    /**
-     * Returns shortcut string representation of the entry. E.g. for
-     * <code>Here's &lt;b&gt;bold text&lt;/b&gt;</code> should return
-     * <code>Here's &lt;b0&gt;bold text&lt;/b0&gt;</code>.
-     */
-    public String translationToShortcut() {
-        if (translation.length() == 0)
-            return sourceToShortcut();
-        return translation;
-    }
-
     /**
      * Returns long XML-encoded representation of the entry translation for
      * storing in TMX.
      */
     public String translationToTMX() {
-        if (translation.length() == 0)
+        if (translatedEntry == null)
             return sourceToTMX();
 
-        StringBuffer buf = new StringBuffer();
+        StringBuilder buf = new StringBuilder();
 
         for (int i = 0; i < getFirstGood(); i++)
             buf.append(get(i).toTMX());
@@ -528,10 +618,10 @@ public class Entry {
      * document.
      */
     public String translationToOriginal() {
-        if (translation.length() == 0)
+        if (translatedEntry == null)
             return sourceToOriginal();
 
-        StringBuffer buf = new StringBuffer();
+        StringBuilder buf = new StringBuilder();
 
         for (int i = 0; i < getFirstGood(); i++)
             buf.append(get(i).toOriginal());
@@ -576,4 +666,9 @@ public class Entry {
     public int size() {
         return elements.size();
     }
+
+    /** Returns whether or not the elements list is empty. */
+    public boolean isEmpty() {
+        return elements.isEmpty();
+    }
 }
