@@ -7,24 +7,27 @@
                Home page: http://www.omegat.org/
                Support center: http://groups.yahoo.com/group/OmegaT/
 
- This program is free software; you can redistribute it and/or modify
+ This file is part of OmegaT.
+
+ OmegaT is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
+ OmegaT is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
 package org.omegat.filters3;
 
-import org.omegat.util.StaticUtils;
+import org.omegat.filters3.xml.Handler;
+import org.omegat.filters3.xml.XMLDialect;
+import org.omegat.util.StringUtil;
 
 /**
  * Class for collecting out of turn pieces of document.
@@ -45,9 +48,10 @@ public abstract class OutOfTurnTag extends Tag {
      * of text that should be translated separately, not breaking currently
      * collected text.
      */
-    public OutOfTurnTag(String tag, String shortcut, Attributes attributes) {
-        super(tag, shortcut, Tag.TYPE_ALONE, attributes);
-        entry = new Entry();
+    public OutOfTurnTag(XMLDialect xmlDialect, Handler handler, String tag, String shortcut,
+            Attributes attributes) {
+        super(tag, shortcut, Tag.Type.ALONE, attributes);
+        entry = new Entry(xmlDialect, handler);
     }
 
     /**
@@ -55,40 +59,26 @@ public abstract class OutOfTurnTag extends Tag {
      * be implemented by the decendant.
      * <p>
      * E.g. for OpenDocument footnote (out of turn tag "text:note-body") <code>
-     * &lt;text:note-body>&lt;text:p text:style-name="Endnote">The endnote 
+     * &lt;text:note-body>&lt;text:p text:style-name="Endnote"&gt;The endnote 
      * appears at the end of the document in OO but in the middle of 
-     * the segment in OmegaT.&lt;/text:p>&lt;/text:note-body>
+     * the segment in OmegaT.&lt;/text:p>&lt;/text:note-body&gt;
      * </code> this method should return the same if not translated, namely
      * <code>
-     * &lt;text:note-body>&lt;text:p text:style-name="Endnote">The endnote 
+     * &lt;text:note-body>&lt;text:p text:style-name="Endnote"&gt;The endnote 
      * appears at the end of the document in OO but in the middle of 
-     * the segment in OmegaT.&lt;/text:p>&lt;/text:note-body>
+     * the segment in OmegaT.&lt;/text:p>&lt;/text:note-body&gt;
      * </code>.
      */
     public abstract String toOriginal();
 
-    /**
-     * Returns shortcut string representation of the element. Returns only the
-     * placeholder.
-     * <p>
-     * E.g. for OpenDocument footnote (out of turn tag "text:note-body") <code>
-     * &lt;text:note-body>&lt;text:p text:style-name="Endnote">The endnote 
-     * appears at the end of the document in OO but in the middle of 
-     * the segment in OmegaT.&lt;/text:p>&lt;/text:note-body>
-     * </code> this method should return only <code>&lt;t7/></code>.
-     */
-    public String toShortcut() {
-        return super.toShortcut();
-    }
-
     /**
      * Returns short XML-encoded representation of the out of turn tag to store
      * in TMX, without enclosing &lt;ph&gt;.
      * <p>
      * E.g. for OpenDocument footnote (out of turn tag "text:note-body") <code>
-     * &lt;text:note-body>&lt;text:p text:style-name="Endnote">The endnote 
+     * &lt;text:note-body>&lt;text:p text:style-name="Endnote"&gt;The endnote 
      * appears at the end of the document in OO but in the middle of 
-     * the segment in OmegaT.&lt;/text:p>&lt;/text:note-body>
+     * the segment in OmegaT.&lt;/text:p>&lt;/text:note-body&gt;
      * </code> this method should return the following, if not translated,
      * <code>
      * &amp;lt;text:note-body&amp;gt;&amp;lt;text:p text:style-name="Endnote"&amp;gt;The 
@@ -98,14 +88,14 @@ public abstract class OutOfTurnTag extends Tag {
      * </code>.
      */
     protected String toPartialTMX() {
-        StringBuffer buf = new StringBuffer();
+        StringBuilder buf = new StringBuilder();
 
         buf.append("&amp;lt;");
         buf.append(getTag());
         buf.append(getAttributes().toString());
         buf.append("&amp;gt;");
 
-        buf.append(StaticUtils.makeValidXML(getEntry().translationToOriginal()));
+        buf.append(StringUtil.makeValidXML(getEntry().translationToOriginal()));
 
         buf.append("&amp;lt;/");
         buf.append(getTag());
