@@ -7,35 +7,40 @@
                Home page: http://www.omegat.org/
                Support center: http://groups.yahoo.com/group/OmegaT/
 
- This program is free software; you can redistribute it and/or modify
+ This file is part of OmegaT.
+
+ OmegaT is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
+ OmegaT is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
 package org.omegat.filters3;
 
+import org.omegat.util.StaticUtils;
+
 /**
  * A tag in a source text.
  * 
  * @author Maxym Mykhalchuk
  */
 public abstract class Tag implements Element {
-    /** Begin of a paired tag. */
-    public static final int TYPE_BEGIN = 1;
-    /** End of a paired tag. */
-    public static final int TYPE_END = 2;
-    /** Standalone tag. */
-    public static final int TYPE_ALONE = 3;
+    public enum Type {
+        /** Begin of a paired tag. */
+        BEGIN,
+        /** End of a paired tag. */
+        END,
+        /** Standalone tag. */
+        ALONE
+    };
 
     private String tag;
 
@@ -48,21 +53,22 @@ public abstract class Tag implements Element {
 
     /** Returns the short form of this tag, most often -- the first letter. */
     public String getShortcut() {
-        if (shortcut != null)
+        if (shortcut != null) {
             return shortcut;
-        else
-            return Character.toString(getTag().charAt(0));
+        } else {
+            return String.valueOf(Character.toChars(getTag().codePointAt(0)));
+        }
     }
 
-    private int type;
+    private Type type;
 
     /** Returns type of this tag. */
-    public int getType() {
+    public Type getType() {
         return type;
     }
 
     /** Sets type of this tag. */
-    public void setType(int type) {
+    public void setType(Type type) {
         this.type = type;
     }
 
@@ -73,6 +79,30 @@ public abstract class Tag implements Element {
         return attributes;
     }
 
+    /**
+     * Returns attribute value by name.
+     */
+    public String getAttribute(String name) {
+        for (Attribute a : attributes.list) {
+            if (name.equals(a.getName())) {
+                return a.getValue();
+            }
+        }
+        return null;
+    }
+
+    /** Attributes of correspondent start tag. */
+    private Attributes startAttributes;
+
+    /** Returns tag's attributes. */
+    public Attributes getStartAttributes() {
+        return startAttributes;
+    }
+
+    public void setStartAttributes(Attributes startAttributes) {
+        this.startAttributes = startAttributes;
+    }
+
     private int index;
 
     /** Returns the index of this tag in the entry. */
@@ -91,7 +121,7 @@ public abstract class Tag implements Element {
     }
 
     /** Creates a new instance of Tag */
-    public Tag(String tag, String shortcut, int type, Attributes attributes) {
+    public Tag(String tag, String shortcut, Type type, Attributes attributes) {
         this.tag = tag;
         this.shortcut = shortcut;
         this.type = type;
@@ -100,29 +130,29 @@ public abstract class Tag implements Element {
 
     /**
      * Returns long XML-encoded representation of the tag to store in TMX. This
-     * implementation encloses {@link toPartialTMX()} in &lt;bpt&gt;,
+     * implementation encloses {@link #toPartialTMX()} in &lt;bpt&gt;,
      * &lt;ept&gt; or &lt;ph&gt;. Can be overriden in ancestors if needed, but
      * most probably you won't ever need to override this method, and override
-     * {@link toPartialTMX()} instead. E.g. for &lt;strong&gt; tag should return
-     * &lt;bpt i="3"&gt;&amp;lt;strong&amp;gt;&lt;/bpt&gt;.
+     * {@link #toPartialTMX()} instead. E.g. for &lt;strong&gt; tag should
+     * return &lt;bpt i="3"&gt;&amp;lt;strong&amp;gt;&lt;/bpt&gt;.
      */
     public String toTMX() {
         String tmxtag;
         switch (getType()) {
-        case TYPE_BEGIN:
+        case BEGIN:
             tmxtag = "bpt";
             break;
-        case TYPE_END:
+        case END:
             tmxtag = "ept";
             break;
-        case TYPE_ALONE:
+        case ALONE:
             tmxtag = "ph";
             break;
         default:
             throw new RuntimeException("Shouldn't hapen!");
         }
 
-        StringBuffer buf = new StringBuffer();
+        StringBuilder buf = new StringBuilder();
 
         buf.append("<");
         buf.append(tmxtag);
@@ -146,14 +176,14 @@ public abstract class Tag implements Element {
      * return &amp;lt;strong&amp;gt;
      */
     protected String toPartialTMX() {
-        StringBuffer buf = new StringBuffer();
+        StringBuilder buf = new StringBuilder();
 
         buf.append("&amp;lt;");
-        if (TYPE_END == getType())
+        if (Type.END == getType())
             buf.append("/");
         buf.append(getTag());
         buf.append(getAttributes().toString());
-        if (TYPE_ALONE == getType())
+        if (Type.ALONE == getType())
             buf.append("/");
         buf.append("&amp;gt;");
 
@@ -165,20 +195,25 @@ public abstract class Tag implements Element {
      * &lt;strong&gt; tag should return &lt;s3&gt;.
      */
     public String toShortcut() {
-        StringBuffer buf = new StringBuffer();
+        StringBuilder buf = new StringBuilder();
 
-        buf.append("<");
-        if (TYPE_END == getType())
-            buf.append("/");
+        buf.append('<');
+        if (Type.END == getType())
+            buf.append('/');
         buf.append(getShortcut());
         buf.append(getIndex());
-        if (TYPE_ALONE == getType())
-            buf.append("/");
-        buf.append(">");
+        if (Type.ALONE == getType())
+            buf.append('/');
+        buf.append('>');
 
         return buf.toString();
     }
 
+    public String toSafeCalcShortcut() {
+        return StaticUtils.TAG_REPLACEMENT_CHAR + getShortcut().replace('<', '_').replace('>', '_')
+                + StaticUtils.TAG_REPLACEMENT_CHAR;
+    }
+
     /**
      * Returns the tag in its original form as it was in original document. Must
      * be overriden by ancestors. E.g. for &lt;strong&gt; tag should return
