@@ -7,24 +7,25 @@
                Home page: http://www.omegat.org/
                Support center: http://groups.yahoo.com/group/OmegaT/
 
- This program is free software; you can redistribute it and/or modify
+ This file is part of OmegaT.
+
+ OmegaT is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
+ OmegaT is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
 package org.omegat.filters3;
 
-import org.omegat.util.StaticUtils;
+import org.omegat.util.StringUtil;
 
 /**
  * Abstract piece of text.
@@ -35,11 +36,11 @@ import org.omegat.util.StaticUtils;
  */
 public abstract class Text implements Element {
     /** The text itself. */
-    private String text;
+    private StringBuilder text;
 
     /** Returns the piece of text stored. */
     public String getText() {
-        return text;
+        return text.toString();
     }
 
     private boolean meaningEvaluated = false;
@@ -48,7 +49,7 @@ public abstract class Text implements Element {
     /** Whether the text is meaningful, i.e. contains anything but space. */
     public boolean isMeaningful() {
         if (!meaningEvaluated) {
-            meaningful = text.trim().length() > 0;
+            meaningful = !text.toString().trim().isEmpty();
             meaningEvaluated = true;
         }
         return meaningful;
@@ -56,7 +57,11 @@ public abstract class Text implements Element {
 
     /** Creates a new instance of Text initialized with some text. */
     public Text(String text) {
-        this.text = text;
+        this.text = new StringBuilder(text);
+    }
+
+    public void append(String text) {
+        this.text.append(text);
     }
 
     /**
@@ -70,7 +75,7 @@ public abstract class Text implements Element {
      * Tag[&lt;i&gt;], Text[pyatnitzu], Tag[&lt;/i&gt;], Text[ ya vlyublyon.]).
      * Tags are the same, but text is translated. Even the number of text
      * elements might change! So OmegaT must be able to create Text classes with
-     * appropriate {@link toOriginal()} methods for storing translated text, so
+     * appropriate {@link #toOriginal()} methods for storing translated text, so
      * it would pick any of the original Text instances, e.g. Text[Friday], and
      * call <code>createInstance("V ")</code>,
      * <code>createInstance("pyatnitzu")</code>, and
@@ -83,22 +88,26 @@ public abstract class Text implements Element {
      * text itself.
      */
     public String toShortcut() {
-        return text;
+        return text.toString();
+    }
+
+    public String toSafeCalcShortcut() {
+        return toShortcut();
     }
 
     /**
      * Returns long XML-encoded representation of the element. Basically, the
-     * XML-encoded text (&lt; -> &amp;lt; etc). E.g. for <code>Rock&Roll</code>
+     * XML-encoded text (&lt; -&gt; &amp;lt; etc). E.g. for <code>Rock&amp;Roll</code>
      * should return <code>Rock&amp;Roll</code>.
      */
     public String toTMX() {
-        return StaticUtils.makeValidXML(text);
+        return StringUtil.makeValidXML(text.toString());
     }
 
     /**
      * Returns the text in its original form as it was in original document.
-     * E.g. for <code>Rock&Roll</code> should return <code>Rock&amp;Roll</code>
-     * for XML and <code>Rock&Roll</code> for text files.
+     * E.g. for <code>Rock&amp;Roll</code> should return <code>Rock&amp;Roll</code>
+     * for XML and <code>Rock&amp;Roll</code> for text files.
      */
     public abstract String toOriginal();
 }
