@@ -8,19 +8,20 @@
                Home page: http://www.omegat.org/
                Support center: http://groups.yahoo.com/group/OmegaT/
 
- This program is free software; you can redistribute it and/or modify
+ This file is part of OmegaT.
+
+ OmegaT is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
+ OmegaT is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
 package org.omegat.filters3.xml;
@@ -71,7 +72,7 @@ public class DTD extends XMLPseudoTag {
      * Returns the DTD in its original form as it was in original document.
      */
     public String toOriginal() {
-        StringBuffer res = new StringBuffer();
+        StringBuilder res = new StringBuilder();
         res.append("<!DOCTYPE");
         res.append(" ");
         res.append(name);
@@ -86,7 +87,7 @@ public class DTD extends XMLPseudoTag {
             res.append("\"" + systemId + "\"");
         }
 
-        if (entities.size() > 0) {
+        if (!entities.isEmpty()) {
             res.append("\n[\n");
             for (Entity entity : entities) {
                 res.append(entity.toString());
