@@ -4,22 +4,24 @@
           glossaries, and translation leveraging into updated projects.
 
  Copyright (C) 2000-2006 Keith Godfrey and Maxym Mykhalchuk
+               2013 Alex Buloichik
                Home page: http://www.omegat.org/
                Support center: http://groups.yahoo.com/group/OmegaT/
 
- This program is free software; you can redistribute it and/or modify
+ This file is part of OmegaT.
+
+ OmegaT is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
+ OmegaT is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
 package org.omegat.filters3.xml;
@@ -30,6 +32,7 @@ package org.omegat.filters3.xml;
  * <code>&lt;!ENTITY % dbnotn.module "INCLUDE"&gt;</code> is internal one.
  * 
  * @author Maxym Mykhalchuk
+ * @author Alex Buloichik
  */
 public class Entity {
     private boolean internal;
@@ -46,13 +49,18 @@ public class Entity {
         return parameter;
     }
 
-    private String name;
+    private String name, originalName;
 
     /** Returns entity's name. */
     public String getName() {
         return name;
     }
 
+    /** Returns entity's original name, with '%' for parameter entities. */
+    public String getOriginalName() {
+        return originalName;
+    }
+
     private String value;
 
     /** Returns entity's value. */
@@ -75,9 +83,10 @@ public class Entity {
     }
 
     private void setName(String name) {
-        if (name.charAt(0) == '%') {
+        originalName = name;
+        if (name.codePointAt(0) == '%') {
             parameter = true;
-            this.name = name.substring(1);
+            this.name = name.substring(name.offsetByCodePoints(0, 1));
         } else {
             parameter = false;
             this.name = name;
@@ -103,7 +112,7 @@ public class Entity {
      * Returns the entity as string.
      */
     public String toString() {
-        StringBuffer res = new StringBuffer();
+        StringBuilder res = new StringBuilder();
         res.append("<!ENTITY");
         if (parameter)
             res.append(" %");
