@@ -8,22 +8,24 @@
                2009 Didier Briel
                2010 Antonio Vilei
                2011 Didier Briel
+               2013 Didier Briel, Alex Buloichik
                Home page: http://www.omegat.org/
                Support center: http://groups.yahoo.com/group/OmegaT/
 
- This program is free software; you can redistribute it and/or modify
+ This file is part of OmegaT.
+
+ OmegaT is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
+ OmegaT is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
 package org.omegat.filters3.xml;
@@ -41,15 +43,15 @@ import java.util.List;
 import java.util.Map;
 import java.util.Stack;
 
-import javax.xml.parsers.SAXParser;
-
+import org.omegat.core.Core;
+import org.omegat.core.data.ProtectedPart;
+import org.omegat.filters2.FilterContext;
 import org.omegat.filters2.TranslationException;
 import org.omegat.filters3.Attribute;
 import org.omegat.filters3.Element;
 import org.omegat.filters3.Entry;
 import org.omegat.filters3.Tag;
 import org.omegat.util.OStrings;
-import org.omegat.util.StaticUtils;
 import org.omegat.util.StringUtil;
 import org.xml.sax.Attributes;
 import org.xml.sax.InputSource;
@@ -63,23 +65,29 @@ import org.xml.sax.helpers.DefaultHandler;
  * The part of XML filter that actually does the job. This class is called back
  * by SAXParser.
  * 
+ * Entities described on
+ * http://www.ibm.com/developerworks/xml/library/x-entities/
+ * http://xmlwriter.net/xml_guide/entity_declaration.shtml
+ * 
  * @author Maxym Mykhalchuk
  * @author Martin Fleurke
  * @author Didier Briel
+ * @author Alex Buloichik (alex73mail@gmail.com)
  */
-class Handler extends DefaultHandler implements LexicalHandler, DeclHandler {
-    private SAXParser parser;
+public class Handler extends DefaultHandler implements LexicalHandler, DeclHandler {
     private Translator translator;
     private XMLDialect dialect;
     private File inFile;
-    private String inEncoding;
     private File outFile;
-    private String outEncoding;
+    private FilterContext context;
 
     /** Main file writer to write translated text to. */
     private BufferedWriter mainWriter;
     /** Current writer for an external included file. */
     private BufferedWriter extWriter = null;
+    
+    /** Current path in XML. */
+    private final Stack<String> currentTagPath = new Stack<String>();
 
     /**
      * Returns current writer we should write into. If we're in main file,
@@ -108,6 +116,8 @@ class Handler extends DefaultHandler implements LexicalHandler, DeclHandler {
     Stack<org.omegat.filters3.Attributes> paragraphTagAttributes = new Stack<org.omegat.filters3.Attributes>();
     /** Keep the attributes of preformat tags. */
     Stack<org.omegat.filters3.Attributes> preformatTagAttributes = new Stack<org.omegat.filters3.Attributes>();
+    /** Keep the attributes of xml tags. */
+    Stack<org.omegat.filters3.Attributes> xmlTagAttributes = new Stack<org.omegat.filters3.Attributes>();
 
     /** Current entry that collects the text surrounded by intact tag. */
     String intacttagName = null;
@@ -117,6 +127,8 @@ class Handler extends DefaultHandler implements LexicalHandler, DeclHandler {
     Stack<String> preformatTagName = new Stack<String>();
     /** Name of the current variable translatable tag */
     Stack<String> translatableTagName = new Stack<String>();
+    /** Names of xml tags. */
+    Stack<String> xmlTagName = new Stack<String>();
     /** Status of the xml:space="preserve" flag */
     private boolean spacePreserve = false;
 
@@ -135,7 +147,11 @@ class Handler extends DefaultHandler implements LexicalHandler, DeclHandler {
     }
 
     private boolean isSpacePreservingTag() {
-        return spacePreserve;
+        if (Core.getFilterMaster().getConfig().isPreserveSpaces()) { // Preserve spaces for all tags
+            return true;
+        } else {
+            return spacePreserve;
+        }
     }
     
     private void resetSpacePreservingTag() {
@@ -144,7 +160,8 @@ class Handler extends DefaultHandler implements LexicalHandler, DeclHandler {
 
     /**
      * Returns current entry we collect text into. If we collect normal text,
-     * returns {@link #entry}, else returns {@link #outofturnEntry}.
+     * returns {@link #entry}, else returns the last of
+     * {@link #outofturnEntries}.
      */
     private Entry currEntry() {
         if (collectingIntactText())
@@ -194,10 +211,7 @@ class Handler extends DefaultHandler implements LexicalHandler, DeclHandler {
      * included into main file. Each entry is {@link File}.
      */
     public List<File> getProcessedFiles() {
-        if (processedFiles.size() > 0)
-            return processedFiles;
-        else
-            return null;
+        return processedFiles.isEmpty() ? null : processedFiles;
     }
 
     /** Throws a nice error message when SAX parser encounders fastal error. */
@@ -214,23 +228,25 @@ class Handler extends DefaultHandler implements LexicalHandler, DeclHandler {
         } else
             filename = inFile.getAbsolutePath();
         throw new SAXException("\n"
-                + StaticUtils.format(e.getMessage() + "\n" + OStrings.getString("XML_FATAL_ERROR"),
-                        new Object[] { filename, new Integer(linenum) }));
+                + StringUtil.format(e.getMessage() + "\n" + OStrings.getString("XML_FATAL_ERROR"),
+                        filename, linenum));
     }
 
     /**
      * Creates a new instance of Handler
      */
-    public Handler(SAXParser parser, Translator translator, XMLDialect dialect, File inFile,
-            String inEncoding, File outFile, String outEncoding) throws IOException {
-        this.parser = parser;
+    public Handler(Translator translator, XMLDialect dialect, File inFile, File outFile, FilterContext fc)
+            throws IOException {
         this.translator = translator;
         this.dialect = dialect;
         this.inFile = inFile;
-        this.inEncoding = inEncoding;
         this.outFile = outFile;
-        this.outEncoding = outEncoding;
-        this.mainWriter = translator.createWriter(outFile, outEncoding);
+        this.context = fc;
+        this.mainWriter = translator.createWriter(outFile, fc.getOutEncoding());
+    }
+
+    public FilterContext getContext() {
+        return context;
     }
 
     private static final String START_JARSCHEMA = "jar:";
@@ -253,7 +269,7 @@ class Handler extends DefaultHandler implements LexicalHandler, DeclHandler {
                 res = inFile.getCanonicalFile().getParent();
             } catch (IOException ex) {
             }
-            if (res.charAt(res.length() - 1) != File.separatorChar) {
+            if (res.codePointBefore(res.length()) != File.separatorChar) {
                 res = res + File.separatorChar;
             }
             sourceFolderAbsolutePath = res;
@@ -292,8 +308,8 @@ class Handler extends DefaultHandler implements LexicalHandler, DeclHandler {
         for (Entity entity : externalEntities) {
             if (entity.isInternal())
                 continue;
-            if (StaticUtils.equal(publicId, entity.getPublicId())
-                    && StaticUtils.equal(systemId, entity.getSystemId()))
+            if (StringUtil.equal(publicId, entity.getPublicId())
+                    && StringUtil.equal(systemId, entity.getSystemId()))
                 return entity;
         }
         return null;
@@ -318,13 +334,19 @@ class Handler extends DefaultHandler implements LexicalHandler, DeclHandler {
     private void doEndEntity(String name) throws SAXException, TranslationException, IOException {
         if (inDTD || extEntity == null)
             return;
-        if (extEntity.getName().equals(name)) {
+        if (extEntity.getOriginalName().equals(name)) {
+            boolean parameterEntiry = extEntity.isParameter();
             extEntity = null;
             translateAndFlush();
             extWriter.close();
             extWriter = null;
-            mainWriter.write('&' + name + ';');
+            if (parameterEntiry) {
+                mainWriter.write(name + ';');
+            } else {
+                mainWriter.write('&' + name + ';');
+            }
         }
+
     }
 
     /**
@@ -334,8 +356,8 @@ class Handler extends DefaultHandler implements LexicalHandler, DeclHandler {
     public InputSource doResolve(String publicId, String systemId) throws SAXException, TranslationException,
             IOException, URISyntaxException {
         if (dtd != null
-                && StaticUtils.equal(publicId, dtd.getPublicId())
-                && (StaticUtils.equal(systemId, dtd.getSystemId()) || StaticUtils.equal(
+                && StringUtil.equal(publicId, dtd.getPublicId())
+                && (StringUtil.equal(systemId, dtd.getSystemId()) || StringUtil.equal(
                         localizeSystemId(systemId), dtd.getSystemId()))) {
             inDTD = true;
         }
@@ -362,36 +384,40 @@ class Handler extends DefaultHandler implements LexicalHandler, DeclHandler {
                         translateAndFlush();
                         File extFile = new File(outFile.getParentFile(), localizeSystemId(systemId));
                         processedFiles.add(new File(inFile.getParent(), localizeSystemId(systemId)));
-                        extWriter = translator.createWriter(extFile, outEncoding);
+                        extWriter = translator.createWriter(extFile, context.getOutEncoding());
                         extWriter.write("<?xml version=\"1.0\"?>\n");
                     }
                 }
                 return entity;
             } else
-                return new InputSource(new java.io.StringReader(new String()));
+                return new InputSource(new java.io.StringReader(""));
         } else {
             InputSource source = dialect.resolveEntity(publicId, systemId);
             if (source != null)
                 return source;
             else
-                return new InputSource(new java.io.StringReader(new String()));
+                return new InputSource(new java.io.StringReader(""));
         }
     }
 
     private void queueText(String s) {
+        if (!translator.isInIgnored()) {
+            translator.text(s);
+        }
+
         // TODO: ideally, xml:space=preserved would be handled at this level, but that would suppose
         // knowing here whether we're inside a preformatted tag, etc.
         if (internalEntityStarted != null && s.equals(internalEntityStarted.getValue()))
             currEntry().add(new XMLEntityText(internalEntityStarted));
         else {
             boolean added = false;
-            if (currEntry().size() > 0) {
+            if (!currEntry().isEmpty()) {
                 Element elem = currEntry().get(currEntry().size() - 1);
                 if (elem instanceof XMLText) {
                     XMLText text = (XMLText) elem;
                     if (text.isInCDATA() == inCDATA) {
-                        currEntry().remove(currEntry().size() - 1);
-                        currEntry().add(new XMLText(text.getText() + s, inCDATA));
+                        currEntry().resetTagDetected();
+                        text.append(s);
                         added = true;
                     }
                 }
@@ -402,18 +428,29 @@ class Handler extends DefaultHandler implements LexicalHandler, DeclHandler {
     }
 
     private void queueTag(String tag, Attributes attributes) {
-        Tag xmltag;
+        Tag xmltag = null;
         XMLIntactTag intacttag = null;
         setTranslatableTag(tag, XMLUtils.convertAttributes(attributes));
         setSpacePreservingTag(XMLUtils.convertAttributes(attributes));
-        if (!collectingIntactText() && isIntactTag(tag, XMLUtils.convertAttributes(attributes))) {
-            intacttag = new XMLIntactTag(tag, getShortcut(tag), attributes);
-            xmltag = intacttag;
-            intacttagName = tag;
-            intacttagAttributes = XMLUtils.convertAttributes(attributes);
-        } else
-            xmltag = new XMLTag(tag, getShortcut(tag), Tag.TYPE_BEGIN, attributes);
-
+        if (!collectingIntactText()) {
+            if (isContentBasedTag(tag, XMLUtils.convertAttributes(attributes))) {
+                intacttag = new XMLContentBasedTag(dialect, this, tag, getShortcut(tag), dialect
+                        .getContentBasedTags().get(tag), attributes);
+                xmltag = intacttag;
+                intacttagName = tag;
+                intacttagAttributes = XMLUtils.convertAttributes(attributes);
+            } else if (isIntactTag(tag, XMLUtils.convertAttributes(attributes))) {
+                intacttag = new XMLIntactTag(dialect, this, tag, getShortcut(tag), attributes);
+                xmltag = intacttag;
+                intacttagName = tag;
+                intacttagAttributes = XMLUtils.convertAttributes(attributes);
+            }
+        }
+        if (xmltag == null) {
+            xmltag = new XMLTag(tag, getShortcut(tag), Tag.Type.BEGIN, attributes, this.translator.getTargetLanguage());
+            xmlTagName.push(xmltag.getTag());
+            xmlTagAttributes.push(xmltag.getAttributes());
+        }
         currEntry().add(xmltag);
 
         if (intacttag != null)
@@ -426,26 +463,53 @@ class Handler extends DefaultHandler implements LexicalHandler, DeclHandler {
                         .getTranslatableTagAttributes().containsPair(tag, attr.getName()))
                         && dialect.validateTranslatableTagAttribute(tag, attr.getName(),
                                 xmltag.getAttributes())) {
-                    attr.setValue(StaticUtils.makeValidXML(
-                            translator.translate(StaticUtils.entitiesToCharacters(attr.getValue()))));
+                    attr.setValue(StringUtil.makeValidXML(
+                            translator.translate(StringUtil.unescapeXMLEntities(attr.getValue()), null)));
                 }
             }
         }
     }
 
+    /**
+     * Queue tag that should be ignored by editor, including content and all subtags.
+     */
+    private void queueIgnoredTag(String tag, Attributes attributes) {
+        Tag xmltag = null;
+        setSpacePreservingTag(XMLUtils.convertAttributes(attributes));
+        if (xmltag == null) {
+            xmltag = new XMLTag(tag, getShortcut(tag), Tag.Type.BEGIN, attributes,
+                    this.translator.getTargetLanguage());
+            xmlTagName.push(xmltag.getTag());
+            xmlTagAttributes.push(xmltag.getAttributes());
+        }
+        currEntry().add(xmltag);
+    }
+
     private void queueEndTag(String tag) {
         int len = currEntry().size();
         if (len > 0
                 && (currEntry().get(len - 1) instanceof XMLTag)
                 && (((XMLTag) currEntry().get(len - 1)).getTag().equals(tag) && ((XMLTag) currEntry().get(
-                        len - 1)).getType() == Tag.TYPE_BEGIN) && !isClosingTagRequired()) {
-            ((XMLTag) currEntry().get(len - 1)).setType(Tag.TYPE_ALONE);
+                        len - 1)).getType() == Tag.Type.BEGIN) && !isClosingTagRequired()) {
+            if (((XMLTag) currEntry().get(len - 1)).getTag().equals(xmlTagName.lastElement())) {
+                xmlTagName.pop();
+                xmlTagAttributes.pop();
+            }
+            ((XMLTag) currEntry().get(len - 1)).setType(Tag.Type.ALONE);
         } else {
-            currEntry().add(new XMLTag(tag, getShortcut(tag), Tag.TYPE_END, null));
+            XMLTag xmltag = new XMLTag(tag, getShortcut(tag), Tag.Type.END, null, this.translator.getTargetLanguage());
+            if (xmltag.getTag().equals(xmlTagName.lastElement())) {
+                xmlTagName.pop();
+                xmltag.setStartAttributes(xmlTagAttributes.pop()); // Restore attributes
+            }
+            currEntry().add(xmltag);
         }
     }
 
     private void queueComment(String comment) {
+        if (!translator.isInIgnored()) {
+            translator.comment(comment);
+        }
         currEntry().add(new Comment(comment));
     }
 
@@ -459,36 +523,59 @@ class Handler extends DefaultHandler implements LexicalHandler, DeclHandler {
 
     /** Is called when the tag is started. */
     private void start(String tag, Attributes attributes) throws SAXException, TranslationException {
-        if (isOutOfTurnTag(tag)) {
-            XMLOutOfTurnTag ootTag = new XMLOutOfTurnTag(tag, getShortcut(tag), attributes);
-            currEntry().add(ootTag);
-            outofturnEntries.push(ootTag.getEntry());
+        boolean prevIgnored = translator.isInIgnored();
+        translatorTagStart(tag, attributes);
+
+        if (!translator.isInIgnored()) {
+            if (isOutOfTurnTag(tag)) {
+                XMLOutOfTurnTag ootTag = new XMLOutOfTurnTag(dialect, this, tag, getShortcut(tag), attributes);
+                currEntry().add(ootTag);
+                outofturnEntries.push(ootTag.getEntry());
+            } else {
+                if (isParagraphTag(tag, XMLUtils.convertAttributes(attributes)) && !collectingOutOfTurnText()
+                        && !collectingIntactText()) {
+                    translateAndFlush();
+                }
+                queueTag(tag, attributes);
+            }
         } else {
-            if (isParagraphTag(tag, XMLUtils.convertAttributes(attributes)) && !collectingOutOfTurnText()
-                    && !collectingIntactText()) {
+            if (!prevIgnored) {
+                // start ignored from this tags - need to flush translation
                 translateAndFlush();
             }
-            queueTag(tag, attributes);
+            queueIgnoredTag(tag, attributes);
         }
     }
 
     /** Is called when the tag is ended. */
     private void end(String tag) throws SAXException, TranslationException {
-        if (collectingIntactText() && isIntactTag(tag, null) && tag.equals(intacttagName)) {
-            intacttagEntry = null;
-            intacttagName = null;
-            intacttagAttributes = null;
-            removeTranslatableTag();
-        } else if (collectingOutOfTurnText() && isOutOfTurnTag(tag)) {
-            translateButDontFlash();
-            outofturnEntries.pop();
+        boolean prevIgnored = translator.isInIgnored();
+        if (!translator.isInIgnored()) {
+            if (collectingIntactText() && tag.equals(intacttagName)
+                    && (isIntactTag(tag, null) || isContentBasedTag(tag, null))) {
+                intacttagEntry = null;
+                intacttagName = null;
+                intacttagAttributes = null;
+                removeTranslatableTag();
+            } else if (collectingOutOfTurnText() && isOutOfTurnTag(tag)) {
+                translateButDontFlash();
+                outofturnEntries.pop();
+            } else {
+                queueEndTag(tag);
+                // TODO: If a file doesn't contain any paragraph tag,
+                // the translatable content will be lost
+                if (isParagraphTag(tag) && !collectingOutOfTurnText() && !collectingIntactText())
+                    translateAndFlush();
+                removeTranslatableTag();
+            }
         } else {
             queueEndTag(tag);
-            // TODO: If a file doesn't contain any paragraph tag,
-            // the translatable content will be lost
-            if (isParagraphTag(tag) && !collectingOutOfTurnText() && !collectingIntactText())
-                translateAndFlush();
-            removeTranslatableTag();
+        }
+        
+        translatorTagEnd(tag);
+        if (!translator.isInIgnored() && prevIgnored) {
+            // stop ignored from this tag - need to flush without translate
+            flushButDontTranslate();
         }
     }
 
@@ -499,11 +586,12 @@ class Handler extends DefaultHandler implements LexicalHandler, DeclHandler {
      * @see #translateAndFlush()
      */
     private void translateButDontFlash() throws TranslationException {
-        if (currEntry().size() == 0)
+        if (currEntry().isEmpty())
             return;
 
+        List<ProtectedPart> shortcutDetails = new ArrayList<ProtectedPart>();
         boolean tagsAggregation = isTagsAggregationEnabled();
-        String src = currEntry().sourceToShortcut(tagsAggregation);
+        String src = currEntry().sourceToShortcut(tagsAggregation, dialect, shortcutDetails);
         Element lead = currEntry().get(0);
         String translation = src;
         if ((lead instanceof Tag)
@@ -512,17 +600,20 @@ class Handler extends DefaultHandler implements LexicalHandler, DeclHandler {
              && isTranslatableTag()
              && !StringUtil.isEmpty(src)) {
             resetSpacePreservingTag();
-            translation = translator.translate(src);
+            translation = translator.translate(src, shortcutDetails);
         } else {
-            String compressed = StaticUtils.compressSpaces(src);
+            String compressed = src;
+            if (Core.getFilterMaster().getConfig().isRemoveSpacesNonseg()) {
+                compressed = StringUtil.compressSpaces(src);
+            } 
             if (isTranslatableTag())
-                translation = translator.translate(compressed);
+                translation = translator.translate(compressed, shortcutDetails);
             // untranslated is written out uncompressed
             if (compressed.equals(translation))
                 translation = src;
         }
 
-        currEntry().setTranslation(translation);
+        currEntry().setTranslation(translation, dialect, new ArrayList<ProtectedPart>());
     }
 
     /**
@@ -542,6 +633,18 @@ class Handler extends DefaultHandler implements LexicalHandler, DeclHandler {
         currEntry().clear();
     }
 
+    /**
+     * Write tag's content without translation. Used for ignored tags.
+     */
+    private void flushButDontTranslate() throws SAXException, TranslationException {
+        try {
+            currWriter().write(currEntry().translationToOriginal());
+        } catch (IOException e) {
+            throw new SAXException(e);
+        }
+        currEntry().clear();
+    }
+
     // /////////////////////////////////////////////////////////////////////////
     // Dialect Helper methods
     // /////////////////////////////////////////////////////////////////////////
@@ -588,6 +691,41 @@ class Handler extends DefaultHandler implements LexicalHandler, DeclHandler {
         }
     }
 
+    /**
+     * Returns whether the tag starts a new paragraph.
+     */
+    public boolean isParagraphTag(Tag tag) {
+        if ((dialect.getParagraphTags() != null && dialect.getParagraphTags().contains(tag.getTag()))
+                || isPreformattingTag(tag.getTag(), tag.getAttributes())) {
+            return true;
+        } else if (tag.getType() == Tag.Type.END
+                && isPreformattingTag(tag.getTag(), tag.getStartAttributes())) {
+            return true;
+        } else {
+            return dialect.validateParagraphTag(tag.getTag(), tag.getAttributes());
+        }
+    }
+
+    /**
+     * Returns whether the tag is content based.
+     * 
+     * @param tag
+     *            A tag
+     * @return <code>true</code> or <code>false</false>
+     */
+    private boolean isContentBasedTag(String tag, org.omegat.filters3.Attributes atts) {
+        if (dialect.getContentBasedTags() != null && dialect.getContentBasedTags().containsKey(tag))
+            return true;
+        else {
+            if (atts == null) {
+                if (tag.equals(intacttagName))
+                    atts = intacttagAttributes; // Restore attributes
+            }
+
+            return dialect.validateContentBasedTag(tag, atts);
+        }
+    }
+
     /**
      * Returns whether the tag surrounds preformatted block of text.
      * 
@@ -672,6 +810,25 @@ class Handler extends DefaultHandler implements LexicalHandler, DeclHandler {
             translatableTagName.pop(); // Remove it
     }
 
+    private void translatorTagStart(String tag, Attributes atts) {
+        currentTagPath.push(tag);
+        translator.tagStart(constructCurrentPath(), atts);
+    }
+
+    private void translatorTagEnd(String tag) {
+        translator.tagEnd(constructCurrentPath());
+        while (!currentTagPath.pop().equals(tag)) {
+        }
+    }
+
+    private String constructCurrentPath() {
+        StringBuilder path = new StringBuilder(256);
+        for (String t : currentTagPath) {
+            path.append('/').append(t);
+        }
+        return path.toString();
+    }
+    
     /**
      * If the space-preserving flag is not set, and the attributes say it is one, set it
      *
@@ -737,6 +894,7 @@ class Handler extends DefaultHandler implements LexicalHandler, DeclHandler {
     /**
      * Resolves an external entity.
      */
+    @Override
     public InputSource resolveEntity(String publicId, String systemId) throws SAXException {
         try {
             return doResolve(publicId, systemId);
@@ -750,6 +908,7 @@ class Handler extends DefaultHandler implements LexicalHandler, DeclHandler {
     }
 
     /** Receive notification of the start of an element. */
+    @Override
     public void startElement(String uri, String localName, String qName, Attributes attributes)
             throws SAXException {
         try {
@@ -760,6 +919,7 @@ class Handler extends DefaultHandler implements LexicalHandler, DeclHandler {
     }
 
     /** Receive notification of the end of an element. */
+    @Override
     public void endElement(String uri, String localName, String qName) throws SAXException {
         try {
             end(qName);
@@ -777,6 +937,7 @@ class Handler extends DefaultHandler implements LexicalHandler, DeclHandler {
     }
 
     /** Receive notification of ignorable whitespace in element content. */
+    @Override
     public void ignorableWhitespace(char[] ch, int start, int length) throws SAXException {
         if (inDTD)
             return;
@@ -794,6 +955,7 @@ class Handler extends DefaultHandler implements LexicalHandler, DeclHandler {
      * Receive notification of an XML processing instruction anywhere in the
      * document.
      */
+    @Override
     public void processingInstruction(String target, String data) throws SAXException {
         if (inDTD)
             return;
@@ -801,6 +963,7 @@ class Handler extends DefaultHandler implements LexicalHandler, DeclHandler {
     }
 
     /** Receive notification of the beginning of the document. */
+    @Override
     public void startDocument() throws SAXException {
         try {
             mainWriter.write("<?xml version=\"1.0\"?>\n");
@@ -808,10 +971,11 @@ class Handler extends DefaultHandler implements LexicalHandler, DeclHandler {
             throw new SAXException(e);
         }
 
-        entry = new Entry();
+        entry = new Entry(dialect, this);
     }
 
     /** Receive notification of the end of the document. */
+    @Override
     public void endDocument() throws SAXException {
         try {
             translateAndFlush();
@@ -831,6 +995,7 @@ class Handler extends DefaultHandler implements LexicalHandler, DeclHandler {
     /**
      * Report a fatal XML parsing error. Is used to provide feedback.
      */
+    @Override
     public void fatalError(org.xml.sax.SAXParseException e) throws SAXException {
         try {
             reportFatalError(e);
