@@ -5,22 +5,25 @@
 
  Copyright (C) 2000-2006 Keith Godfrey and Maxym Mykhalchuk
                2007-2008 Didier Briel
+               2013 Didier Briel, Alex Buloichik
+               2015 Aaron Madlon-Kay
                Home page: http://www.omegat.org/
                Support center: http://groups.yahoo.com/group/OmegaT/
 
- This program is free software; you can redistribute it and/or modify
+ This file is part of OmegaT.
+
+ OmegaT is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
+ OmegaT is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
 package org.omegat.filters3.xml;
@@ -31,6 +34,7 @@ import java.io.File;
 import java.io.IOException;
 import java.io.StringWriter;
 import java.io.UnsupportedEncodingException;
+import java.util.List;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
@@ -38,12 +42,14 @@ import javax.xml.parsers.ParserConfigurationException;
 import javax.xml.parsers.SAXParser;
 import javax.xml.parsers.SAXParserFactory;
 
+import org.omegat.core.data.ProtectedPart;
 import org.omegat.filters2.AbstractFilter;
 import org.omegat.filters2.FilterContext;
 import org.omegat.filters2.TranslationException;
-import org.omegat.util.Log;
+import org.omegat.util.Language;
 import org.omegat.util.OConsts;
 import org.omegat.util.PatternConsts;
+import org.xml.sax.Attributes;
 import org.xml.sax.InputSource;
 import org.xml.sax.SAXException;
 
@@ -53,6 +59,8 @@ import org.xml.sax.SAXException;
  * 
  * @author Maxym Mykhalchuk
  * @author Didier Briel
+ * @author Alex Buloichik
+ * @author Aaron Madlon-Kay
  */
 public abstract class XMLFilter extends AbstractFilter implements Translator {
     /** Factory for SAX parsers. */
@@ -80,6 +88,9 @@ public abstract class XMLFilter extends AbstractFilter implements Translator {
     /** Detected encoding of the input XML file. */
     private String encoding;
 
+    /** Detected EOL chars. */
+    private String eol;
+
     /**
      * Creates a special XML-encoding-aware reader of an input file.
      * 
@@ -94,10 +105,12 @@ public abstract class XMLFilter extends AbstractFilter implements Translator {
      * @throws IOException
      *             If any I/O Error occurs upon reader creation.
      */
+    @Override
     public BufferedReader createReader(File inFile, String inEncoding) throws UnsupportedEncodingException,
             IOException {
         XMLReader xmlreader = new XMLReader(inFile, inEncoding);
         this.encoding = xmlreader.getEncoding();
+        this.eol = xmlreader.getEol();
         return new BufferedReader(xmlreader);
     }
 
@@ -116,6 +129,7 @@ public abstract class XMLFilter extends AbstractFilter implements Translator {
      * @throws IOException
      *             If any I/O Error occurs upon writer creation
      */
+    @Override
     public BufferedWriter createWriter(File outFile, String outEncoding) throws UnsupportedEncodingException,
             IOException {
         if (outEncoding == null)
@@ -124,32 +138,49 @@ public abstract class XMLFilter extends AbstractFilter implements Translator {
         if (outFile == null)
             return new BufferedWriter(new StringWriter());
         else
-            return new BufferedWriter(new XMLWriter(outFile, outEncoding));
+            return new BufferedWriter(new XMLWriter(outFile, outEncoding, eol));
     }
 
+    /**
+     * Target language of the project
+     */
+    private Language targetLanguage;
+    
+    /**
+     * @return The target language of the project
+     */
+    @Override
+    public Language getTargetLanguage() {
+        return targetLanguage;
+    }
     /** Processes an XML file. */
+    @Override
     public void processFile(File inFile, File outFile, FilterContext fc) throws IOException,
             TranslationException {
+        BufferedReader inReader = null;
         try {
-            BufferedReader inReader = createReader(inFile, fc.getInEncoding());
+            inReader = createReader(inFile, fc.getInEncoding());
+            inEncodingLastParsedFile = this.encoding;
+            targetLanguage = fc.getTargetLang();
             InputSource source = new InputSource(inReader);
-            source.setSystemId("file:///" + inFile.getCanonicalPath().replace(File.separatorChar, '/'));
+            source.setSystemId(inFile.toURI().toString());
             SAXParser parser = parserFactory.newSAXParser();
-            Handler handler = new Handler(parser, this, dialect, inFile, fc.getInEncoding(), outFile,
-                    fc.getOutEncoding());
+            Handler handler = new Handler(this, dialect, inFile, outFile, fc);
             parser.setProperty("http://xml.org/sax/properties/lexical-handler", handler);
             parser.setProperty("http://xml.org/sax/properties/declaration-handler", handler);
             parser.parse(source, handler);
-            inReader.close();
         } catch (ParserConfigurationException e) {
-            throw new TranslationException(e.getLocalizedMessage());
+            throw new TranslationException(e);
         } catch (SAXException e) {
-            Log.log(e);
-            throw new TranslationException(e.getLocalizedMessage());
+            throw new TranslationException(e);
+        } finally {
+            if (inReader != null) {
+                inReader.close();
+            }
         }
     }
-
-    protected void processFile(BufferedReader inFile, BufferedWriter outFile) throws IOException,
+    @Override
+    protected void processFile(BufferedReader inFile, BufferedWriter outFile, FilterContext fc) throws IOException,
             TranslationException {
         throw new UnsupportedOperationException(
                 "XMLFilter.processFile(BufferedReader,BufferedWriter) should never be called!");
@@ -161,6 +192,7 @@ public abstract class XMLFilter extends AbstractFilter implements Translator {
      * 
      * @return <code>false</code>
      */
+    @Override
     public boolean isSourceEncodingVariable() {
         return false;
     }
@@ -170,6 +202,7 @@ public abstract class XMLFilter extends AbstractFilter implements Translator {
      * 
      * @return <code>true</code>
      */
+    @Override
     public boolean isTargetEncodingVariable() {
         return true;
     }
@@ -177,8 +210,18 @@ public abstract class XMLFilter extends AbstractFilter implements Translator {
     /**
      * The method the Handler would call to pass translatable content to OmegaT core and receive translation.
      */
-    public String translate(String entry) {
-        return super.processEntry(entry);
+    @Override
+    public String translate(String entry, List<ProtectedPart> protectedParts) {
+        if (entryParseCallback != null) {
+            entryParseCallback.addEntry(null, entry, null, false, null, null, this, protectedParts);
+            return entry;
+        } else if (entryTranslateCallback != null) {
+            String translation = entryTranslateCallback.getTranslation(null, entry, null);
+            return translation != null ? translation : entry;
+        } else { // We're not supposed to be there,  (parsing called from inside isFileSupported, for instance)
+            return entry; // so what we return is not important
+        }
+            
     }
 
     /**
@@ -186,9 +229,11 @@ public abstract class XMLFilter extends AbstractFilter implements Translator {
      * Reads {@link org.omegat.util.OConsts#READ_AHEAD_LIMIT} and tries to detect constrained text and match
      * constraints defined in {@link XMLDialect} against them.
      */
+    @Override
     public boolean isFileSupported(BufferedReader reader) {
-        if (dialect.getConstraints() == null || dialect.getConstraints().size() == 0)
+        if (dialect.getConstraints() == null || dialect.getConstraints().isEmpty()) {
             return true;
+        }
 
         try {
             char[] cbuf = new char[OConsts.READ_AHEAD_LIMIT];
@@ -238,4 +283,25 @@ public abstract class XMLFilter extends AbstractFilter implements Translator {
             return false;
         }
     }
+    
+    @Override
+    public void tagStart(String path, Attributes atts) {
+    }
+    
+    @Override
+    public void tagEnd(String path) {
+    }
+
+    @Override
+    public void comment(String comment) {
+    }
+
+    @Override
+    public void text(String text) {
+    }
+
+    @Override
+    public boolean isInIgnored() {
+        return false;
+    }
 }
