@@ -7,19 +7,20 @@
                Home page: http://www.omegat.org/
                Support center: http://groups.yahoo.com/group/OmegaT/
 
- This program is free software; you can redistribute it and/or modify
+ This file is part of OmegaT.
+
+ OmegaT is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
+ OmegaT is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
 package org.omegat.filters3.xml;
@@ -32,7 +33,7 @@ import org.omegat.filters3.Tag;
  * @author Maxym Mykhalchuk
  */
 public class XMLIntactTag extends Tag {
-    private XMLIntactEntry intactContents = new XMLIntactEntry();
+    protected XMLIntactEntry intactContents;
 
     /** Returns the entry to collect text surrounded by intact tag. */
     public XMLIntactEntry getIntactContents() {
@@ -40,8 +41,17 @@ public class XMLIntactTag extends Tag {
     }
 
     /** Creates a new instance of XML Tag */
-    public XMLIntactTag(String tag, String shortcut, org.xml.sax.Attributes attributes) {
-        super(tag, shortcut, TYPE_ALONE, XMLUtils.convertAttributes(attributes));
+    public XMLIntactTag(XMLDialect xmlDialect, Handler handler, String tag, String shortcut,
+            org.xml.sax.Attributes attributes) {
+        super(tag, shortcut, Type.ALONE, XMLUtils.convertAttributes(attributes));
+        this.intactContents = new XMLIntactEntry(xmlDialect, handler);
+    }
+
+    /** Creates a new instance of XML Tag */
+    public XMLIntactTag(XMLDialect xmlDialect, Handler handler, String tag, String shortcut, Type type,
+            org.xml.sax.Attributes attributes) {
+        super(tag, shortcut, type, XMLUtils.convertAttributes(attributes));
+        this.intactContents = new XMLIntactEntry(xmlDialect, handler);
     }
 
     /**
@@ -54,7 +64,7 @@ public class XMLIntactTag extends Tag {
      * should return absolutely the same.
      */
     public String toOriginal() {
-        StringBuffer buf = new StringBuffer();
+        StringBuilder buf = new StringBuilder();
 
         buf.append("<");
         buf.append(getTag());
