@@ -7,19 +7,20 @@
                Home page: http://www.omegat.org/
                Support center: http://groups.yahoo.com/group/OmegaT/
 
- This program is free software; you can redistribute it and/or modify
+ This file is part of OmegaT.
+
+ OmegaT is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
+ OmegaT is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
 package org.omegat.filters3.xml;
@@ -33,8 +34,9 @@ import org.omegat.filters3.OutOfTurnTag;
  */
 public class XMLOutOfTurnTag extends OutOfTurnTag {
     /** Creates a new instance of XML Tag */
-    public XMLOutOfTurnTag(String tag, String shortcut, org.xml.sax.Attributes attributes) {
-        super(tag, shortcut, XMLUtils.convertAttributes(attributes));
+    public XMLOutOfTurnTag(XMLDialect xmlDialect, Handler handler, String tag, String shortcut,
+            org.xml.sax.Attributes attributes) {
+        super(xmlDialect, handler, tag, shortcut, XMLUtils.convertAttributes(attributes));
     }
 
     /**
@@ -52,7 +54,7 @@ public class XMLOutOfTurnTag extends OutOfTurnTag {
      * </code>.
      */
     public String toOriginal() {
-        StringBuffer buf = new StringBuffer();
+        StringBuilder buf = new StringBuilder();
 
         buf.append("<");
         buf.append(getTag());
