@@ -6,38 +6,37 @@
  Copyright (C) 2000-2006 Keith Godfrey and Maxym Mykhalchuk
                2007-2010 Didier Briel
                2010 Antonio Vilei
-               2011 Didier Briel
+               2011-2015 Didier Briel
                Home page: http://www.omegat.org/
                Support center: http://groups.yahoo.com/group/OmegaT/
 
- This program is free software; you can redistribute it and/or modify
+ This file is part of OmegaT.
+
+ OmegaT is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
+ OmegaT is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
 package org.omegat.filters3.xml.openxml;
 
 import java.awt.event.ActionEvent;
-import java.awt.event.KeyEvent;
 import java.util.Map;
 import java.util.TreeMap;
 
 import javax.swing.AbstractAction;
-import javax.swing.Action;
-import javax.swing.JComponent;
-import javax.swing.KeyStroke;
 
 import org.omegat.util.OStrings;
+import org.omegat.util.gui.DockingUI;
+import org.omegat.util.gui.StaticUIUtils;
 
 /**
  * Modal dialog to edit OpenXML filter options.
@@ -46,6 +45,7 @@ import org.omegat.util.OStrings;
  * @author Didier Briel
  * @author Antonio Vilei
  */
+@SuppressWarnings("serial")
 public class EditOpenXMLOptionsDialog extends javax.swing.JDialog {
     /** A return status code - returned if Cancel button has been pressed */
     public static final int RET_CANCEL = 0;
@@ -64,30 +64,31 @@ public class EditOpenXMLOptionsDialog extends javax.swing.JDialog {
         translateEndnotesCB.setSelected(options.getTranslateEndnotes());
         translateHeadersCB.setSelected(options.getTranslateHeaders());
         translateFootersCB.setSelected(options.getTranslateFooters());
+        translateFallbackTextCB.setSelected(options.getTranslateFallbackText());
         translateDiagramsCB.setSelected(options.getTranslateDiagrams());
         translateExcelCommentsCB.setSelected(options.getTranslateExcelComments());
         translateSheetNamesCB.setSelected(options.getTranslateSheetNames());
         translateSlideCommentsCB.setSelected(options.getTranslateSlideComments());
         translateSlideMastersCB.setSelected(options.getTranslateSlideMasters());
         translateSlideLayoutsCB.setSelected(options.getTranslateSlideLayouts());
+        translateSlideLinksCB.setSelected(options.getTranslateSlideLinks());
         translateChartsCB.setSelected(options.getTranslateCharts());
         translateDrawingsCB.setSelected(options.getTranslateDrawings());
         translateWordArtCB.setSelected(options.getTranslateWordArt());
         aggregateTagsCB.setSelected(options.getAggregateTags());
         preserveSpacesCB.setSelected(options.getSpacePreserving());
+        breakOnBrCB.setSelected(options.getBreakOnBr());
 
-        // Handle escape key to close the window
-        KeyStroke escape = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0, false);
-        Action escapeAction = new AbstractAction() {
+        StaticUIUtils.setEscapeAction(this, new AbstractAction() {
+            @Override
             public void actionPerformed(ActionEvent e) {
                 doClose(RET_CANCEL);
             }
-        };
-        getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(escape, "ESCAPE");
-        getRootPane().getActionMap().put("ESCAPE", escapeAction);
+        });
+        DockingUI.displayCentered(this);
     }
 
-    private OpenXMLOptions options;
+    private final OpenXMLOptions options;
 
     public OpenXMLOptions getOptions() {
         return options;
@@ -109,7 +110,6 @@ public class EditOpenXMLOptionsDialog extends javax.swing.JDialog {
     // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
     private void initComponents() {
 
-        buttonGroup1 = new javax.swing.ButtonGroup();
         buttonPanel = new javax.swing.JPanel();
         jPanel2 = new javax.swing.JPanel();
         okButton = new javax.swing.JButton();
@@ -123,6 +123,7 @@ public class EditOpenXMLOptionsDialog extends javax.swing.JDialog {
         translateEndnotesCB = new javax.swing.JCheckBox();
         translateHeadersCB = new javax.swing.JCheckBox();
         translateFootersCB = new javax.swing.JCheckBox();
+        translateFallbackTextCB = new javax.swing.JCheckBox();
         jLabel4 = new javax.swing.JLabel();
         translateExcelCommentsCB = new javax.swing.JCheckBox();
         translateSheetNamesCB = new javax.swing.JCheckBox();
@@ -130,6 +131,7 @@ public class EditOpenXMLOptionsDialog extends javax.swing.JDialog {
         translateSlideCommentsCB = new javax.swing.JCheckBox();
         translateSlideMastersCB = new javax.swing.JCheckBox();
         translateSlideLayoutsCB = new javax.swing.JCheckBox();
+        translateSlideLinksCB = new javax.swing.JCheckBox();
         jLabel7 = new javax.swing.JLabel();
         translateChartsCB = new javax.swing.JCheckBox();
         translateDiagramsCB = new javax.swing.JCheckBox();
@@ -139,6 +141,7 @@ public class EditOpenXMLOptionsDialog extends javax.swing.JDialog {
         jLabel6 = new javax.swing.JLabel();
         aggregateTagsCB = new javax.swing.JCheckBox();
         preserveSpacesCB = new javax.swing.JCheckBox();
+        breakOnBrCB = new javax.swing.JCheckBox();
 
         setTitle(OStrings.getString("OpenXML_FILTER_OPTIONS")); // NOI18N
         setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR));
@@ -177,7 +180,7 @@ public class EditOpenXMLOptionsDialog extends javax.swing.JDialog {
         org.openide.awt.Mnemonics.setLocalizedText(jLabel2, OStrings.getString("OpenDoc_TRANSLATE_ELEMENTS")); // NOI18N
         jPanel1.add(jLabel2);
 
-        jLabel3.setFont(new java.awt.Font("MS Sans Serif", 1, 11));
+        jLabel3.setFont(jLabel3.getFont().deriveFont(jLabel3.getFont().getStyle() | java.awt.Font.BOLD, jLabel3.getFont().getSize()-2));
         jLabel3.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
         org.openide.awt.Mnemonics.setLocalizedText(jLabel3, OStrings.getString("OpenXML_WORD")); // NOI18N
         jPanel1.add(jLabel3);
@@ -235,7 +238,15 @@ public class EditOpenXMLOptionsDialog extends javax.swing.JDialog {
         });
         jPanel1.add(translateFootersCB);
 
-        jLabel4.setFont(new java.awt.Font("MS Sans Serif", 1, 11));
+        org.openide.awt.Mnemonics.setLocalizedText(translateFallbackTextCB, OStrings.getString("OpenXML_TRANSLATE_FALLBACK_TEXT")); // NOI18N
+        translateFallbackTextCB.addActionListener(new java.awt.event.ActionListener() {
+            public void actionPerformed(java.awt.event.ActionEvent evt) {
+                translateFallbackTextCBradiosActionPerformed(evt);
+            }
+        });
+        jPanel1.add(translateFallbackTextCB);
+
+        jLabel4.setFont(jLabel4.getFont().deriveFont(jLabel4.getFont().getStyle() | java.awt.Font.BOLD, jLabel4.getFont().getSize()-2));
         jLabel4.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
         org.openide.awt.Mnemonics.setLocalizedText(jLabel4, OStrings.getString("OpenXML_EXCEL")); // NOI18N
         jPanel1.add(jLabel4);
@@ -257,7 +268,7 @@ public class EditOpenXMLOptionsDialog extends javax.swing.JDialog {
         });
         jPanel1.add(translateSheetNamesCB);
 
-        jLabel5.setFont(new java.awt.Font("MS Sans Serif", 1, 11));
+        jLabel5.setFont(jLabel5.getFont().deriveFont(jLabel5.getFont().getStyle() | java.awt.Font.BOLD, jLabel5.getFont().getSize()-2));
         jLabel5.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
         org.openide.awt.Mnemonics.setLocalizedText(jLabel5, OStrings.getString("OpenXML_POWER_POINT")); // NOI18N
         jPanel1.add(jLabel5);
@@ -277,7 +288,10 @@ public class EditOpenXMLOptionsDialog extends javax.swing.JDialog {
         org.openide.awt.Mnemonics.setLocalizedText(translateSlideLayoutsCB, OStrings.getString("OpenXML_TRANSLATE_SLIDE_LAYOUTS")); // NOI18N
         jPanel1.add(translateSlideLayoutsCB);
 
-        jLabel7.setFont(new java.awt.Font("MS Sans Serif", 1, 11));
+        org.openide.awt.Mnemonics.setLocalizedText(translateSlideLinksCB, OStrings.getString("OpenXML_TRANSLATE_SLIDE_LINKS")); // NOI18N
+        jPanel1.add(translateSlideLinksCB);
+
+        jLabel7.setFont(jLabel7.getFont().deriveFont(jLabel7.getFont().getStyle() | java.awt.Font.BOLD, jLabel7.getFont().getSize()-2));
         jLabel7.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
         org.openide.awt.Mnemonics.setLocalizedText(jLabel7, OStrings.getString("OpenXML_GLOBAL")); // NOI18N
         jPanel1.add(jLabel7);
@@ -332,12 +346,17 @@ public class EditOpenXMLOptionsDialog extends javax.swing.JDialog {
         });
         jPanel1.add(preserveSpacesCB);
 
+        org.openide.awt.Mnemonics.setLocalizedText(breakOnBrCB, OStrings.getString("OpenXML_BREAK_ON_BR")); // NOI18N
+        breakOnBrCB.addActionListener(new java.awt.event.ActionListener() {
+            public void actionPerformed(java.awt.event.ActionEvent evt) {
+                breakOnBrCBradiosActionPerformed(evt);
+            }
+        });
+        jPanel1.add(breakOnBrCB);
+
         getContentPane().add(jPanel1, java.awt.BorderLayout.CENTER);
 
         pack();
-        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
-        java.awt.Dimension dialogSize = getSize();
-        setLocation((screenSize.width-dialogSize.width)/2,(screenSize.height-dialogSize.height)/2);
     }// </editor-fold>//GEN-END:initComponents
 
     private void preserveSpacesCBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_preserveSpacesCBActionPerformed
@@ -348,6 +367,14 @@ public class EditOpenXMLOptionsDialog extends javax.swing.JDialog {
         // TODO add your handling code here:
     }//GEN-LAST:event_translateWordArtCBradiosActionPerformed
 
+    private void translateFallbackTextCBradiosActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_translateFallbackTextCBradiosActionPerformed
+        // TODO add your handling code here:
+    }//GEN-LAST:event_translateFallbackTextCBradiosActionPerformed
+
+    private void breakOnBrCBradiosActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_breakOnBrCBradiosActionPerformed
+        // TODO add your handling code here:
+    }//GEN-LAST:event_breakOnBrCBradiosActionPerformed
+
     private void translateExcelCommentsCBradiosActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_translateExcelCommentsCBradiosActionPerformed
     // TODO add your handling code here:
     }// GEN-LAST:event_translateExcelCommentsCBradiosActionPerformed
@@ -387,17 +414,20 @@ public class EditOpenXMLOptionsDialog extends javax.swing.JDialog {
         options.setTranslateEndnotes(translateEndnotesCB.isSelected());
         options.setTranslateHeaders(translateHeadersCB.isSelected());
         options.setTranslateFooters(translateFootersCB.isSelected());
+        options.setTranslateFallbackText(translateFallbackTextCB.isSelected());
         options.setTranslateDiagrams(translateDiagramsCB.isSelected());
         options.setTranslateExcelComments(translateExcelCommentsCB.isSelected());
         options.setTranslateSheetNames(translateSheetNamesCB.isSelected());
         options.setTranslateSlideComments(translateSlideCommentsCB.isSelected());
         options.setTranslateSlideMasters(translateSlideMastersCB.isSelected());
         options.setTranslateSlideLayouts(translateSlideLayoutsCB.isSelected());
+        options.setTranslateSlideLinks(translateSlideLinksCB.isSelected());
         options.setTranslateCharts(translateChartsCB.isSelected());
         options.setTranslateDrawings(translateDrawingsCB.isSelected());
         options.setTranslateWordArt(translateWordArtCB.isSelected());
         options.setAggregateTags(aggregateTagsCB.isSelected());
         options.setSpacePreserving(preserveSpacesCB.isSelected());
+        options.setBreakOnBr(breakOnBrCB.isSelected());
         doClose(RET_OK);
     }// GEN-LAST:event_okButtonActionPerformed
 
@@ -436,7 +466,7 @@ public class EditOpenXMLOptionsDialog extends javax.swing.JDialog {
 
     // Variables declaration - do not modify//GEN-BEGIN:variables
     private javax.swing.JCheckBox aggregateTagsCB;
-    private javax.swing.ButtonGroup buttonGroup1;
+    private javax.swing.JCheckBox breakOnBrCB;
     private javax.swing.JPanel buttonPanel;
     private javax.swing.JButton cancelButton;
     private javax.swing.JLabel jLabel1;
@@ -456,6 +486,7 @@ public class EditOpenXMLOptionsDialog extends javax.swing.JDialog {
     private javax.swing.JCheckBox translateDrawingsCB;
     private javax.swing.JCheckBox translateEndnotesCB;
     private javax.swing.JCheckBox translateExcelCommentsCB;
+    private javax.swing.JCheckBox translateFallbackTextCB;
     private javax.swing.JCheckBox translateFootersCB;
     private javax.swing.JCheckBox translateFootnotesCB;
     private javax.swing.JCheckBox translateHeadersCB;
@@ -463,6 +494,7 @@ public class EditOpenXMLOptionsDialog extends javax.swing.JDialog {
     private javax.swing.JCheckBox translateSheetNamesCB;
     private javax.swing.JCheckBox translateSlideCommentsCB;
     private javax.swing.JCheckBox translateSlideLayoutsCB;
+    private javax.swing.JCheckBox translateSlideLinksCB;
     private javax.swing.JCheckBox translateSlideMastersCB;
     private javax.swing.JCheckBox translateWordArtCB;
     // End of variables declaration//GEN-END:variables
