@@ -5,23 +5,24 @@
 
  Copyright (C) 2007-2010 Didier Briel
                2010 Antonio Vilei
-               2011 Didier Briel
+               2011-2015 Didier Briel               
                Home page: http://www.omegat.org/
                Support center: http://groups.yahoo.com/group/OmegaT/
 
- This program is free software; you can redistribute it and/or modify
+ This file is part of OmegaT.
+
+ OmegaT is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
+ OmegaT is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
 package org.omegat.filters3.xml.openxml;
@@ -43,11 +44,13 @@ import org.omegat.filters2.AbstractOptions;
  * <li>[+] Endnotes (Word)
  * <li>[+] Header (Word)
  * <li>[+] Footer (Word)
+ * <li>[] Duplicate fallback text (Word)
  * <li>[+] Comments (Excel)
  * <li>[] Sheet names (Excel)
  * <li>[+] Slide comments (PowerPoint)
  * <li>[] Slide Masters (PowerPoint)
  * <li>[] Slide Layouts (PowerPoint)
+ * <li>[] External links (PowerPoint)
  * <li>[] Charts (Global)
  * <li>[] Diagrams (Global)
  * <li>[] Drawings (Global)
@@ -57,6 +60,7 @@ import org.omegat.filters2.AbstractOptions;
  * <ul>
  * <li>[+] Tags Aggregation
  * <li>[+] Preserve space for all tags
+ * <li>[+] Start a new paragraph on Word soft return
  * </ul>
  * 
  * @author Didier Briel, Antonio Vilei
@@ -68,17 +72,20 @@ public class OpenXMLOptions extends AbstractOptions {
     private static final String OPTION_TRANSLATE_ENDNOTES = "translateEndnotes";
     private static final String OPTION_TRANSLATE_HEADERS = "translateHeaders";
     private static final String OPTION_TRANSLATE_FOOTERS = "translateFooters";
+    private static final String OPTION_TRANSLATE_FALLBACK_TEXT = "translateFallbackText";
     private static final String OPTION_TRANSLATE_DIAGRAMS = "translateDiagrams";
     private static final String OPTION_TRANSLATE_EXCEL_COMMENTS = "translateExcelComments";
     private static final String OPTION_TRANSLATE_SHEET_NAMES = "translateSheetNames";
     private static final String OPTION_TRANSLATE_SLIDE_COMMENTS = "translateSlideComments";
     private static final String OPTION_TRANSLATE_SLIDE_MASTERS = "translateSlideMasters";
     private static final String OPTION_TRANSLATE_SLIDE_LAYOUTS = "translateSlideLayouts";
+    private static final String OPTION_TRANSLATE_SLIDE_LINKS = "translateSlideLinks";   
     private static final String OPTION_TRANSLATE_CHARTS = "translateCharts";
     private static final String OPTION_TRANSLATE_DRAWINGS = "translateDrawings";
     private static final String OPTION_TRANSLATE_WORDART = "translateWordArt";
     private static final String OPTION_AGGREGATE_TAGS = "aggregateTags";
     private static final String OPTION_PRESERVE_SPACES = "preserveSpaces";
+    private static final String OPTION_BREAK_ON_BR = "breakOnBr";
 
     public OpenXMLOptions(Map<String, String> options) {
         super(options);
@@ -167,6 +174,20 @@ public class OpenXMLOptions extends AbstractOptions {
     public void setTranslateFooters(boolean translateFooters) {
         setBoolean(OPTION_TRANSLATE_FOOTERS, translateFooters);
     }
+    
+    /**
+     * @return whether Fallback Text should be translated.
+     */
+    public boolean getTranslateFallbackText() {
+        return getBoolean(OPTION_TRANSLATE_FALLBACK_TEXT, false);
+    }
+
+    /**
+     * Sets whether Fallback Text should be translated.
+     */
+    public void setTranslateFallbackText(boolean translateFallbackText) {
+        setBoolean(OPTION_TRANSLATE_FALLBACK_TEXT, translateFallbackText);
+    }
 
     /**
      * Returns whether Diagrams should be translated.
@@ -252,6 +273,22 @@ public class OpenXMLOptions extends AbstractOptions {
         setBoolean(OPTION_TRANSLATE_SLIDE_LAYOUTS, translateSlideLayouts);
     }
 
+    /**
+     * Returns whether Slide Links should be translated.
+     * @return the state of OPTION_TRANSLATE_SLIDE_LINKS
+     */
+    public boolean getTranslateSlideLinks() {
+        return getBoolean(OPTION_TRANSLATE_SLIDE_LINKS, false);
+    }
+
+    /**
+     * Sets whether Slide Links should be translated.
+     * @param translateSlideLinks The option to translate external links
+     */
+    public void setTranslateSlideLinks(boolean translateSlideLinks) {
+        setBoolean(OPTION_TRANSLATE_SLIDE_LINKS, translateSlideLinks);
+    }
+    
     /**
      * Returns whether Charts should be translated.
      */
@@ -323,4 +360,22 @@ public class OpenXMLOptions extends AbstractOptions {
     public void setSpacePreserving(boolean onOff) {
         setBoolean(OPTION_PRESERVE_SPACES, onOff);
     }
+    
+    /**
+     * Returns whether a new paragraph should be created on "w:br"
+     * @return the state of OPTION_BREAK_ON_BR
+     */
+    public boolean getBreakOnBr() {
+        return getBoolean(OPTION_BREAK_ON_BR, true);
+    }
+    
+    /**
+     * Sets whether a new paragraph should be created on "w:br"
+     * @param breakOnBr The option whether to break or not
+     */
+    public void setBreakOnBr(boolean breakOnBr) {
+        setBoolean(OPTION_BREAK_ON_BR, breakOnBr);
+    }
+            
+
 }
