@@ -4,22 +4,24 @@
           glossaries, and translation leveraging into updated projects.
 
  Copyright (C) 2000-2006 Keith Godfrey and Maxym Mykhalchuk
+               2013 Aaron Madlon-Kay
                Home page: http://www.omegat.org/
                Support center: http://groups.yahoo.com/group/OmegaT/
 
- This program is free software; you can redistribute it and/or modify
+ This file is part of OmegaT.
+
+ OmegaT is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
+ OmegaT is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
 package org.omegat.gui;
@@ -30,6 +32,7 @@ import javax.swing.event.HyperlinkListener;
 
 import org.omegat.core.Core;
 import org.omegat.gui.main.MainWindow;
+import org.omegat.gui.tagvalidation.TagValidationFrame;
 
 /**
  * A listener for old Hyperlink-like style
@@ -37,18 +40,25 @@ import org.omegat.gui.main.MainWindow;
  * @author Keith Godfrey
  */
 public class HListener implements HyperlinkListener {
-    public HListener(MainWindow t, boolean grabFocus) {
+    public HListener(MainWindow t, TagValidationFrame f, boolean grabFocus) {
         m_transFrame = t;
+        m_tagValFrame = f;
         m_grabFocus = grabFocus;
     }
 
     public void hyperlinkUpdate(HyperlinkEvent e) {
         if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
-            final String s = e.getDescription();
+            final String desc = e.getDescription();
             SwingUtilities.invokeLater(new Runnable() {
                 public void run() {
                     try {
-                        Core.getEditor().gotoEntry(Integer.parseInt(s));
+                        if (desc.startsWith(TagValidationFrame.FIX_URL_PREFIX)) {
+                            int entry = Integer.parseInt(desc.substring(TagValidationFrame.FIX_URL_PREFIX.length()));
+                            String fixedSource = m_tagValFrame.fixEntry(entry);
+                            Core.getEditor().gotoEntryAfterFix(entry, fixedSource);
+                        } else {
+                            Core.getEditor().gotoEntry(Integer.parseInt(desc));
+                        }
                     } catch (NumberFormatException ex) {
                     }
                 }
@@ -62,4 +72,5 @@ public class HListener implements HyperlinkListener {
 
     private MainWindow m_transFrame;
     private boolean m_grabFocus;
+    private TagValidationFrame m_tagValFrame;
 }
