@@ -7,35 +7,35 @@
                Home page: http://www.omegat.org/
                Support center: http://groups.yahoo.com/group/OmegaT/
 
- This program is free software; you can redistribute it and/or modify
+ This file is part of OmegaT.
+
+ OmegaT is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
+ OmegaT is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
 package org.omegat.gui.dialogs;
 
 import java.awt.Frame;
-import java.awt.event.ActionEvent;
-import java.awt.event.KeyEvent;
 
-import javax.swing.AbstractAction;
-import javax.swing.Action;
-import javax.swing.JComponent;
+import javax.swing.ImageIcon;
 import javax.swing.JDialog;
-import javax.swing.KeyStroke;
 
+import org.omegat.gui.common.OmegaTIcons;
 import org.omegat.util.OStrings;
-import org.omegat.util.StaticUtils;
+import org.omegat.util.Platform;
+import org.omegat.util.StringUtil;
+import org.omegat.util.gui.DockingUI;
+import org.omegat.util.gui.StaticUIUtils;
 
 /**
  * About dialog, showing OmegaT version and information on contributors.
@@ -43,6 +43,7 @@ import org.omegat.util.StaticUtils;
  * @author Maxym Mykhalchuk
  * @author Henry Pijffers (henry.pijffers@saxnot.com)
  */
+@SuppressWarnings("serial")
 public class AboutDialog extends JDialog {
     /** A return status code - returned if Cancel button has been pressed */
     public static final int RET_CANCEL = 0;
@@ -53,34 +54,36 @@ public class AboutDialog extends JDialog {
     public AboutDialog(Frame parent) {
         super(parent, true);
 
-        // HP
-        // Handle escape key to close the window
-        KeyStroke escape = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0, false);
-        Action escapeAction = new AbstractAction() {
-            public void actionPerformed(ActionEvent e) {
-                dispose();
-            }
-        };
-        getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(escape, "ESCAPE");
-        getRootPane().getActionMap().put("ESCAPE", escapeAction);
-        // END HP
+        StaticUIUtils.setEscapeClosable(this);
 
         initComponents();
-        if ((OStrings.UPDATE != null) && !OStrings.UPDATE.equals("0")) {
-            versionLabel.setText(StaticUtils.format(OStrings.getString("ABOUTDIALOG_VERSION_UPDATE"),
-                    new Object[] { OStrings.VERSION, OStrings.UPDATE }));
-        } else {
-            versionLabel.setText(StaticUtils.format(OStrings.getString("ABOUTDIALOG_VERSION"),
-                    new Object[] { OStrings.VERSION }));
-        }
+        versionLabel.setText(getVersionString());
+
         Object[] args = { Runtime.getRuntime().totalMemory() / 1024 / 1024,
                 Runtime.getRuntime().freeMemory() / 1024 / 1024,
                 Runtime.getRuntime().maxMemory() / 1024 / 1024 };
-        String memoryUsage = StaticUtils.format(OStrings.getString("MEMORY_USAGE"), args);
+        String memoryUsage = StringUtil.format(OStrings.getString("MEMORY_USAGE"), args);
         memoryusage.setText(memoryUsage);
+        
+        String javaVersion = StringUtil.format(OStrings.getString("JAVA_VERSION"),
+                System.getProperty("java.version"), Platform.is64Bit() ? 64 : 32);
+        javaversion.setText(javaVersion);
 
         invalidate();
         pack();
+
+        StaticUIUtils.fitInScreen(this);
+        DockingUI.displayCentered(this);
+    }
+
+    private String getVersionString() {
+        if (!StringUtil.isEmpty(OStrings.UPDATE) && !OStrings.UPDATE.equals("0")) {
+            return StringUtil.format(OStrings.getString("ABOUTDIALOG_VERSION_UPDATE_REVISION"),
+                    OStrings.VERSION, OStrings.UPDATE, OStrings.REVISION);
+        } else {
+            return StringUtil.format(OStrings.getString("ABOUTDIALOG_VERSION_REVISION"),
+                    OStrings.VERSION, OStrings.REVISION);
+        }
     }
 
     /** @return the return status of this dialog - one of RET_OK or RET_CANCEL */
@@ -94,7 +97,7 @@ public class AboutDialog extends JDialog {
      * regenerated by the Form Editor.
      */
     // <editor-fold defaultstate="collapsed"
-    // desc="Generated Code">//GEN-BEGIN:initComponents
+    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
     private void initComponents() {
 
         buttonPanel = new javax.swing.JPanel();
@@ -102,13 +105,15 @@ public class AboutDialog extends JDialog {
         jPanel1 = new javax.swing.JPanel();
         licenseButton = new javax.swing.JButton();
         okButton = new javax.swing.JButton();
+        jPanel2 = new javax.swing.JPanel();
+        javaversion = new javax.swing.JLabel();
         memoryusage = new javax.swing.JLabel();
         versionLabel = new javax.swing.JLabel();
+        aboutpane = new javax.swing.JScrollPane();
         abouttext = new javax.swing.JTextArea();
 
-        setTitle(OStrings.getString("ABOUTDIALOG_TITLE"));
+        setTitle(OStrings.getString("ABOUTDIALOG_TITLE")); // NOI18N
         setModal(true);
-        setResizable(false);
         addWindowListener(new java.awt.event.WindowAdapter() {
             public void windowClosing(java.awt.event.WindowEvent evt) {
                 closeDialog(evt);
@@ -119,12 +124,11 @@ public class AboutDialog extends JDialog {
         buttonPanel.setBorder(javax.swing.BorderFactory.createEmptyBorder(5, 5, 5, 5));
         buttonPanel.setLayout(new java.awt.BorderLayout());
 
-        org.openide.awt.Mnemonics.setLocalizedText(jLabel2, OStrings.getString("ABOUTDIALOG_COPYRIGHT"));
+        org.openide.awt.Mnemonics.setLocalizedText(jLabel2, OStrings.getString("ABOUTDIALOG_COPYRIGHT")); // NOI18N
         jLabel2.setBorder(javax.swing.BorderFactory.createEmptyBorder(5, 5, 5, 5));
         buttonPanel.add(jLabel2, java.awt.BorderLayout.WEST);
 
-        org.openide.awt.Mnemonics.setLocalizedText(licenseButton,
-                OStrings.getString("ABOUTDIALOG_LICENSE_BUTTON"));
+        org.openide.awt.Mnemonics.setLocalizedText(licenseButton, OStrings.getString("ABOUTDIALOG_LICENSE_BUTTON")); // NOI18N
         licenseButton.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                 licenseButtonActionPerformed(evt);
@@ -132,7 +136,7 @@ public class AboutDialog extends JDialog {
         });
         jPanel1.add(licenseButton);
 
-        org.openide.awt.Mnemonics.setLocalizedText(okButton, OStrings.getString("BUTTON_OK"));
+        org.openide.awt.Mnemonics.setLocalizedText(okButton, OStrings.getString("BUTTON_OK")); // NOI18N
         okButton.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                 okButtonActionPerformed(evt);
@@ -142,46 +146,51 @@ public class AboutDialog extends JDialog {
 
         buttonPanel.add(jPanel1, java.awt.BorderLayout.EAST);
 
-        org.openide.awt.Mnemonics.setLocalizedText(memoryusage, OStrings.getString("MEMORY_USAGE"));
-        buttonPanel.add(memoryusage, java.awt.BorderLayout.PAGE_START);
+        jPanel2.setLayout(new java.awt.GridLayout(0, 1));
+
+        org.openide.awt.Mnemonics.setLocalizedText(javaversion, OStrings.getString("JAVA_VERSION")); // NOI18N
+        jPanel2.add(javaversion);
+
+        org.openide.awt.Mnemonics.setLocalizedText(memoryusage, OStrings.getString("MEMORY_USAGE")); // NOI18N
+        jPanel2.add(memoryusage);
+
+        buttonPanel.add(jPanel2, java.awt.BorderLayout.NORTH);
 
         getContentPane().add(buttonPanel, java.awt.BorderLayout.SOUTH);
 
-        versionLabel.setIcon(new javax.swing.ImageIcon(getClass().getResource(
-                "/org/omegat/gui/resources/OmegaT.gif")));
-        org.openide.awt.Mnemonics.setLocalizedText(versionLabel, OStrings.getString("ABOUTDIALOG_VERSION"));
+        versionLabel.setIcon(new ImageIcon(OmegaTIcons.ICONS.get(1)));
+        org.openide.awt.Mnemonics.setLocalizedText(versionLabel, OStrings.getString("ABOUTDIALOG_VERSION_REVISION")); // NOI18N
         versionLabel.setBorder(javax.swing.BorderFactory.createEmptyBorder(5, 5, 5, 5));
         getContentPane().add(versionLabel, java.awt.BorderLayout.NORTH);
 
-        abouttext.setBackground(javax.swing.UIManager.getDefaults().getColor("Label.background"));
+        aboutpane.setHorizontalScrollBarPolicy(javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
+        aboutpane.setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR));
+
         abouttext.setEditable(false);
+        abouttext.setBackground(javax.swing.UIManager.getDefaults().getColor("Label.background"));
         abouttext.setFont(versionLabel.getFont());
         abouttext.setLineWrap(true);
-        abouttext.setText(OStrings.getString("ABOUTDIALOG_CONTRIBUTORS"));
+        abouttext.setText(OStrings.getString("ABOUTDIALOG_CONTRIBUTORS")); // NOI18N
         abouttext.setWrapStyleWord(true);
         abouttext.setBorder(javax.swing.BorderFactory.createEmptyBorder(3, 3, 3, 3));
-        getContentPane().add(abouttext, java.awt.BorderLayout.CENTER);
+        abouttext.setCaretPosition(0);
+        aboutpane.setViewportView(abouttext);
 
-        pack();
-        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
-        java.awt.Dimension dialogSize = getSize();
-        setLocation((screenSize.width - dialogSize.width) / 2, (screenSize.height - dialogSize.height) / 2);
+        getContentPane().add(aboutpane, java.awt.BorderLayout.CENTER);
     }// </editor-fold>//GEN-END:initComponents
 
-    private void licenseButtonActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_licenseButtonActionPerformed
+    private void licenseButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_licenseButtonActionPerformed
         new LicenseDialog(this).setVisible(true);
-    }// GEN-LAST:event_licenseButtonActionPerformed
+    }//GEN-LAST:event_licenseButtonActionPerformed
 
-    private void okButtonActionPerformed(java.awt.event.ActionEvent evt)// GEN-FIRST:event_okButtonActionPerformed
-    {
+    private void okButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_okButtonActionPerformed
         doClose(RET_OK);
-    }// GEN-LAST:event_okButtonActionPerformed
+    }//GEN-LAST:event_okButtonActionPerformed
 
     /** Closes the dialog */
-    private void closeDialog(java.awt.event.WindowEvent evt)// GEN-FIRST:event_closeDialog
-    {
+    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
         doClose(RET_CANCEL);
-    }// GEN-LAST:event_closeDialog
+    }//GEN-LAST:event_closeDialog
 
     private void doClose(int retStatus) {
         returnStatus = retStatus;
@@ -190,10 +199,13 @@ public class AboutDialog extends JDialog {
     }
 
     // Variables declaration - do not modify//GEN-BEGIN:variables
+    private javax.swing.JScrollPane aboutpane;
     private javax.swing.JTextArea abouttext;
     private javax.swing.JPanel buttonPanel;
     private javax.swing.JLabel jLabel2;
     private javax.swing.JPanel jPanel1;
+    private javax.swing.JPanel jPanel2;
+    private javax.swing.JLabel javaversion;
     private javax.swing.JButton licenseButton;
     private javax.swing.JLabel memoryusage;
     private javax.swing.JButton okButton;
