@@ -8,36 +8,32 @@
                Home page: http://www.omegat.org/
                Support center: http://groups.yahoo.com/group/OmegaT/
 
- This program is free software; you can redistribute it and/or modify
+ This file is part of OmegaT.
+
+ OmegaT is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
+ OmegaT is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
 package org.omegat.gui.dialogs;
 
 import java.awt.Font;
 import java.awt.Frame;
-import java.awt.event.ActionEvent;
-import java.awt.event.KeyEvent;
-
-import javax.swing.AbstractAction;
-import javax.swing.Action;
-import javax.swing.JComponent;
-import javax.swing.KeyStroke;
 
 import org.omegat.util.OStrings;
 import org.omegat.util.Preferences;
 import org.omegat.util.StaticUtils;
+import org.omegat.util.gui.DockingUI;
+import org.omegat.util.gui.StaticUIUtils;
 
 /**
  * The dialog to change the font of OmegaT windows.
@@ -45,6 +41,7 @@ import org.omegat.util.StaticUtils;
  * @author Maxym Mykhalchuk
  * @author Didier Briel
  */
+@SuppressWarnings("serial")
 public class FontSelectionDialog extends javax.swing.JDialog {
     /**
      * A return status code - returned if Cancel button has been pressed or User
@@ -58,7 +55,7 @@ public class FontSelectionDialog extends javax.swing.JDialog {
     public static final int RET_OK_CHANGED = 1;
 
     /** The old font, passed to this Dialog */
-    private Font oldFont;
+    private final Font oldFont;
 
     /** New Font, selected by the user */
     public Font getSelectedFont() {
@@ -70,17 +67,7 @@ public class FontSelectionDialog extends javax.swing.JDialog {
     public FontSelectionDialog(Frame parent, Font font) {
         super(parent, true);
 
-        // HP
-        // Handle escape key to close the window
-        KeyStroke escape = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0, false);
-        Action escapeAction = new AbstractAction() {
-            public void actionPerformed(ActionEvent e) {
-                dispose();
-            }
-        };
-        getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(escape, "ESCAPE");
-        getRootPane().getActionMap().put("ESCAPE", escapeAction);
-        // END HP
+        StaticUIUtils.setEscapeClosable(this);
 
         initComponents();
 
@@ -89,9 +76,9 @@ public class FontSelectionDialog extends javax.swing.JDialog {
         oldFont = font;
         previewTextArea.setFont(oldFont);
         fontComboBox.setSelectedItem(oldFont.getName());
-        sizeSpinner.setValue(new Integer(oldFont.getSize()));
+        sizeSpinner.setValue(oldFont.getSize());
         applyToProjectFilesCheckBox.setSelected(Preferences.isPreference(Preferences.PROJECT_FILES_USE_FONT));
-
+        DockingUI.displayCentered(this);
     }
 
     /** @return the return status of this dialog - one of RET_OK or RET_CANCEL */
@@ -105,7 +92,7 @@ public class FontSelectionDialog extends javax.swing.JDialog {
      * regenerated by the Form Editor.
      */
     // <editor-fold defaultstate="collapsed"
-    // desc="Generated Code">//GEN-BEGIN:initComponents
+    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
     private void initComponents() {
         java.awt.GridBagConstraints gridBagConstraints;
 
@@ -120,7 +107,7 @@ public class FontSelectionDialog extends javax.swing.JDialog {
         okButton = new javax.swing.JButton();
         cancelButton = new javax.swing.JButton();
 
-        setTitle(OStrings.getString("TF_SELECT_FONTS_TITLE"));
+        setTitle(OStrings.getString("TF_SELECT_FONTS_TITLE")); // NOI18N
         setModal(true);
         addWindowListener(new java.awt.event.WindowAdapter() {
             public void windowClosing(java.awt.event.WindowEvent evt) {
@@ -143,7 +130,7 @@ public class FontSelectionDialog extends javax.swing.JDialog {
         gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
         getContentPane().add(fontComboBox, gridBagConstraints);
 
-        org.openide.awt.Mnemonics.setLocalizedText(fontLabel, OStrings.getString("TF_SELECT_SOURCE_FONT"));
+        org.openide.awt.Mnemonics.setLocalizedText(fontLabel, OStrings.getString("TF_SELECT_SOURCE_FONT")); // NOI18N
         gridBagConstraints = new java.awt.GridBagConstraints();
         gridBagConstraints.gridx = 0;
         gridBagConstraints.gridy = 0;
@@ -165,7 +152,7 @@ public class FontSelectionDialog extends javax.swing.JDialog {
         gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
         getContentPane().add(sizeSpinner, gridBagConstraints);
 
-        org.openide.awt.Mnemonics.setLocalizedText(sizeLabel, OStrings.getString("TF_SELECT_FONTSIZE"));
+        org.openide.awt.Mnemonics.setLocalizedText(sizeLabel, OStrings.getString("TF_SELECT_FONTSIZE")); // NOI18N
         gridBagConstraints = new java.awt.GridBagConstraints();
         gridBagConstraints.gridx = 0;
         gridBagConstraints.gridy = 1;
@@ -174,15 +161,12 @@ public class FontSelectionDialog extends javax.swing.JDialog {
         gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
         getContentPane().add(sizeLabel, gridBagConstraints);
 
-        previewTextArea.setBackground(javax.swing.UIManager.getDefaults().getColor("Label.background"));
         previewTextArea.setEditable(false);
         previewTextArea.setLineWrap(true);
-        previewTextArea.setText(OStrings.getString("TF_FONT_SAMPLE_TEXT"));
+        previewTextArea.setText(OStrings.getString("TF_FONT_SAMPLE_TEXT")); // NOI18N
         previewTextArea.setWrapStyleWord(true);
-        previewTextArea.setBorder(javax.swing.BorderFactory.createTitledBorder(null,
-                OStrings.getString("TF_FONT_SAMPLE_TITLE"),
-                javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
-                javax.swing.border.TitledBorder.DEFAULT_POSITION, fontLabel.getFont()));
+        previewTextArea.setBorder(javax.swing.BorderFactory.createTitledBorder(null, OStrings.getString("TF_FONT_SAMPLE_TITLE"), javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, fontLabel.getFont())); // NOI18N
+        previewTextArea.setOpaque(false);
         previewTextArea.setPreferredSize(new java.awt.Dimension(116, 100));
         gridBagConstraints = new java.awt.GridBagConstraints();
         gridBagConstraints.gridx = 0;
@@ -196,13 +180,7 @@ public class FontSelectionDialog extends javax.swing.JDialog {
 
         jPanel1.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));
 
-        org.openide.awt.Mnemonics.setLocalizedText(applyToProjectFilesCheckBox,
-                OStrings.getString("TF_APPLY_TO_PROJECT_FILES"));
-        applyToProjectFilesCheckBox.addActionListener(new java.awt.event.ActionListener() {
-            public void actionPerformed(java.awt.event.ActionEvent evt) {
-                applyToProjectFilesCheckBoxActionPerformed(evt);
-            }
-        });
+        org.openide.awt.Mnemonics.setLocalizedText(applyToProjectFilesCheckBox, OStrings.getString("TF_APPLY_TO_PROJECT_FILES")); // NOI18N
         jPanel1.add(applyToProjectFilesCheckBox);
 
         gridBagConstraints = new java.awt.GridBagConstraints();
@@ -213,7 +191,7 @@ public class FontSelectionDialog extends javax.swing.JDialog {
 
         buttonPanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));
 
-        org.openide.awt.Mnemonics.setLocalizedText(okButton, OStrings.getString("BUTTON_OK"));
+        org.openide.awt.Mnemonics.setLocalizedText(okButton, OStrings.getString("BUTTON_OK")); // NOI18N
         okButton.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                 okButtonActionPerformed(evt);
@@ -221,7 +199,7 @@ public class FontSelectionDialog extends javax.swing.JDialog {
         });
         buttonPanel.add(okButton);
 
-        org.openide.awt.Mnemonics.setLocalizedText(cancelButton, OStrings.getString("BUTTON_CANCEL"));
+        org.openide.awt.Mnemonics.setLocalizedText(cancelButton, OStrings.getString("BUTTON_CANCEL")); // NOI18N
         cancelButton.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                 cancelButtonActionPerformed(evt);
@@ -239,47 +217,35 @@ public class FontSelectionDialog extends javax.swing.JDialog {
         getContentPane().add(buttonPanel, gridBagConstraints);
 
         pack();
-        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
-        java.awt.Dimension dialogSize = getSize();
-        setLocation((screenSize.width - dialogSize.width) / 2, (screenSize.height - dialogSize.height) / 2);
     }// </editor-fold>//GEN-END:initComponents
 
-    private void fontComboBoxActionPerformed(java.awt.event.ActionEvent evt)// GEN-FIRST:event_fontComboBoxActionPerformed
-    {// GEN-HEADEREND:event_fontComboBoxActionPerformed
+    private void fontComboBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_fontComboBoxActionPerformed
         previewTextArea.setFont(getSelectedFont());
-    }// GEN-LAST:event_fontComboBoxActionPerformed
+    }//GEN-LAST:event_fontComboBoxActionPerformed
 
-    private void sizeSpinnerStateChanged(javax.swing.event.ChangeEvent evt)// GEN-FIRST:event_sizeSpinnerStateChanged
-    {// GEN-HEADEREND:event_sizeSpinnerStateChanged
+    private void sizeSpinnerStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_sizeSpinnerStateChanged
         previewTextArea.setFont(getSelectedFont());
-    }// GEN-LAST:event_sizeSpinnerStateChanged
+    }//GEN-LAST:event_sizeSpinnerStateChanged
 
-    private void okButtonActionPerformed(java.awt.event.ActionEvent evt)// GEN-FIRST:event_okButtonActionPerformed
-    {
-        if ((!getSelectedFont().equals(oldFont))
-                || (applyToProjectFilesCheckBox.isSelected() != Preferences
-                        .isPreference(Preferences.PROJECT_FILES_USE_FONT))) {
-            Preferences.setPreference(Preferences.PROJECT_FILES_USE_FONT,
-                    applyToProjectFilesCheckBox.isSelected());
+    private void okButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_okButtonActionPerformed
+        boolean applyToProjFiles = applyToProjectFilesCheckBox.isSelected();
+        if (!getSelectedFont().equals(oldFont)
+                || applyToProjFiles != Preferences.isPreference(Preferences.PROJECT_FILES_USE_FONT)) {
+            Preferences.setPreference(Preferences.PROJECT_FILES_USE_FONT, applyToProjFiles);
             doClose(RET_OK_CHANGED);
-        } else
+        } else {
             doClose(RET_CANCEL_OR_UNCHANGED);
-    }// GEN-LAST:event_okButtonActionPerformed
+        }
+    }//GEN-LAST:event_okButtonActionPerformed
 
-    private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt)// GEN-FIRST:event_cancelButtonActionPerformed
-    {
+    private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancelButtonActionPerformed
         doClose(RET_CANCEL_OR_UNCHANGED);
-    }// GEN-LAST:event_cancelButtonActionPerformed
+    }//GEN-LAST:event_cancelButtonActionPerformed
 
     /** Closes the dialog */
-    private void closeDialog(java.awt.event.WindowEvent evt)// GEN-FIRST:event_closeDialog
-    {
+    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
         doClose(RET_CANCEL_OR_UNCHANGED);
-    }// GEN-LAST:event_closeDialog
-
-    private void applyToProjectFilesCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_applyToProjectFilesCheckBoxActionPerformed
-    // TODO add your handling code here:
-    }// GEN-LAST:event_applyToProjectFilesCheckBoxActionPerformed
+    }//GEN-LAST:event_closeDialog
 
     private void doClose(int retStatus) {
         returnStatus = retStatus;
