@@ -7,19 +7,20 @@
                Home page: http://www.omegat.org/
                Support center: http://groups.yahoo.com/group/OmegaT/
 
- This program is free software; you can redistribute it and/or modify
+ This file is part of OmegaT.
+
+ OmegaT is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
+ OmegaT is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
 package org.omegat.gui.filters2;
@@ -30,19 +31,12 @@ import gen.core.filters.Filter;
 import java.awt.Dialog;
 import java.awt.Dimension;
 import java.awt.Toolkit;
-import java.awt.event.ActionEvent;
-import java.awt.event.KeyEvent;
-import java.util.Vector;
 
-import javax.swing.AbstractAction;
-import javax.swing.Action;
 import javax.swing.DefaultCellEditor;
 import javax.swing.JComboBox;
-import javax.swing.JComponent;
 import javax.swing.JDialog;
 import javax.swing.JLabel;
 import javax.swing.JOptionPane;
-import javax.swing.KeyStroke;
 import javax.swing.ListSelectionModel;
 import javax.swing.event.ListSelectionEvent;
 import javax.swing.event.ListSelectionListener;
@@ -52,8 +46,9 @@ import org.omegat.filters2.IFilter;
 import org.omegat.filters2.master.FilterMaster;
 import org.omegat.filters2.master.OneFilterTableModel;
 import org.omegat.util.OStrings;
-import org.omegat.util.StaticUtils;
 import org.omegat.util.StringUtil;
+import org.omegat.util.gui.DockingUI;
+import org.omegat.util.gui.StaticUIUtils;
 
 /**
  * Editor for a single filter. Filter is a class that allows for reading and
@@ -61,6 +56,7 @@ import org.omegat.util.StringUtil;
  * 
  * @author Maxym Mykhalchuk
  */
+@SuppressWarnings("serial")
 public class FilterEditor extends JDialog implements ListSelectionListener {
 
     private Filter filter;
@@ -70,32 +66,22 @@ public class FilterEditor extends JDialog implements ListSelectionListener {
     /** Creates new form SingleFilterEditor */
     public FilterEditor(Dialog parent, Filter filter) {
         super(parent, true);
-        this.filter = FilterMaster.getInstance().cloneFilter(filter);
+        this.filter = FilterMaster.cloneFilter(filter);
 
-        // HP
-        // Handle escape key to close the window
-        KeyStroke escape = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0, false);
-        Action escapeAction = new AbstractAction() {
-            public void actionPerformed(ActionEvent e) {
-                dispose();
-            }
-        };
-        getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(escape, "ESCAPE");
-        getRootPane().getActionMap().put("ESCAPE", escapeAction);
-        // END HP
+        StaticUIUtils.setEscapeClosable(this);
 
         initComponents();
 
-        IFilter f = FilterMaster.getInstance().getFilterInstance(filter.getClassName());
+        IFilter f = FilterMaster.getFilterInstance(filter.getClassName());
         fileFormatTextField.setText(f.getFileFormatName());
-        if (!StringUtil.isEmpty(f.getHint()))
+        if (!StringUtil.isEmpty(f.getHint())) {
             hintTextArea.setText(f.getHint());
-        else
+        } else {
             hintTextArea.setVisible(false);
+        }
 
         getRootPane().setDefaultButton(okButton);
 
-        instances.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
         instances.getSelectionModel().addListSelectionListener(this);
 
         TableColumn sourceEnc = instances.getColumnModel().getColumn(1);
@@ -107,18 +93,19 @@ public class FilterEditor extends JDialog implements ListSelectionListener {
         // accomodating table dimensions
         Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
         int width = instancesScrollPane.getPreferredSize().width + 200;
-        if (width > screenSize.width)
+        if (width > screenSize.width) {
             width = screenSize.width - addButton.getWidth() - 50;
+        }
         instancesScrollPane.setPreferredSize(new Dimension(width, instances.getPreferredSize().height + 70));
         pack();
-        Dimension dialogSize = getSize();
-        setLocation((screenSize.width - dialogSize.width) / 2, (screenSize.height - dialogSize.height) / 2);
+        DockingUI.displayCentered(this);
     }
 
     private JComboBox encodingComboBox() {
-        return new JComboBox(new Vector<String>(FilterMaster.getSupportedEncodings()));
+        return new JComboBox(FilterMaster.getSupportedEncodings().toArray());
     }
 
+    @Override
     public void valueChanged(ListSelectionEvent e) {
         if (e.getValueIsAdjusting())
             return;
@@ -137,8 +124,7 @@ public class FilterEditor extends JDialog implements ListSelectionListener {
      * WARNING: Do NOT modify this code. The content of this method is always
      * regenerated by the Form Editor.
      */
-    // <editor-fold defaultstate="collapsed"
-    // desc=" Generated Code ">//GEN-BEGIN:initComponents
+    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
     private void initComponents() {
         java.awt.GridBagConstraints gridBagConstraints;
 
@@ -158,47 +144,41 @@ public class FilterEditor extends JDialog implements ListSelectionListener {
         addButton = new javax.swing.JButton();
         hintTextArea = new javax.swing.JTextArea();
 
-        getContentPane().setLayout(new java.awt.GridBagLayout());
-
-        setTitle(OStrings.getString("FILTEREDITOR_Edit_a_single_file_filter"));
-        setResizable(false);
+        setTitle(OStrings.getString("FILTEREDITOR_Edit_a_single_file_filter")); // NOI18N
         addWindowListener(new java.awt.event.WindowAdapter() {
             public void windowClosing(java.awt.event.WindowEvent evt) {
                 closeDialog(evt);
             }
         });
+        getContentPane().setLayout(new java.awt.GridBagLayout());
 
         buttonPanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));
 
-        org.openide.awt.Mnemonics
-                .setLocalizedText(toDefaultsButton, OStrings.getString("BUTTON_TO_DEFAULTS"));
+        org.openide.awt.Mnemonics.setLocalizedText(toDefaultsButton, OStrings.getString("BUTTON_TO_DEFAULTS")); // NOI18N
         toDefaultsButton.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                 toDefaultsButtonActionPerformed(evt);
             }
         });
-
         buttonPanel.add(toDefaultsButton);
 
         jLabel1.setPreferredSize(new java.awt.Dimension(20, 0));
         buttonPanel.add(jLabel1);
 
-        org.openide.awt.Mnemonics.setLocalizedText(okButton, OStrings.getString("BUTTON_OK"));
+        org.openide.awt.Mnemonics.setLocalizedText(okButton, OStrings.getString("BUTTON_OK")); // NOI18N
         okButton.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                 okButtonActionPerformed(evt);
             }
         });
-
         buttonPanel.add(okButton);
 
-        org.openide.awt.Mnemonics.setLocalizedText(cancelButton, OStrings.getString("BUTTON_CANCEL"));
+        org.openide.awt.Mnemonics.setLocalizedText(cancelButton, OStrings.getString("BUTTON_CANCEL")); // NOI18N
         cancelButton.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                 cancelButtonActionPerformed(evt);
             }
         });
-
         buttonPanel.add(cancelButton);
 
         gridBagConstraints = new java.awt.GridBagConstraints();
@@ -209,8 +189,7 @@ public class FilterEditor extends JDialog implements ListSelectionListener {
         gridBagConstraints.insets = new java.awt.Insets(3, 3, 3, 3);
         getContentPane().add(buttonPanel, gridBagConstraints);
 
-        org.openide.awt.Mnemonics.setLocalizedText(fileFormatLabel,
-                OStrings.getString("FILTEREDITOR_File_Format"));
+        org.openide.awt.Mnemonics.setLocalizedText(fileFormatLabel, OStrings.getString("FILTEREDITOR_File_Format")); // NOI18N
         gridBagConstraints = new java.awt.GridBagConstraints();
         gridBagConstraints.gridx = 0;
         gridBagConstraints.gridy = 2;
@@ -229,12 +208,12 @@ public class FilterEditor extends JDialog implements ListSelectionListener {
         gridBagConstraints.insets = new java.awt.Insets(3, 3, 3, 3);
         getContentPane().add(fileFormatTextField, gridBagConstraints);
 
-        descTextArea.setBackground(javax.swing.UIManager.getDefaults().getColor("Label.background"));
         descTextArea.setEditable(false);
         descTextArea.setFont(new JLabel().getFont());
         descTextArea.setLineWrap(true);
-        descTextArea.setText(OStrings.getString("FILTEREDITOR_DESC"));
+        descTextArea.setText(OStrings.getString("FILTEREDITOR_DESC")); // NOI18N
         descTextArea.setWrapStyleWord(true);
+        descTextArea.setOpaque(false);
         gridBagConstraints = new java.awt.GridBagConstraints();
         gridBagConstraints.gridx = 0;
         gridBagConstraints.gridy = 0;
@@ -247,6 +226,7 @@ public class FilterEditor extends JDialog implements ListSelectionListener {
         jPanel3.setLayout(new java.awt.GridBagLayout());
 
         instances.setModel(new OneFilterTableModel(filter));
+        instances.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
         instancesScrollPane.setViewportView(instances);
 
         gridBagConstraints = new java.awt.GridBagConstraints();
@@ -259,14 +239,13 @@ public class FilterEditor extends JDialog implements ListSelectionListener {
         gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
         jPanel3.add(instancesScrollPane, gridBagConstraints);
 
-        org.openide.awt.Mnemonics.setLocalizedText(removeButton, OStrings.getString("BUTTON_REMOVE"));
+        org.openide.awt.Mnemonics.setLocalizedText(removeButton, OStrings.getString("BUTTON_REMOVE")); // NOI18N
         removeButton.setEnabled(false);
         removeButton.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                 removeButtonActionPerformed(evt);
             }
         });
-
         gridBagConstraints = new java.awt.GridBagConstraints();
         gridBagConstraints.gridx = 1;
         gridBagConstraints.gridy = 2;
@@ -275,14 +254,13 @@ public class FilterEditor extends JDialog implements ListSelectionListener {
         gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
         jPanel3.add(removeButton, gridBagConstraints);
 
-        org.openide.awt.Mnemonics.setLocalizedText(editButton, OStrings.getString("BUTTON_EDIT"));
+        org.openide.awt.Mnemonics.setLocalizedText(editButton, OStrings.getString("BUTTON_EDIT")); // NOI18N
         editButton.setEnabled(false);
         editButton.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                 editButtonActionPerformed(evt);
             }
         });
-
         gridBagConstraints = new java.awt.GridBagConstraints();
         gridBagConstraints.gridx = 1;
         gridBagConstraints.gridy = 1;
@@ -291,13 +269,12 @@ public class FilterEditor extends JDialog implements ListSelectionListener {
         gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
         jPanel3.add(editButton, gridBagConstraints);
 
-        org.openide.awt.Mnemonics.setLocalizedText(addButton, OStrings.getString("BUTTON_ADD"));
+        org.openide.awt.Mnemonics.setLocalizedText(addButton, OStrings.getString("BUTTON_ADD")); // NOI18N
         addButton.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                 addButtonActionPerformed(evt);
             }
         });
-
         gridBagConstraints = new java.awt.GridBagConstraints();
         gridBagConstraints.gridx = 1;
         gridBagConstraints.gridy = 0;
@@ -316,11 +293,11 @@ public class FilterEditor extends JDialog implements ListSelectionListener {
         gridBagConstraints.insets = new java.awt.Insets(3, 3, 3, 3);
         getContentPane().add(jPanel3, gridBagConstraints);
 
-        hintTextArea.setBackground(javax.swing.UIManager.getDefaults().getColor("Label.background"));
         hintTextArea.setEditable(false);
         hintTextArea.setFont(new JLabel().getFont());
         hintTextArea.setLineWrap(true);
         hintTextArea.setWrapStyleWord(true);
+        hintTextArea.setOpaque(false);
         gridBagConstraints = new java.awt.GridBagConstraints();
         gridBagConstraints.gridx = 0;
         gridBagConstraints.gridy = 1;
@@ -329,41 +306,34 @@ public class FilterEditor extends JDialog implements ListSelectionListener {
         gridBagConstraints.weightx = 1.0;
         gridBagConstraints.insets = new java.awt.Insets(3, 3, 3, 3);
         getContentPane().add(hintTextArea, gridBagConstraints);
+    }// </editor-fold>//GEN-END:initComponents
 
-        pack();
-    }
-
-    // </editor-fold>//GEN-END:initComponents
-
-    private void removeButtonActionPerformed(java.awt.event.ActionEvent evt)// GEN-FIRST:event_removeButtonActionPerformed
-    {// GEN-HEADEREND:event_removeButtonActionPerformed
+    private void removeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_removeButtonActionPerformed
         int row = instances.getSelectedRow();
         Files instance = filter.getFiles().get(row);
-        if (JOptionPane.YES_OPTION == JOptionPane.showConfirmDialog(this, StaticUtils.format(
-                OStrings.getString("FILTEREDITOR_really_delete_filter_instance"),
-                new Object[] { instance.getSourceFilenameMask() }), OStrings
-                .getString("FILTEREDITOR_Confirm_deletion_TITLE"), JOptionPane.YES_NO_OPTION,
+        if (JOptionPane.YES_OPTION == JOptionPane.showConfirmDialog(this,
+                StringUtil.format(OStrings.getString("FILTEREDITOR_really_delete_filter_instance"),
+                        instance.getSourceFilenameMask()),
+                OStrings.getString("FILTEREDITOR_Confirm_deletion_TITLE"), JOptionPane.YES_NO_OPTION,
                 JOptionPane.QUESTION_MESSAGE)) {
             filter.getFiles().remove(row);
             instances.setModel(new OneFilterTableModel(filter));
         }
-    }// GEN-LAST:event_removeButtonActionPerformed
+    }//GEN-LAST:event_removeButtonActionPerformed
 
-    private void toDefaultsButtonActionPerformed(java.awt.event.ActionEvent evt)// GEN-FIRST:event_toDefaultsButtonActionPerformed
-    {// GEN-HEADEREND:event_toDefaultsButtonActionPerformed
+    private void toDefaultsButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_toDefaultsButtonActionPerformed
         try {
-            filter = FilterMaster.getInstance().getDefaultSettingsFromFilter(filter.getClassName());
+            filter = FilterMaster.getDefaultSettingsFromFilter(filter.getClassName());
             instances.setModel(new OneFilterTableModel(filter));
         } catch (Exception e) {
             JOptionPane.showMessageDialog(this,
                     OStrings.getString("FILTEREDITOR_ERROR_Reverting_To_Def") + e,
                     OStrings.getString("ERROR_TITLE"), JOptionPane.ERROR_MESSAGE);
         }
-    }// GEN-LAST:event_toDefaultsButtonActionPerformed
+    }//GEN-LAST:event_toDefaultsButtonActionPerformed
 
-    private void addButtonActionPerformed(java.awt.event.ActionEvent evt)// GEN-FIRST:event_addButtonActionPerformed
-    {// GEN-HEADEREND:event_addButtonActionPerformed
-        IFilter f = FilterMaster.getInstance().getFilterInstance(filter.getClassName());
+    private void addButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_addButtonActionPerformed
+        IFilter f = FilterMaster.getFilterInstance(filter.getClassName());
 
         InstanceEditor ie = new InstanceEditor(this, f.isSourceEncodingVariable(),
                 f.isTargetEncodingVariable(), f.getHint());
@@ -377,13 +347,12 @@ public class FilterEditor extends JDialog implements ListSelectionListener {
             filter.getFiles().add(ff);
             instances.setModel(new OneFilterTableModel(filter));
         }
-    }// GEN-LAST:event_addButtonActionPerformed
+    }//GEN-LAST:event_addButtonActionPerformed
 
-    private void editButtonActionPerformed(java.awt.event.ActionEvent evt)// GEN-FIRST:event_editButtonActionPerformed
-    {// GEN-HEADEREND:event_editButtonActionPerformed
+    private void editButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_editButtonActionPerformed
         int row = instances.getSelectedRow();
 
-        IFilter f = FilterMaster.getInstance().getFilterInstance(filter.getClassName());
+        IFilter f = FilterMaster.getFilterInstance(filter.getClassName());
         InstanceEditor ie = new InstanceEditor(this, f.isSourceEncodingVariable(),
                 f.isTargetEncodingVariable(), f.getHint(),
                 instances.getModel().getValueAt(row, 0).toString(), instances.getModel().getValueAt(row, 1)
@@ -399,20 +368,20 @@ public class FilterEditor extends JDialog implements ListSelectionListener {
             filter.getFiles().set(row, ff);
             instances.setModel(new OneFilterTableModel(filter));
         }
-    }// GEN-LAST:event_editButtonActionPerformed
+    }//GEN-LAST:event_editButtonActionPerformed
 
-    private void okButtonActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_okButtonActionPerformed
+    private void okButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_okButtonActionPerformed
         doClose(filter);
-    }// GEN-LAST:event_okButtonActionPerformed
+    }//GEN-LAST:event_okButtonActionPerformed
 
-    private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt) {// GEN-FIRST:event_cancelButtonActionPerformed
+    private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancelButtonActionPerformed
         doClose(null);
-    }// GEN-LAST:event_cancelButtonActionPerformed
+    }//GEN-LAST:event_cancelButtonActionPerformed
 
     /** Closes the dialog */
-    private void closeDialog(java.awt.event.WindowEvent evt) {// GEN-FIRST:event_closeDialog
+    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
         doClose(null);
-    }// GEN-LAST:event_closeDialog
+    }//GEN-LAST:event_closeDialog
 
     private void doClose(Filter r) {
         result = r;
