@@ -7,22 +7,26 @@
                          Benjamin Siband, and Kim Bruning
                2007 Zoltan Bartko
                2008 Andrzej Sawula, Alex Buloichik, Didier Briel
+               2013 Yu Tang, Aaron Madlon-Kay
+               2014 Piotr Kulik
+               2015 Yu Tang, Aaron Madlon-Kay
                Home page: http://www.omegat.org/
                Support center: http://groups.yahoo.com/group/OmegaT/
 
- This program is free software; you can redistribute it and/or modify
+ This file is part of OmegaT.
+
+ OmegaT is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
+ OmegaT is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
 package org.omegat.gui.main;
@@ -31,37 +35,48 @@ import java.awt.BorderLayout;
 import java.awt.Container;
 import java.awt.Font;
 import java.awt.Frame;
+import java.awt.HeadlessException;
+import java.awt.Window;
+import java.awt.event.ActionEvent;
+import java.awt.event.ActionListener;
 import java.awt.event.WindowAdapter;
 import java.awt.event.WindowEvent;
 import java.io.File;
 import java.io.IOException;
 import java.util.ArrayList;
-import java.util.HashSet;
+import java.util.Collections;
 import java.util.List;
-import java.util.Set;
 
 import javax.swing.JFileChooser;
 import javax.swing.JFrame;
 import javax.swing.JLabel;
 import javax.swing.JOptionPane;
+import javax.swing.SwingUtilities;
+import javax.swing.Timer;
 import javax.swing.WindowConstants;
+import javax.swing.text.JTextComponent;
 
 import org.omegat.core.Core;
 import org.omegat.core.CoreEvents;
+import org.omegat.core.data.ExternalTMX;
 import org.omegat.core.events.IApplicationEventListener;
 import org.omegat.core.events.IProjectEventListener;
 import org.omegat.core.matching.NearString;
-import org.omegat.gui.filelist.ProjectFrame;
-import org.omegat.gui.search.SearchWindow;
-import org.omegat.util.LFileCopy;
+import org.omegat.gui.common.OmegaTIcons;
+import org.omegat.gui.dialogs.FileCollisionDialog;
+import org.omegat.gui.filelist.ProjectFilesListController;
+import org.omegat.gui.matches.IMatcher;
+import org.omegat.gui.search.SearchWindowController;
+import org.omegat.util.FileUtil;
+import org.omegat.util.FileUtil.ICollisionCallback;
+import org.omegat.util.Log;
 import org.omegat.util.OConsts;
 import org.omegat.util.OStrings;
 import org.omegat.util.Preferences;
-import org.omegat.util.StaticUtils;
+import org.omegat.util.StringUtil;
 import org.omegat.util.WikiGet;
 import org.omegat.util.gui.DockingUI;
 import org.omegat.util.gui.OmegaTFileChooser;
-import org.omegat.util.gui.ResourcesUtil;
 import org.omegat.util.gui.UIThreadsUtil;
 
 import com.vlsolutions.swing.docking.Dockable;
@@ -81,11 +96,15 @@ import com.vlsolutions.swing.docking.FloatingDialog;
  * @author Zoltan Bartko - bartkozoltan@bartkozoltan.com
  * @author Andrzej Sawula
  * @author Alex Buloichik (alex73mail@gmail.com)
+ * @author Yu Tang
+ * @author Aaron Madlon-Kay
+ * @author Piotr Kulik
  */
+@SuppressWarnings("serial")
 public class MainWindow extends JFrame implements IMainWindow {
     public final MainWindowMenu menu;
 
-    protected ProjectFrame m_projWin;
+    protected ProjectFilesListController m_projWin;
 
     /**
      * The font for main window (source and target text) and for match and
@@ -94,7 +113,7 @@ public class MainWindow extends JFrame implements IMainWindow {
     private Font m_font;
 
     /** Set of all open search windows. */
-    private final Set<SearchWindow> m_searches = new HashSet<SearchWindow>();
+    private final List<SearchWindowController> m_searches = new ArrayList<SearchWindowController>();
 
     protected JLabel lengthLabel;
     protected JLabel progressLabel;
@@ -107,8 +126,6 @@ public class MainWindow extends JFrame implements IMainWindow {
         menu = new MainWindowMenu(this, new MainWindowMenuHandler(this));
 
         setJMenuBar(menu.initComponents());
-        getContentPane().add(MainWindowUI.createStatusBar(this), BorderLayout.SOUTH);
-        pack();
 
         setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
 
@@ -116,6 +133,11 @@ public class MainWindow extends JFrame implements IMainWindow {
             public void windowClosing(WindowEvent e) {
                 menu.mainWindowMenuHandler.projectExitMenuItemActionPerformed();
             }
+            @Override
+            public void windowDeactivated(WindowEvent we) {
+                Core.getEditor().windowDeactivated();
+                super.windowDeactivated(we);
+            }
         });
 
         // load default font from preferences
@@ -127,8 +149,10 @@ public class MainWindow extends JFrame implements IMainWindow {
         MainWindowUI.createMainComponents(this, m_font);
 
         getContentPane().add(MainWindowUI.initDocking(this), BorderLayout.CENTER);
-
-        setIconImage(ResourcesUtil.getIcon("/org/omegat/gui/resources/OmegaT_small.gif").getImage());
+        pack();
+        getContentPane().add(MainWindowUI.createStatusBar(this), BorderLayout.SOUTH);
+        
+        OmegaTIcons.setIconImages(this);
 
         CoreEvents.registerProjectChangeListener(new IProjectEventListener() {
             public void onProjectChanged(PROJECT_CHANGE_TYPE eventType) {
@@ -141,8 +165,7 @@ public class MainWindow extends JFrame implements IMainWindow {
 
         CoreEvents.registerApplicationEventListener(new IApplicationEventListener() {
             public void onApplicationStartup() {
-                MainWindowUI.resetDesktopLayout(MainWindow.this);
-                MainWindowUI.loadScreenLayout(MainWindow.this);
+                MainWindowUI.initializeScreenLayout(MainWindow.this);
 
                 DockingUI.removeUnusedMenuSeparators(menu.getOptionsMenu().getPopupMenu());
             }
@@ -151,6 +174,8 @@ public class MainWindow extends JFrame implements IMainWindow {
             }
         });
 
+        MainWindowUI.handlePerProjectLayouts(this);
+        
         updateTitle();
     }
 
@@ -200,42 +225,83 @@ public class MainWindow extends JFrame implements IMainWindow {
      * Sets the title of the main window appropriately
      */
     private void updateTitle() {
-        String s = OStrings.getDisplayVersion();
+        String s = OStrings.getDisplayNameAndVersion();
         if (Core.getProject().isProjectLoaded()) {
             s += " :: " + Core.getProject().getProjectProperties().getProjectName();
         }
         setTitle(s);
     }
 
-    /** insert current fuzzy match at cursor position */
+    /** insert current fuzzy match or selection at cursor position */
     public void doInsertTrans() {
-        if (!Core.getProject().isProjectLoaded())
+        if (!Core.getProject().isProjectLoaded()) {
             return;
+        }
 
-        NearString near = Core.getMatcher().getActiveMatch();
-        if (near != null) {
-            Core.getEditor().insertText(near.translation);
+        String text = getSelectedTextInMatcher();
+        if (StringUtil.isEmpty(text)) {
+            NearString near = Core.getMatcher().getActiveMatch();
+            if (near != null) {
+                text = near.translation;
+            }
+        }
+        if (!StringUtil.isEmpty(text)) {
+            Core.getEditor().insertText(text);
+            Core.getEditor().requestFocus();
         }
     }
 
-    /** replace entire edit area with active fuzzy match */
+    /** replace entire edit area with active fuzzy match or selection */
     public void doRecycleTrans() {
-        if (!Core.getProject().isProjectLoaded())
+        if (!Core.getProject().isProjectLoaded()) {
             return;
+        }
+
+        String selection = getSelectedTextInMatcher();
+        if (!StringUtil.isEmpty(selection)) {
+            Core.getEditor().replaceEditText(selection);
+            Core.getEditor().requestFocus();
+            return;
+        }
 
         NearString near = Core.getMatcher().getActiveMatch();
         if (near != null) {
-            Core.getEditor().replaceEditText(near.translation);
+            String translation = near.translation;
+            if (Preferences.isPreference(Preferences.CONVERT_NUMBERS)) {
+                translation = Core.getMatcher().substituteNumbers(Core.getEditor().getCurrentEntry().getSrcText(),
+                        near.source, near.translation);
+            }
+            if (near.comesFrom == NearString.MATCH_SOURCE.TM
+                    && ExternalTMX.isInPath(new File(Core.getProject().getProjectProperties().getTMRoot(), "mt"),
+                            new File(near.projs[0]))) {
+                Core.getEditor().replaceEditTextAndMark(translation);
+            } else {
+                Core.getEditor().replaceEditText(translation);
+            }
+            Core.getEditor().requestFocus();
         }
     }
 
-    protected void addSearchWindow(SearchWindow newSearchWindow) {
+    private String getSelectedTextInMatcher() {
+        IMatcher matcher = Core.getMatcher();
+        return matcher instanceof JTextComponent
+                ? ((JTextComponent) matcher).getSelectedText()
+                : null;
+    }
+
+    protected void addSearchWindow(final SearchWindowController newSearchWindow) {
+        newSearchWindow.addWindowListener(new WindowAdapter() {
+            @Override
+            public void windowClosed(WindowEvent e) {
+                removeSearchWindow(newSearchWindow);
+            }
+        });
         synchronized (m_searches) {
             m_searches.add(newSearchWindow);
         }
     }
 
-    public void removeSearchWindow(SearchWindow searchWindow) {
+    private void removeSearchWindow(SearchWindowController searchWindow) {
         synchronized (m_searches) {
             m_searches.remove(searchWindow);
         }
@@ -244,20 +310,21 @@ public class MainWindow extends JFrame implements IMainWindow {
     private void closeSearchWindows() {
         synchronized (m_searches) {
             // dispose other windows
-            for (SearchWindow sw : m_searches) {
+            for (SearchWindowController sw : m_searches) {
                 sw.dispose();
             }
             m_searches.clear();
         }
     }
 
+    protected List<SearchWindowController> getSearchWindows() {
+        return Collections.unmodifiableList(m_searches);
+    }
+
     /**
      * Imports the file/files/folder into project's source files.
-     * 
-     * @author Kim Bruning
-     * @author Maxym Mykhalchuk
      */
-    public void doImportSourceFiles() {
+    public void doPromptImportSourceFiles() {
         OmegaTFileChooser chooser = new OmegaTFileChooser();
         chooser.setMultiSelectionEnabled(true);
         chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
@@ -265,47 +332,74 @@ public class MainWindow extends JFrame implements IMainWindow {
 
         int result = chooser.showOpenDialog(this);
         if (result == OmegaTFileChooser.APPROVE_OPTION) {
-            String projectsource = Core.getProject().getProjectProperties().getSourceRoot();
-            File sourcedir = new File(projectsource);
             File[] selFiles = chooser.getSelectedFiles();
-            try {
-                for (int i = 0; i < selFiles.length; i++) {
-                    File selSrc = selFiles[i];
-                    if (selSrc.isDirectory()) {
-                        List<String> files = new ArrayList<String>();
-                        StaticUtils.buildFileList(files, selSrc, true);
-                        String selSourceParent = selSrc.getParent();
-                        for (String filename : files) {
-                            String midName = filename.substring(selSourceParent.length());
-                            File src = new File(filename);
-                            File dest = new File(sourcedir, midName);
-                            LFileCopy.copy(src, dest);
-                        }
-                    } else {
-                        File dest = new File(sourcedir, selFiles[i].getName());
-                        LFileCopy.copy(selSrc, dest);
-                    }
-                }
-                ProjectUICommands.projectReload();
-            } catch (IOException ioe) {
-                displayErrorRB(ioe, "MAIN_ERROR_File_Import_Failed");
-            }
+            importFiles(Core.getProject().getProjectProperties().getSourceRoot(), selFiles);
         }
     }
+    
+    public void importFiles(String destination, File[] toImport) {
+        importFiles(destination, toImport, true);
+    }
+    
+    public void importFiles(String destination, File[] toImport, boolean doReload) {
+        try {
+            FileUtil.copyFilesTo(new File(destination), toImport, new CollisionCallback());
+            if (doReload) {
+                ProjectUICommands.projectReload();
+            }
+        } catch (IOException ioe) {
+            displayErrorRB(ioe, "MAIN_ERROR_File_Import_Failed");
+        }
+    }
+    
+    private class CollisionCallback implements ICollisionCallback {
+        private boolean isCanceled = false;
+        private boolean yesToAll = false;
+        
+        @Override
+        public boolean shouldReplace(File file, int index, int total) {
+            if (isCanceled) {
+                return false;
+            }
+            if (yesToAll) {
+                return true;
+            }
+            FileCollisionDialog dialog = new FileCollisionDialog(MainWindow.this);
+            dialog.setFilename(file.getName());
+            dialog.enableApplyToAll(total - index > 1);
+            dialog.pack();
+            dialog.setVisible(true);
+            isCanceled = dialog.userDidCancel();
+            if (isCanceled) {
+                return false;
+            }
+            yesToAll = dialog.isApplyToAll() && dialog.shouldReplace();
+            return yesToAll || dialog.shouldReplace();
+        }
+        
+        @Override
+        public boolean isCanceled() {
+            return isCanceled;
+        }
+    };
 
     /**
      * Does wikiread
-     * 
-     * @author Kim Bruning
      */
     public void doWikiImport() {
         String remote_url = JOptionPane.showInputDialog(this, OStrings.getString("TF_WIKI_IMPORT_PROMPT"),
                 OStrings.getString("TF_WIKI_IMPORT_TITLE"), JOptionPane.OK_CANCEL_OPTION);
         String projectsource = Core.getProject().getProjectProperties().getSourceRoot();
-        // [1762625] Only try to get MediaWiki page if a string has been entered
-        if ((remote_url != null) && (remote_url.trim().length() > 0)) {
+        if (remote_url == null || remote_url.trim().isEmpty()) {
+            // [1762625] Only try to get MediaWiki page if a string has been entered
+            return;
+        }
+        try {
             WikiGet.doWikiGet(remote_url, projectsource);
             ProjectUICommands.projectReload();
+        } catch (Exception ex) {
+            Log.log(ex);
+            displayErrorRB(ex, "TF_WIKI_IMPORT_FAILED");
         }
     }
 
@@ -313,22 +407,58 @@ public class MainWindow extends JFrame implements IMainWindow {
      * {@inheritDoc}
      */
     public void showStatusMessageRB(final String messageKey, final Object... params) {
-        final String msg;
-        if (messageKey == null) {
-            msg = new String() + ' ';
-        } else {
-            if (params != null) {
-                msg = StaticUtils.format(OStrings.getString(messageKey), params);
-            } else {
-                msg = OStrings.getString(messageKey);
-            }
-        }
+        final String msg = getLocalizedString(messageKey, params);
         UIThreadsUtil.executeInSwingThread(new Runnable() {
+            @Override
             public void run() {
                 statusLabel.setText(msg);
             }
         });
     }
+    
+    private String getLocalizedString(String messageKey, Object... params) {
+        if (messageKey == null) {
+            return " ";
+        } else if (params == null) {
+            return OStrings.getString(messageKey);
+        } else {
+            return StringUtil.format(OStrings.getString(messageKey), params);
+        }
+    }
+
+    /**
+     * Same as {@link #showStatusMessageRB(String, Object...)} but 
+     * this will clear the message after ten seconds.
+     * 
+     * @param messageKey
+     *            message key in resource bundle
+     * @param params
+     *            message parameters for formatting
+     */
+    public void showTimedStatusMessageRB(String messageKey, Object... params) {
+        showStatusMessageRB(messageKey, params);
+
+        if (messageKey == null) {
+            return;
+        }
+
+        // clear the message after 10 seconds
+        final String localizedString = getLocalizedString(messageKey, params);
+        ActionListener clearStatus = new ActionListener() {
+            @Override
+            public void actionPerformed(ActionEvent evt) {
+                String text = statusLabel.getText();
+                if (localizedString.equals(text)) {
+                    statusLabel.setText(null);
+                }
+            }
+        };
+
+        final int DELAY = 10000; // milliseconds
+        final Timer timer = new Timer(DELAY, clearStatus);
+        timer.setRepeats(false);  // one-time only
+        timer.start();
+    }
 
     /**
      * Show message in progress bar.
@@ -340,6 +470,15 @@ public class MainWindow extends JFrame implements IMainWindow {
         progressLabel.setText(messageText);
     }
 
+    /* Set progress bar tooltip text.
+     * 
+     * @param tooltipText
+     *            tooltip text
+     */
+    public void setProgressToolTipText(String toolTipText) {
+        progressLabel.setToolTipText(toolTipText);
+    }
+
     /**
      * Show message in length label.
      * 
@@ -353,23 +492,43 @@ public class MainWindow extends JFrame implements IMainWindow {
     // /////////////////////////////////////////////////////////////
     // /////////////////////////////////////////////////////////////
     // display oriented code
+    
+    private JLabel lastDialogText;
+    private String lastDialogKey;
 
     /**
-     * Displays a warning message.
-     * 
-     * @param msg
-     *            the message to show
-     * @param e
-     *            exception occurred. may be null
+     * {@inheritDoc}
      */
-    public void displayWarning(final String msg, final Throwable e) {
+    public void displayWarningRB(String warningKey, Object... params) {
+        displayWarningRB(warningKey, null, params);
+    };
+    
+    /**
+     * {@inheritDoc}
+     */
+    public void displayWarningRB(final String warningKey, final String supercedesKey, final Object... params) {
         UIThreadsUtil.executeInSwingThread(new Runnable() {
             public void run() {
+                String msg;
+                if (params != null) {
+                    msg = StringUtil.format(OStrings.getString(warningKey), params);
+                } else {
+                    msg = OStrings.getString(warningKey);
+                }
+                
+                if (supercedesKey != null && lastDialogText != null && supercedesKey.equals(lastDialogKey)) {
+                    Window w = SwingUtilities.getWindowAncestor(lastDialogText);
+                    if (w != null) {
+                        w.dispose();
+                    }
+                }
+                
+                lastDialogText = new JLabel(msg);
+                lastDialogKey = warningKey;
+
                 statusLabel.setText(msg);
-                String fulltext = msg;
-                if (e != null)
-                    fulltext += "\n" + e.toString();
-                JOptionPane.showMessageDialog(MainWindow.this, fulltext, OStrings.getString("TF_WARNING"),
+                
+                JOptionPane.showMessageDialog(MainWindow.this, lastDialogText, OStrings.getString("TF_WARNING"),
                         JOptionPane.WARNING_MESSAGE);
             }
         });
@@ -383,7 +542,7 @@ public class MainWindow extends JFrame implements IMainWindow {
             public void run() {
                 String msg;
                 if (params != null) {
-                    msg = StaticUtils.format(OStrings.getString(errorKey), params);
+                    msg = StringUtil.format(OStrings.getString(errorKey), params);
                 } else {
                     msg = OStrings.getString(errorKey);
                 }
@@ -453,10 +612,25 @@ public class MainWindow extends JFrame implements IMainWindow {
     /**
      * {@inheritDoc}
      */
-    public void showErrorDialogRB(String message, Object[] args, String title) {
+    public void showErrorDialogRB(String title, String message, Object... args) {
 
         JOptionPane.showMessageDialog(this.getApplicationFrame(),
-                StaticUtils.format(OStrings.getString(message), args), OStrings.getString(title),
+                StringUtil.format(OStrings.getString(message), args), OStrings.getString(title),
                 JOptionPane.ERROR_MESSAGE);
     }
+
+    /**
+     * {@inheritDoc}
+     * 
+     * @see JOptionPane#showConfirmDialog(java.awt.Component, Object, String,
+     *      int, int)
+     */
+    public int showConfirmDialog(Object message, String title, int optionType,
+            int messageType) throws HeadlessException {
+        return JOptionPane.showConfirmDialog(this, message, title, optionType, messageType);
+    }
+
+    public void showMessageDialog(String message) {
+        JOptionPane.showMessageDialog(this, message);
+    }
 }
