@@ -7,37 +7,33 @@
                Home page: http://www.omegat.org/
                Support center: http://groups.yahoo.com/group/OmegaT/
 
- This program is free software; you can redistribute it and/or modify
+ This file is part of OmegaT.
+
+ OmegaT is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
+ OmegaT is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
 package org.omegat.gui.segmentation;
 
 import java.awt.Dialog;
-import java.awt.Dimension;
 import java.awt.Frame;
 import java.awt.event.ActionEvent;
-import java.awt.event.KeyEvent;
 import java.beans.ExceptionListener;
 
 import javax.swing.AbstractAction;
-import javax.swing.Action;
-import javax.swing.JComponent;
 import javax.swing.JDialog;
 import javax.swing.JLabel;
 import javax.swing.JOptionPane;
-import javax.swing.KeyStroke;
 import javax.swing.ListSelectionModel;
 import javax.swing.event.ListSelectionEvent;
 import javax.swing.event.ListSelectionListener;
@@ -48,7 +44,9 @@ import org.omegat.core.segmentation.SRX;
 import org.omegat.core.segmentation.datamodels.MappingRulesModel;
 import org.omegat.core.segmentation.datamodels.SegmentationRulesModel;
 import org.omegat.util.OStrings;
-import org.omegat.util.StaticUtils;
+import org.omegat.util.StringUtil;
+import org.omegat.util.gui.DockingUI;
+import org.omegat.util.gui.StaticUIUtils;
 
 /**
  * Main dialog for for setting up sentence segmenting. The dialog is created as
@@ -56,27 +54,42 @@ import org.omegat.util.StaticUtils;
  * 
  * @author Maxym Mykhalchuk
  */
+@SuppressWarnings("serial")
 public class SegmentationCustomizer extends JDialog implements ListSelectionListener {
     /** A return status code - returned if Cancel button has been pressed */
     public static final int RET_CANCEL = 0;
     /** A return status code - returned if OK button has been pressed */
     public static final int RET_OK = 1;
+    
+    /** SRX from OmegaT. */
+    private SRX defaultSRX;
+    /** SRX from user preferences. */
+    private SRX userSRX;
+    /** SRX from current project. */
+    private SRX projectSRX;
+    /** SRX which editable now. */
+    private SRX editableSRX;
 
-    private void constructor() {
-        // HP
-        // Handle escape key to close the window
-        KeyStroke escape = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0, false);
-        Action escapeAction = new AbstractAction() {
+    /**
+     * Flag if this customizer shows project specific segmentation rules or not
+     */
+    private boolean isProjectSpecific;
+
+    private void constructor(boolean projectSpecific, SRX defaultSRX, SRX userSRX, SRX projectSRX) {
+        this.isProjectSpecific = projectSpecific;
+        this.defaultSRX = defaultSRX;
+        this.userSRX = userSRX;
+        this.projectSRX = projectSRX;
+        this.editableSRX = isProjectSpecific && projectSRX != null ? projectSRX.clone() : userSRX.clone();
+        
+        StaticUIUtils.setEscapeAction(this, new AbstractAction() {
+            @Override
             public void actionPerformed(ActionEvent e) {
                 cancelButtonActionPerformed(null);
             }
-        };
-        getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(escape, "ESCAPE");
-        getRootPane().getActionMap().put("ESCAPE", escapeAction);
-        // END HP
+        });
 
         initComponents();
-        hintTextArea.setBackground(javax.swing.UIManager.getDefaults().getColor("Label.background"));
 
         getRootPane().setDefaultButton(okButton);
 
@@ -85,38 +98,84 @@ public class SegmentationCustomizer extends JDialog implements ListSelectionList
 
         MappingRulesModel model = (MappingRulesModel) mapTable.getModel();
         model.addExceptionListener(new ExceptionListener() {
+            @Override
             public void exceptionThrown(Exception e) {
                 mapErrorsLabel.setText(e.getLocalizedMessage());
             }
         });
+        
+        if (isProjectSpecific) {
+            setTitle(OStrings.getString("GUI_SEGMENTATION_TITLE_PROJECTSPECIFIC")); // NOI18N
+        } else {
+            projectSpecificCB.setVisible(false);
+        }
+        if (projectSpecific && projectSRX == null) {
+            mapTable.setEnabled(false);
+            mapTable.setFocusable(false);
+            mapInsertButton.setEnabled(false);
+            toDefaultsButton.setEnabled(false);
+        } else {
+            if (projectSpecific) projectSpecificCB.setSelected(true);
+            mapTable.setEnabled(true);
+            mapTable.setFocusable(true);
+            mapInsertButton.setEnabled(true);
+            toDefaultsButton.setEnabled(true);
+        }
 
         pack();
         setSize(getWidth() * 5 / 4, getHeight() * 5 / 4);
-        Dimension screen = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
-        setLocation((screen.width - getWidth()) / 2, (screen.height - getHeight()) / 2);
+        DockingUI.displayCentered(this);
     }
 
     /**
      * Creates new form SegmentationCustomizer (to be called from a frame)
      */
-    public SegmentationCustomizer(Frame parent) {
+    public SegmentationCustomizer(Frame parent, boolean projectSpecific, SRX defaultSRX, SRX userSRX, SRX projectSRX) {
         super(parent, true);
-        constructor();
+        constructor(projectSpecific, defaultSRX, userSRX, projectSRX);
     }
 
     /**
      * Creates new form SegmentationCustomizer (to be called from a dialog)
      */
-    public SegmentationCustomizer(Dialog parent) {
+    public SegmentationCustomizer(Dialog parent, boolean projectSpecific, SRX defaultSRX, SRX userSRX, SRX projectSRX) {
         super(parent, true);
-        constructor();
+        constructor(projectSpecific, defaultSRX, userSRX, projectSRX);
     }
 
     /** @return the return status of this dialog - one of RET_OK or RET_CANCEL */
     public int getReturnStatus() {
         return returnStatus;
     }
+    
+    /**
+     * Returns the SRX that was edited, so it can be used. If project-specific 
+     * segmentation rules are requested, and user has not checked 
+     * 'enable project specific segmentation', then null is returned.
+     * @return
+     */
+    public SRX getSRX() {
+        return editableSRX;
+    }
 
+    public boolean isProjectSpecific() {
+        return projectSpecificCB.isSelected();
+    }
+
+    protected void setEditableSRX(SRX srx) {
+        editableSRX = srx.clone();
+        MappingRulesModel model = new MappingRulesModel(editableSRX);
+        mapTable.setModel(model);
+        model.addExceptionListener(new ExceptionListener() {
+            @Override
+            public void exceptionThrown(Exception e) {
+                mapErrorsLabel.setText(e.getLocalizedMessage());
+            }
+        });
+        ruleTable.setModel(new DefaultTableModel());
+    }
+
+    @Override
     public void valueChanged(ListSelectionEvent e) {
         if (e.getValueIsAdjusting())
             return;
@@ -146,10 +205,11 @@ public class SegmentationCustomizer extends JDialog implements ListSelectionList
                 else
                     mapDownButton.setEnabled(false);
 
-                MapRule maprule = SRX.getSRX().getMappingRules().get(selrow);
+                MapRule maprule = this.editableSRX.getMappingRules().get(selrow);
                 SegmentationRulesModel model = new SegmentationRulesModel(maprule.getRules());
                 ruleTable.setModel(model);
                 model.addExceptionListener(new ExceptionListener() {
+                    @Override
                     public void exceptionThrown(Exception e) {
                         ruleErrorsLabel.setText(e.getLocalizedMessage());
                     }
@@ -194,8 +254,7 @@ public class SegmentationCustomizer extends JDialog implements ListSelectionList
      * WARNING: Do NOT modify this code. The content of this method is always
      * regenerated by the Form Editor.
      */
-    // <editor-fold defaultstate="collapsed"
-    // desc=" Generated Code ">//GEN-BEGIN:initComponents
+    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
     private void initComponents() {
         java.awt.GridBagConstraints gridBagConstraints;
 
@@ -213,6 +272,7 @@ public class SegmentationCustomizer extends JDialog implements ListSelectionList
         mapDownButton = new javax.swing.JButton();
         hintTextArea = new javax.swing.JTextArea();
         mapErrorsLabel = new javax.swing.JLabel();
+        projectSpecificCB = new javax.swing.JCheckBox();
         rulePanel = new javax.swing.JPanel();
         ruleScrollPane = new javax.swing.JScrollPane();
         ruleTable = new javax.swing.JTable();
@@ -222,46 +282,41 @@ public class SegmentationCustomizer extends JDialog implements ListSelectionList
         ruleDownButton = new javax.swing.JButton();
         ruleErrorsLabel = new javax.swing.JLabel();
 
-        getContentPane().setLayout(new java.awt.GridBagLayout());
-
-        setTitle(OStrings.getString("GUI_SEGMENTATION_TITLE"));
+        setTitle(OStrings.getString("GUI_SEGMENTATION_TITLE")); // NOI18N
         addWindowListener(new java.awt.event.WindowAdapter() {
             public void windowClosing(java.awt.event.WindowEvent evt) {
                 closeDialog(evt);
             }
         });
+        getContentPane().setLayout(new java.awt.GridBagLayout());
 
         buttonPanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));
 
-        org.openide.awt.Mnemonics
-                .setLocalizedText(toDefaultsButton, OStrings.getString("BUTTON_TO_DEFAULTS"));
+        org.openide.awt.Mnemonics.setLocalizedText(toDefaultsButton, OStrings.getString("BUTTON_TO_DEFAULTS")); // NOI18N
         toDefaultsButton.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                 toDefaultsButtonActionPerformed(evt);
             }
         });
-
         buttonPanel.add(toDefaultsButton);
 
         jLabel1.setPreferredSize(new java.awt.Dimension(20, 0));
         buttonPanel.add(jLabel1);
 
-        org.openide.awt.Mnemonics.setLocalizedText(okButton, OStrings.getString("BUTTON_OK"));
+        org.openide.awt.Mnemonics.setLocalizedText(okButton, OStrings.getString("BUTTON_OK")); // NOI18N
         okButton.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                 okButtonActionPerformed(evt);
             }
         });
-
         buttonPanel.add(okButton);
 
-        org.openide.awt.Mnemonics.setLocalizedText(cancelButton, OStrings.getString("BUTTON_CANCEL"));
+        org.openide.awt.Mnemonics.setLocalizedText(cancelButton, OStrings.getString("BUTTON_CANCEL")); // NOI18N
         cancelButton.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                 cancelButtonActionPerformed(evt);
             }
         });
-
         buttonPanel.add(cancelButton);
 
         gridBagConstraints = new java.awt.GridBagConstraints();
@@ -272,17 +327,17 @@ public class SegmentationCustomizer extends JDialog implements ListSelectionList
         gridBagConstraints.weightx = 1.0;
         getContentPane().add(buttonPanel, gridBagConstraints);
 
+        mapPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(OStrings.getString("GUI_SEGMENTATION_RULESETS"))); // NOI18N
         mapPanel.setLayout(new java.awt.GridBagLayout());
 
-        mapPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(OStrings
-                .getString("GUI_SEGMENTATION_RULESETS")));
         mapScrollPane.setPreferredSize(new java.awt.Dimension(300, 100));
-        mapTable.setModel(new MappingRulesModel(SRX.getSRX()));
+
+        mapTable.setModel(new MappingRulesModel(editableSRX));
         mapScrollPane.setViewportView(mapTable);
 
         gridBagConstraints = new java.awt.GridBagConstraints();
         gridBagConstraints.gridx = 0;
-        gridBagConstraints.gridy = 1;
+        gridBagConstraints.gridy = 3;
         gridBagConstraints.gridheight = 4;
         gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
         gridBagConstraints.weightx = 1.0;
@@ -290,66 +345,60 @@ public class SegmentationCustomizer extends JDialog implements ListSelectionList
         gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
         mapPanel.add(mapScrollPane, gridBagConstraints);
 
-        org.openide.awt.Mnemonics.setLocalizedText(mapUpButton,
-                OStrings.getString("GUI_SEGMENTATION_BUTTON_UP_1"));
+        org.openide.awt.Mnemonics.setLocalizedText(mapUpButton, OStrings.getString("GUI_SEGMENTATION_BUTTON_UP_1")); // NOI18N
         mapUpButton.setEnabled(false);
         mapUpButton.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                 mapUpButtonActionPerformed(evt);
             }
         });
-
         gridBagConstraints = new java.awt.GridBagConstraints();
         gridBagConstraints.gridx = 1;
-        gridBagConstraints.gridy = 3;
+        gridBagConstraints.gridy = 5;
         gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
         gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTH;
         gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
         mapPanel.add(mapUpButton, gridBagConstraints);
 
-        org.openide.awt.Mnemonics.setLocalizedText(mapDeleteButton, OStrings.getString("BUTTON_REMOVE"));
+        org.openide.awt.Mnemonics.setLocalizedText(mapDeleteButton, OStrings.getString("BUTTON_REMOVE")); // NOI18N
         mapDeleteButton.setEnabled(false);
         mapDeleteButton.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                 mapDeleteButtonActionPerformed(evt);
             }
         });
-
         gridBagConstraints = new java.awt.GridBagConstraints();
         gridBagConstraints.gridx = 1;
-        gridBagConstraints.gridy = 2;
+        gridBagConstraints.gridy = 4;
         gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
         gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTH;
         gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
         mapPanel.add(mapDeleteButton, gridBagConstraints);
 
-        org.openide.awt.Mnemonics.setLocalizedText(mapInsertButton, OStrings.getString("BUTTON_ADD_NODOTS"));
+        org.openide.awt.Mnemonics.setLocalizedText(mapInsertButton, OStrings.getString("BUTTON_ADD_NODOTS")); // NOI18N
         mapInsertButton.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                 mapInsertButtonActionPerformed(evt);
             }
         });
-
         gridBagConstraints = new java.awt.GridBagConstraints();
         gridBagConstraints.gridx = 1;
-        gridBagConstraints.gridy = 1;
+        gridBagConstraints.gridy = 3;
         gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
         gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTH;
         gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
         mapPanel.add(mapInsertButton, gridBagConstraints);
 
-        org.openide.awt.Mnemonics.setLocalizedText(mapDownButton,
-                OStrings.getString("GUI_SEGMENTATION_BUTTON_DOWN_1"));
+        org.openide.awt.Mnemonics.setLocalizedText(mapDownButton, OStrings.getString("GUI_SEGMENTATION_BUTTON_DOWN_1")); // NOI18N
         mapDownButton.setEnabled(false);
         mapDownButton.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                 mapDownButtonActionPerformed(evt);
             }
         });
-
         gridBagConstraints = new java.awt.GridBagConstraints();
         gridBagConstraints.gridx = 1;
-        gridBagConstraints.gridy = 4;
+        gridBagConstraints.gridy = 6;
         gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
         gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTH;
         gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
@@ -358,8 +407,9 @@ public class SegmentationCustomizer extends JDialog implements ListSelectionList
         hintTextArea.setEditable(false);
         hintTextArea.setFont(new JLabel().getFont());
         hintTextArea.setLineWrap(true);
-        hintTextArea.setText(OStrings.getString("GUI_SEGMENTATION_NOTE"));
+        hintTextArea.setText(OStrings.getString("GUI_SEGMENTATION_NOTE")); // NOI18N
         hintTextArea.setWrapStyleWord(true);
+        hintTextArea.setOpaque(false);
         gridBagConstraints = new java.awt.GridBagConstraints();
         gridBagConstraints.gridwidth = 2;
         gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
@@ -378,6 +428,20 @@ public class SegmentationCustomizer extends JDialog implements ListSelectionList
         gridBagConstraints.insets = new java.awt.Insets(0, 2, 0, 2);
         mapPanel.add(mapErrorsLabel, gridBagConstraints);
 
+        org.openide.awt.Mnemonics.setLocalizedText(projectSpecificCB, OStrings.getString("PP_CHECKBOX_PROJECT_SPECIFIC_SEGMENTATION_RULES")); // NOI18N
+        projectSpecificCB.addActionListener(new java.awt.event.ActionListener() {
+            public void actionPerformed(java.awt.event.ActionEvent evt) {
+                projectSpecificCBActionPerformed(evt);
+            }
+        });
+        gridBagConstraints = new java.awt.GridBagConstraints();
+        gridBagConstraints.gridx = 0;
+        gridBagConstraints.gridy = 2;
+        gridBagConstraints.gridwidth = 2;
+        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
+        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
+        mapPanel.add(projectSpecificCB, gridBagConstraints);
+
         gridBagConstraints = new java.awt.GridBagConstraints();
         gridBagConstraints.gridx = 0;
         gridBagConstraints.gridy = 0;
@@ -386,10 +450,9 @@ public class SegmentationCustomizer extends JDialog implements ListSelectionList
         gridBagConstraints.weighty = 0.5;
         getContentPane().add(mapPanel, gridBagConstraints);
 
+        rulePanel.setBorder(javax.swing.BorderFactory.createTitledBorder(OStrings.getString("GUI_SEGMENTATION_RULEORDER"))); // NOI18N
         rulePanel.setLayout(new java.awt.GridBagLayout());
 
-        rulePanel.setBorder(javax.swing.BorderFactory.createTitledBorder(OStrings
-                .getString("GUI_SEGMENTATION_RULEORDER")));
         ruleScrollPane.setPreferredSize(new java.awt.Dimension(300, 120));
         ruleScrollPane.setViewportView(ruleTable);
 
@@ -403,15 +466,13 @@ public class SegmentationCustomizer extends JDialog implements ListSelectionList
         gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
         rulePanel.add(ruleScrollPane, gridBagConstraints);
 
-        org.openide.awt.Mnemonics.setLocalizedText(ruleUpButton,
-                OStrings.getString("GUI_SEGMENTATION_BUTTON_UP_2"));
+        org.openide.awt.Mnemonics.setLocalizedText(ruleUpButton, OStrings.getString("GUI_SEGMENTATION_BUTTON_UP_2")); // NOI18N
         ruleUpButton.setEnabled(false);
         ruleUpButton.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                 ruleUpButtonActionPerformed(evt);
             }
         });
-
         gridBagConstraints = new java.awt.GridBagConstraints();
         gridBagConstraints.gridx = 1;
         gridBagConstraints.gridy = 3;
@@ -420,14 +481,13 @@ public class SegmentationCustomizer extends JDialog implements ListSelectionList
         gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
         rulePanel.add(ruleUpButton, gridBagConstraints);
 
-        org.openide.awt.Mnemonics.setLocalizedText(ruleDeleteButton, OStrings.getString("BUTTON_REMOVE_2"));
+        org.openide.awt.Mnemonics.setLocalizedText(ruleDeleteButton, OStrings.getString("BUTTON_REMOVE_2")); // NOI18N
         ruleDeleteButton.setEnabled(false);
         ruleDeleteButton.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                 ruleDeleteButtonActionPerformed(evt);
             }
         });
-
         gridBagConstraints = new java.awt.GridBagConstraints();
         gridBagConstraints.gridx = 1;
         gridBagConstraints.gridy = 2;
@@ -436,15 +496,13 @@ public class SegmentationCustomizer extends JDialog implements ListSelectionList
         gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
         rulePanel.add(ruleDeleteButton, gridBagConstraints);
 
-        org.openide.awt.Mnemonics
-                .setLocalizedText(ruleInsertButton, OStrings.getString("BUTTON_ADD_NODOTS2"));
+        org.openide.awt.Mnemonics.setLocalizedText(ruleInsertButton, OStrings.getString("BUTTON_ADD_NODOTS2")); // NOI18N
         ruleInsertButton.setEnabled(false);
         ruleInsertButton.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                 ruleInsertButtonActionPerformed(evt);
             }
         });
-
         gridBagConstraints = new java.awt.GridBagConstraints();
         gridBagConstraints.gridx = 1;
         gridBagConstraints.gridy = 1;
@@ -453,15 +511,13 @@ public class SegmentationCustomizer extends JDialog implements ListSelectionList
         gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
         rulePanel.add(ruleInsertButton, gridBagConstraints);
 
-        org.openide.awt.Mnemonics.setLocalizedText(ruleDownButton,
-                OStrings.getString("GUI_SEGMENTATION_BUTTON_DOWN_2"));
+        org.openide.awt.Mnemonics.setLocalizedText(ruleDownButton, OStrings.getString("GUI_SEGMENTATION_BUTTON_DOWN_2")); // NOI18N
         ruleDownButton.setEnabled(false);
         ruleDownButton.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(java.awt.event.ActionEvent evt) {
                 ruleDownButtonActionPerformed(evt);
             }
         });
-
         gridBagConstraints = new java.awt.GridBagConstraints();
         gridBagConstraints.gridx = 1;
         gridBagConstraints.gridy = 4;
@@ -487,114 +543,110 @@ public class SegmentationCustomizer extends JDialog implements ListSelectionList
         gridBagConstraints.weightx = 1.0;
         gridBagConstraints.weighty = 1.0;
         getContentPane().add(rulePanel, gridBagConstraints);
-
     }// </editor-fold>//GEN-END:initComponents
 
-    private void toDefaultsButtonActionPerformed(java.awt.event.ActionEvent evt)// GEN-FIRST:event_toDefaultsButtonActionPerformed
-    {// GEN-HEADEREND:event_toDefaultsButtonActionPerformed
-        commitTableEdits();
-        SRX.init();
-        MappingRulesModel model = new MappingRulesModel(SRX.getSRX());
-        mapTable.setModel(model);
-        model.addExceptionListener(new ExceptionListener() {
-            public void exceptionThrown(Exception e) {
-                mapErrorsLabel.setText(e.getLocalizedMessage());
-            }
-        });
-        ruleTable.setModel(new DefaultTableModel());
-    }// GEN-LAST:event_toDefaultsButtonActionPerformed
+    private void projectSpecificCBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_projectSpecificCBActionPerformed
+        if (projectSpecificCB.isSelected()) {
+            mapTable.setEnabled(true);
+            mapTable.setFocusable(true);
+            mapInsertButton.setEnabled(true);
+            toDefaultsButton.setEnabled(true);
+            setEditableSRX(editableSRX);
+        } else {
+            mapTable.setEnabled(false);
+            mapTable.setFocusable(false);
+            mapInsertButton.setEnabled(false);
+            toDefaultsButton.setEnabled(false);
+            setEditableSRX(userSRX);
+        }
+    }//GEN-LAST:event_projectSpecificCBActionPerformed
 
-    private void ruleDownButtonActionPerformed(java.awt.event.ActionEvent evt)// GEN-FIRST:event_ruleDownButtonActionPerformed
-    {// GEN-HEADEREND:event_ruleDownButtonActionPerformed
+    private void toDefaultsButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_toDefaultsButtonActionPerformed
+        commitTableEdits();
+        setEditableSRX(defaultSRX);
+    }//GEN-LAST:event_toDefaultsButtonActionPerformed
+
+    private void ruleDownButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_ruleDownButtonActionPerformed
         commitTableEdits();
         SegmentationRulesModel model = (SegmentationRulesModel) ruleTable.getModel();
         int selrow = ruleTable.getSelectedRow();
         model.moveRowDown(selrow);
         ruleTable.getSelectionModel().clearSelection();
         ruleTable.getSelectionModel().addSelectionInterval(selrow + 1, selrow + 1);
-    }// GEN-LAST:event_ruleDownButtonActionPerformed
+    }//GEN-LAST:event_ruleDownButtonActionPerformed
 
-    private void ruleUpButtonActionPerformed(java.awt.event.ActionEvent evt)// GEN-FIRST:event_ruleUpButtonActionPerformed
-    {// GEN-HEADEREND:event_ruleUpButtonActionPerformed
+    private void ruleUpButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_ruleUpButtonActionPerformed
         commitTableEdits();
         SegmentationRulesModel model = (SegmentationRulesModel) ruleTable.getModel();
         int selrow = ruleTable.getSelectedRow();
         model.moveRowUp(selrow);
         ruleTable.getSelectionModel().clearSelection();
         ruleTable.getSelectionModel().addSelectionInterval(selrow - 1, selrow - 1);
-    }// GEN-LAST:event_ruleUpButtonActionPerformed
+    }//GEN-LAST:event_ruleUpButtonActionPerformed
 
-    private void ruleDeleteButtonActionPerformed(java.awt.event.ActionEvent evt)// GEN-FIRST:event_ruleDeleteButtonActionPerformed
-    {// GEN-HEADEREND:event_ruleDeleteButtonActionPerformed
+    private void ruleDeleteButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_ruleDeleteButtonActionPerformed
         commitTableEdits();
         SegmentationRulesModel model = (SegmentationRulesModel) ruleTable.getModel();
         model.removeRow(ruleTable.getSelectedRow());
-    }// GEN-LAST:event_ruleDeleteButtonActionPerformed
+    }//GEN-LAST:event_ruleDeleteButtonActionPerformed
 
-    private void mapDownButtonActionPerformed(java.awt.event.ActionEvent evt)// GEN-FIRST:event_mapDownButtonActionPerformed
-    {// GEN-HEADEREND:event_mapDownButtonActionPerformed
+    private void mapDownButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mapDownButtonActionPerformed
         commitTableEdits();
         MappingRulesModel model = (MappingRulesModel) mapTable.getModel();
         int selrow = mapTable.getSelectedRow();
         model.moveRowDown(selrow);
         mapTable.getSelectionModel().clearSelection();
         mapTable.getSelectionModel().addSelectionInterval(selrow + 1, selrow + 1);
-    }// GEN-LAST:event_mapDownButtonActionPerformed
+    }//GEN-LAST:event_mapDownButtonActionPerformed
 
-    private void mapUpButtonActionPerformed(java.awt.event.ActionEvent evt)// GEN-FIRST:event_mapUpButtonActionPerformed
-    {// GEN-HEADEREND:event_mapUpButtonActionPerformed
+    private void mapUpButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mapUpButtonActionPerformed
         commitTableEdits();
         MappingRulesModel model = (MappingRulesModel) mapTable.getModel();
         int selrow = mapTable.getSelectedRow();
         model.moveRowUp(selrow);
         mapTable.getSelectionModel().clearSelection();
         mapTable.getSelectionModel().addSelectionInterval(selrow - 1, selrow - 1);
-    }// GEN-LAST:event_mapUpButtonActionPerformed
+    }//GEN-LAST:event_mapUpButtonActionPerformed
 
-    private void mapDeleteButtonActionPerformed(java.awt.event.ActionEvent evt)// GEN-FIRST:event_mapDeleteButtonActionPerformed
-    {// GEN-HEADEREND:event_mapDeleteButtonActionPerformed
+    private void mapDeleteButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mapDeleteButtonActionPerformed
         commitTableEdits();
         MappingRulesModel model = (MappingRulesModel) mapTable.getModel();
         String set = model.getValueAt(mapTable.getSelectedRow(), 0).toString();
         String title = OStrings.getString("CONFIRM_DIALOG_TITLE");
-        String message = StaticUtils.format(OStrings.getString("SEG_CONFIRM_REMOVE_SENTSEG_SET"),
-                new Object[] { set });
+        String message = StringUtil.format(OStrings.getString("SEG_CONFIRM_REMOVE_SENTSEG_SET"), set);
         if (JOptionPane.showConfirmDialog(this, message, title, JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION)
             model.removeRow(mapTable.getSelectedRow());
-    }// GEN-LAST:event_mapDeleteButtonActionPerformed
+    }//GEN-LAST:event_mapDeleteButtonActionPerformed
 
-    private void ruleInsertButtonActionPerformed(java.awt.event.ActionEvent evt)// GEN-FIRST:event_ruleInsertButtonActionPerformed
-    {// GEN-HEADEREND:event_ruleInsertButtonActionPerformed
+    private void ruleInsertButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_ruleInsertButtonActionPerformed
         commitTableEdits();
         SegmentationRulesModel model = (SegmentationRulesModel) ruleTable.getModel();
         model.addRow();
-    }// GEN-LAST:event_ruleInsertButtonActionPerformed
+        ruleTable.changeSelection(ruleTable.getRowCount() - 1, 0, false, false);
+        ruleTable.changeSelection(ruleTable.getRowCount() - 1, ruleTable.getColumnCount() - 1, false, true);
+    }//GEN-LAST:event_ruleInsertButtonActionPerformed
 
-    private void mapInsertButtonActionPerformed(java.awt.event.ActionEvent evt)// GEN-FIRST:event_mapInsertButtonActionPerformed
-    {// GEN-HEADEREND:event_mapInsertButtonActionPerformed
+    private void mapInsertButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mapInsertButtonActionPerformed
         commitTableEdits();
         MappingRulesModel model = (MappingRulesModel) mapTable.getModel();
         model.addRow();
-    }// GEN-LAST:event_mapInsertButtonActionPerformed
+        mapTable.changeSelection(mapTable.getRowCount() - 1, 0, false, false);
+        mapTable.changeSelection(mapTable.getRowCount() - 1, mapTable.getColumnCount() - 1, false, true);
+    }//GEN-LAST:event_mapInsertButtonActionPerformed
 
-    private void okButtonActionPerformed(java.awt.event.ActionEvent evt)// GEN-FIRST:event_okButtonActionPerformed
-    {
+    private void okButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_okButtonActionPerformed
         commitTableEdits();
-        SRX.getSRX().save();
         doClose(RET_OK);
-    }// GEN-LAST:event_okButtonActionPerformed
+    }//GEN-LAST:event_okButtonActionPerformed
 
-    private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt)// GEN-FIRST:event_cancelButtonActionPerformed
-    {
-        SRX.reload();
+    private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancelButtonActionPerformed
         doClose(RET_CANCEL);
-    }// GEN-LAST:event_cancelButtonActionPerformed
+    }//GEN-LAST:event_cancelButtonActionPerformed
 
     /** Closes the dialog */
-    private void closeDialog(java.awt.event.WindowEvent evt)// GEN-FIRST:event_closeDialog
-    {
+    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
         doClose(RET_CANCEL);
-    }// GEN-LAST:event_closeDialog
+    }//GEN-LAST:event_closeDialog
 
     private void doClose(int retStatus) {
         returnStatus = retStatus;
@@ -616,6 +668,7 @@ public class SegmentationCustomizer extends JDialog implements ListSelectionList
     private javax.swing.JTable mapTable;
     private javax.swing.JButton mapUpButton;
     private javax.swing.JButton okButton;
+    private javax.swing.JCheckBox projectSpecificCB;
     private javax.swing.JButton ruleDeleteButton;
     private javax.swing.JButton ruleDownButton;
     private javax.swing.JLabel ruleErrorsLabel;
