@@ -6,22 +6,25 @@
  Copyright (C) 2000-2006 Keith Godfrey and Maxym Mykhalchuk
                2007 Didier Briel, Zoltan Bartko
                2010-2011 Didier Briel
+               2012 Guido Leenders
+               2016 Aaron Madlon-Kay
                Home page: http://www.omegat.org/
                Support center: http://groups.yahoo.com/group/OmegaT/
 
- This program is free software; you can redistribute it and/or modify
+ This file is part of OmegaT.
+
+ OmegaT is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
+ OmegaT is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
 package org.omegat.util;
@@ -33,25 +36,27 @@ import java.util.regex.Matcher;
  * This class is here, because the Locale has hard-coded '_' inside, and we must
  * adhere to ISO standard LL-CC.
  * <p>
- * This class tries to follow <a
- * href="http://www.lisa.org/standards/tmx/tmx.html#xml:lang">TMX Specification
- * on languages</a>, which is based on <a
- * href="http://www.ietf.org/rfc/rfc3066.txt">RFC 3066</a>, i.e.
+ * This class tries to follow
+ * <a href="http://www.lisa.org/standards/tmx/tmx.html#xml:lang">TMX
+ * Specification on languages</a>, which is based on
+ * <a href="http://www.ietf.org/rfc/rfc3066.txt">RFC 3066</a>, i.e.
  * <ul>
  * <li>Language is composed from 1-8 alpha (A-Za-z) chars, then "-", then 1-8
  * alpha/digit chars (A-Za-z0-9).
  * <li>Case insensitive
  * <li>Case is not altered by this class, even though there exist conventions
- * for capitalization ([ISO 3166] recommends that country codes are capitalized
- * (MN Mongolia), and [ISO 639] recommends that language codes are written in
+ * for capitalization (ISO 3166 recommends that country codes are capitalized
+ * (MN Mongolia), and ISO 639 recommends that language codes are written in
  * lower case (mn Mongolian)).
- * <ul>
+ * </ul>
  * 
  * @author Maxym Mykhalchuk
  * @author Didier Briel
  * @author Zoltan Bartko bartkozoltan@bartkozoltan.com
+ * @author Guido Leenders
+ * @author Aaron Madlon-Kay
  */
-public class Language {
+public class Language implements Comparable<Object> {
     private Locale locale = new Locale("");
     private String languageCode;
     private String countryCode;
@@ -72,7 +77,7 @@ public class Language {
      * The form xx-xxxx-xx is also accepted, where "xxxx" is a 4 alpha characters script as defined in
      * <a href="http://unicode.org/iso15924/iso15924-codes.html">ISO 15924</a>. E.g., sr-Latn-RS, 
      * which represents Serbian ('sr') written using Latin script ('Latn') as used in Serbia ('RS').
-     * This form is described in <a href="http://www.rfc-editor.org/rfc/bcp/bcp47.tx">BCP47</a>.
+     * This form is described in <a href="http://www.rfc-editor.org/rfc/bcp/bcp47.txt">BCP47</a>.
      */
     public Language(String str) {
         this.languageCode = "";
@@ -111,8 +116,9 @@ public class Language {
      */
     public String getLanguage() {
         String langstring = getLanguageCode();
-        if (langstring.length() > 0 && getCountryCode().length() > 0)
+        if (!langstring.isEmpty() && !getCountryCode().isEmpty()) {
             langstring += "-" + getCountryCode();
+        }
         return langstring;
     }
 
@@ -120,9 +126,9 @@ public class Language {
      * Returns a string representation as a Java locale (xx_YY).
      */
     public String getLocaleCode() {
-        if (locale == null)
+        if (locale == null) {
             return "";
-        else { 
+        } else { 
             // Patch Java locale, to return correct locales instead of obsolete codes
             String returnString = locale.toString();
             if (returnString.length()<2)
@@ -139,6 +145,13 @@ public class Language {
             
     }
 
+    /**
+     * Returns a string representation as a Microsoft locale (xx-yy).
+     */
+    public String getLocaleLCID() {
+        return getLocaleCode().toLowerCase().replace("_", "-");
+    }
+
     /**
      * returns the Java locale
      */
@@ -166,6 +179,18 @@ public class Language {
             return this.countryCode;
     }
 
+    /**
+     * Determine whether or not the language is space-delimited. Only Chinese,
+     * Japanese, and Tibetan are not space-delimited.
+     * 
+     * @see <a href="https://linguistlist.org/issues/6/6-1302.html">LINGUIST
+     *      List 6.1302</a>
+     */
+    public boolean isSpaceDelimited() {
+        return !"ZH".equalsIgnoreCase(this.languageCode) && !"JA".equalsIgnoreCase(this.languageCode)
+                && !"BO".equalsIgnoreCase(this.languageCode);
+    }
+
     // /////////////////////////////////////////////////////////////////////////
     // /////////////////////////////////////////////////////////////////////////
     // /////////////////////////////////////////////////////////////////////////
@@ -619,4 +644,11 @@ public class Language {
         return locale.hashCode();
     }
 
+	public int compareTo(Object o) {
+		if (o instanceof Language) {
+			return this.getLanguage().compareTo(((Language)o).getLanguage());
+		}
+		return this.getLanguage().compareTo(o.toString());
+	}
+
 }
