@@ -7,19 +7,20 @@
                 Home page: http://www.omegat.org/
                Support center: http://groups.yahoo.com/group/OmegaT/
 
- This program is free software; you can redistribute it and/or modify
+ This file is part of OmegaT.
+
+ OmegaT is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
+ OmegaT is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
 package org.omegat.util;
@@ -37,11 +38,7 @@ import java.io.Reader;
 public class LinebreakPreservingReader extends Reader {
 
     /** Reader for input, must be able to re-insert non-linebreak characters. */
-    private PushbackReader in;
-
-    /** Default constructor, hidden. */
-    private LinebreakPreservingReader() {
-    }
+    private final PushbackReader in;
 
     /** Buffer that contains the last linebreak */
     private StringBuffer linebreak = new StringBuffer(2);;
@@ -91,7 +88,7 @@ public class LinebreakPreservingReader extends Reader {
         linebreak.setLength(0);
 
         // Create a buffer to contain the result
-        StringBuffer line = new StringBuffer(1024);
+        StringBuilder line = new StringBuilder(1024);
 
         // Get the next character and check if it's not the end of the stream
         int chr = in.read();
@@ -110,16 +107,17 @@ public class LinebreakPreservingReader extends Reader {
             linebreak.append((char) chr);
 
             // If the linebreak character is \r, check if it's followed by \n
-            if (((char) chr) == '\r') {
+            if (chr == '\r') {
                 // Get the next character
                 chr = in.read();
 
                 // If the next character is \n, add it to the current linebreak,
                 // otherwise push it back into the input reader
-                if (((char) chr) == '\n')
+                if (chr == '\n') {
                     linebreak.append((char) chr);
-                else
+                } else {
                     in.unread(chr);
+                }
             }
         }
 
@@ -140,7 +138,7 @@ public class LinebreakPreservingReader extends Reader {
     }
 
     private final boolean isLinebreakCharacter(int chr) {
-        return (((char) chr) == '\n') || (((char) chr) == '\r');
+        return chr == '\n' || chr == '\r';
     }
 
     /* FOR TESTING ONLY */
