@@ -5,22 +5,24 @@
 
  Copyright (C) 2000-2006 Keith Godfrey, Maxym Mykhalchuk, and Henry Pijffers
                2008 Alex Buloichik
+               2013 Didier Briel
                Home page: http://www.omegat.org/
                Support center: http://groups.yahoo.com/group/OmegaT/
 
- This program is free software; you can redistribute it and/or modify
+ This file is part of OmegaT.
+
+ OmegaT is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
+ OmegaT is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
 package org.omegat.util;
@@ -39,6 +41,8 @@ import java.util.logging.LogManager;
 import java.util.logging.LogRecord;
 import java.util.logging.Logger;
 
+import org.omegat.util.logging.OmegaTFileHandler;
+
 /**
  * A collection of methods to make logging things easier.
  * 
@@ -50,7 +54,7 @@ public class Log {
     private static Logger LOGGER;
 
     static {
-        LOGGER = Logger.global;
+        LOGGER = Logger.getLogger("global");
 
         boolean loaded = false;
         File usersLogSettings = new File(StaticUtils.getConfigDir(), "logger.properties");
@@ -110,11 +114,11 @@ public class Log {
             for (String hn : hs) {
                 String word = hn.trim();
                 try {
-                    Class clz = Log.class.getClassLoader().loadClass(word);
+                    Class<?> clz = Log.class.getClassLoader().loadClass(word);
                     Handler h = (Handler) clz.newInstance();
                     String fname = props.getProperty(word + ".formatter");
                     if (fname != null) {
-                        Class clzF = Log.class.getClassLoader().loadClass(fname.trim());
+                        Class<?> clzF = Log.class.getClassLoader().loadClass(fname.trim());
                         h.setFormatter((Formatter) clzF.newInstance());
                     }
                     String level = props.getProperty(word + ".level");
@@ -136,6 +140,29 @@ public class Log {
     public static String getLogLocation() {
         return StaticUtils.getConfigDir() + "/logs";
     }
+    
+    /**
+     * Compute the filename of the log file
+     * @return the filename of the log, or an empty string
+     */
+    public static String getLogFileName() {
+        Handler[] hand = LOGGER.getParent().getHandlers();
+        if (hand[1] instanceof OmegaTFileHandler) {
+            OmegaTFileHandler omegatLog = (OmegaTFileHandler) hand[1];
+            return omegatLog.getOmegaTLogFileName()+ ".log";
+        } else {
+            return "";
+        }
+            
+    }
+    
+    /**
+     * Compute the full path of the log file
+     * @return the full path of the log file
+     */
+    public static String getLogFilePath() {
+        return getLogLocation() + "/" + getLogFileName();
+    }
 
     /**
      * Logs what otherwise would go to System.out
@@ -152,8 +179,6 @@ public class Log {
      * @param parameters
      *            Parameters for the message. These are inserted by using
      *            StaticUtils.format.
-     * 
-     * @author Henry Pijffers (henry.pijffers@saxnot.com)
      */
     public static void logRB(String key, Object... parameters) {
         if (LOGGER.isLoggable(Level.INFO)) {
@@ -173,8 +198,6 @@ public class Log {
      * 
      * @param throwable
      *            The exception or error to log
-     * 
-     * @author Henry Pijffers (henry.pijffers@saxnot.com)
      */
     public static void log(Throwable throwable) {
         LOGGER.log(Level.SEVERE, "", throwable);
@@ -182,19 +205,17 @@ public class Log {
 
     /**
      * Writes a warning message to the log (to be retrieved from the resource
-     * bundle), preceded by a line containing the warning ID.
-     * 
-     * While the warning message can be localised, the warning ID should not, so
-     * developers can determine what warning was given by looking at the warning
-     * ID, instead of trying to interpret localised messages.
-     * 
+     * bundle)
+     * <p>
+     * While the warning message can be localized, the warning key is also
+     * logged, so developers can determine what warning was given by looking at
+     * the error key, instead of trying to interpret localized messages.
+     *
      * @param key
-     *            The key of the warning message in the resource bundle
+     *            The key of the error message in the resource bundle
      * @param parameters
-     *            Parameters for the warning message. These are inserted by
-     *            using StaticUtils.format.
-     * 
-     * @author Henry Pijffers (henry.pijffers@saxnot.com)
+     *            Parameters for the error message. These are inserted by using
+     *            StaticUtils.format.
      */
     public static void logWarningRB(String key, Object... parameters) {
         if (LOGGER.isLoggable(Level.WARNING)) {
@@ -207,21 +228,18 @@ public class Log {
     }
 
     /**
-     * Writes the specified info message to the log, preceded by a line
-     * containing the info ID.
-     * 
-     * While the info message can be localised, the info ID should not, so
-     * developers can determine what info was given by looking at the info ID,
-     * instead of trying to interpret localised messages.
-     * 
-     * @param id
-     *            An identification string for the info
+     * Writes an info message to the log (to be retrieved from the resource
+     * bundle)
+     * <p>
+     * While the info message can be localized, the info key is also logged, so
+     * developers can determine what info was given by looking at the error key,
+     * instead of trying to interpret localized messages.
+     *
+     * @param key
+     *            The key of the error message in the resource bundle
      * @param parameters
-     *            Parameters for the info message. These are inserted by using
+     *            Parameters for the error message. These are inserted by using
      *            StaticUtils.format.
-     * 
-     * @author Henry Pijffers (henry.pijffers@saxnot.com)
-     * @author Alex Buloichik (alex73mail@gmail.com)
      */
     public static void logInfoRB(String id, Object... parameters) {
         if (LOGGER.isLoggable(Level.INFO)) {
@@ -230,24 +248,22 @@ public class Log {
             rec.setParameters(parameters);
             rec.setLoggerName(LOGGER.getName());
             LOGGER.log(rec);
+            }
         }
-    }
 
     /**
      * Writes an error message to the log (to be retrieved from the resource
-     * bundle), preceded by a line containing the error ID.
-     * 
-     * While the error message can be localised, the error ID should not, so
-     * developers can determine what error was given by looking at the error ID,
-     * instead of trying to interpret localised messages.
-     * 
-     * @param id
+     * bundle)
+     * <p>
+     * While the error message can be localized, the error key is also logged,
+     * so developers can determine what error was given by looking at the error
+     * key, instead of trying to interpret localized messages.
+     *
+     * @param key
      *            The key of the error message in the resource bundle
      * @param parameters
      *            Parameters for the error message. These are inserted by using
      *            StaticUtils.format.
-     * 
-     * @author Henry Pijffers (henry.pijffers@saxnot.com)
      */
     public static void logErrorRB(String key, Object... parameters) {
         if (LOGGER.isLoggable(Level.SEVERE)) {
@@ -261,19 +277,19 @@ public class Log {
 
     /**
      * Writes an error message to the log (to be retrieved from the resource
-     * bundle), preceded by a line containing the error ID.
-     * 
-     * While the error message can be localised, the error ID should not, so
-     * developers can determine what error was given by looking at the error ID,
-     * instead of trying to interpret localised messages.
-     * 
-     * @param id
+     * bundle)
+     * <p>
+     * While the error message can be localized, the error key is also logged,
+     * so developers can determine what error was given by looking at the error
+     * key, instead of trying to interpret localized messages.
+     *
+     * @param ex
+     *            The error that was thrown
+     * @param key
      *            The key of the error message in the resource bundle
      * @param parameters
      *            Parameters for the error message. These are inserted by using
      *            StaticUtils.format.
-     * 
-     * @author Henry Pijffers (henry.pijffers@saxnot.com)
      */
     public static void logErrorRB(Throwable ex, String key, Object... parameters) {
         if (LOGGER.isLoggable(Level.SEVERE)) {
@@ -285,4 +301,22 @@ public class Log {
             LOGGER.log(rec);
         }
     }
+
+    /**
+     * Writes debug message to log (without localization)
+     * 
+     * @param message
+     *            message text
+     * @param parameters
+     *            Parameters for the error message. These are inserted by using
+     *            StaticUtils.format.
+     */
+    public static void logDebug(Logger logger, String message, Object... parameters) {
+        if (logger.isLoggable(Level.FINE)) {
+            LogRecord rec = new LogRecord(Level.FINE, message);
+            rec.setParameters(parameters);
+            rec.setLoggerName(logger.getName());
+            logger.log(rec);
+        }
+    }
 }
