@@ -8,22 +8,25 @@
                2008 Martin Fleurke
                2009 Didier Briel, Martin Fleurke
                2010 Didier Briel
+               2012 Martin Fleurke
+               2015 Didier Briel
                Home page: http://www.omegat.org/
                Support center: http://groups.yahoo.com/group/OmegaT/
 
- This program is free software; you can redistribute it and/or modify
+ This file is part of OmegaT.
+
+ OmegaT is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
+ OmegaT is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
 package org.omegat.util;
@@ -39,6 +42,11 @@ import java.util.regex.Pattern;
  * @author Martin Fleurke
  */
 public class PatternConsts {
+
+    private static final String RE_OMEGAT_TAG = "<\\/?[a-zA-Z]+[0-9]+\\/?>";
+    private static final String RE_PRINTF_VARS = "%([1-9]+\\$)?([+-])?('.)?(-)?([0-9]*)(\\.[0-9]*)?[bcdeEfFgGinopsuxX%]";
+    private static final String RE_SIMPLE_PRINTF_VARS = "%([1-9]+\\$)?([0-9]*)(\\.[0-9]*)?[bcdeEfFgGinopsuxX%]";
+    private static final String RE_SIMPLE_JAVA_MESSAGEFORMAT_PATTERN_VARS = "\\{([0-9])+\\}";
     /**
      * Compiled pattern to extract the encoding from XML file, if any. Found
      * encoding is stored in group #1.
@@ -77,8 +85,14 @@ public class PatternConsts {
             "<meta.*?content\\s*=\\s*[\"']\\s*text/html\\s*;\\s*charset\\s*=\\s*(\\S+?)[\"'].*?/?\\s*>",
             Pattern.CASE_INSENSITIVE);
 
-    /** compiled pattern to look for HTML file HEAD declaration */
-    public static final Pattern HTML_HEAD = Pattern.compile("<head.*?>", Pattern.CASE_INSENSITIVE);
+    /** compiled pattern to extract the encoding from HTML5 file, if any */
+    public static final Pattern HTML5_ENCODING = Pattern.compile(
+            "<meta.*?charset\\s*=\\s*[\"'](\\S+?)[\"'].*?/?\\s*>",
+            Pattern.CASE_INSENSITIVE);
+
+    /** Compiled pattern to look for HTML file HEAD declaration. 
+     Using [^e] instead of . prevents confusing &lt;head&gt; with &lt;header&gt; from HTML 5*/
+    public static final Pattern HTML_HEAD = Pattern.compile("<head[^e]*?>", Pattern.CASE_INSENSITIVE);
 
     /** compiled pattern to look for HTML file HTML declaration */
     public static final Pattern HTML_HTML = Pattern.compile("<html.*?>", Pattern.CASE_INSENSITIVE);
@@ -96,7 +110,7 @@ public class PatternConsts {
      * Pattern that matches omegat-specific tags (with leading &lt; and trailing
      * &gt; in any place of a string).
      */
-    public static final Pattern OMEGAT_TAG = Pattern.compile("<\\/?[a-zA-Z]+[0-9]+\\/?>");
+    public static final Pattern OMEGAT_TAG = Pattern.compile(RE_OMEGAT_TAG);
 
     /**
      * Pattern that matches omegat-specific tags (with leading &lt; and trailing
@@ -122,14 +136,24 @@ public class PatternConsts {
      * Call <code>matcher.group(n)</code> to get each piece.
      */
     public static final Pattern OMEGAT_TAG_DECOMPILE = Pattern.compile("<(\\/?)([a-zA-Z]+)([0-9]+)(\\/?)>");
+    
+    /**
+     * Pattern that matches paired tag in protected parts
+     * <ol>
+     * <li>opening tag
+     * <li>text between tags
+     * <li>closing tag
+     * </ol>
+     * Call <code>matcher.group(n)</code> to get each piece.
+     */
+    public static final Pattern PROTECTED_PARTS_PAIRED_TAG_DECOMPILE = Pattern.compile("^(<.+?>)([^<]+?)(<\\/.+?>)");
+
 
     /**
-     * Pattern that matches OmegaT-specific tags in the HTML form i.e, with
-     * &amp;lt; instead of &lt; and &amp;gt instead of &gt;. The pattern is also
-     * a capturing group.
+     * Pattern that matches an equiv-text attribute as used in XLIFF inline codes.
      */
-    public static final Pattern OMEGAT_HTML_TAG = Pattern.compile("(&lt;\\/?[a-zA-Z]+[0-9]+\\/?&gt;)");
-
+    public static final Pattern EQUIV_TEXT_ATTRIBUTE_DECOMPILE = Pattern.compile("equiv-text=\"([^\"]+)\"");
+    
     /** Pattern that detects space-only regular expressions. */
     public static final Pattern SPACY_REGEX = Pattern.compile("((\\s|\\\\n|\\\\t|\\\\s)(\\+|\\*)?)+");
 
@@ -146,6 +170,8 @@ public class PatternConsts {
             // [ 2138846 ] French dictionary cannot be downloaded and installed
             "\"([a-z]{1,8})(_([A-Z]{1,8})?)(_1-3-2)?\\.zip\"");
 
+    public static final Pattern SPACE_TAB = Pattern.compile("( |	)+");
+
     /**
      * Pattern for detecting the placeholders in a printf-function string which
      * can occur in languages like php, C and others. placeholder ::= "%"
@@ -168,16 +194,102 @@ public class PatternConsts {
      * <code>echo printf(gettext("%s is very %s"), "OmegaT", "great");</code>
      */
     public static final Pattern PRINTF_VARS = Pattern
-            .compile("%([1-9]+\\$)?([+-])?('.)?(-)?([0-9]*)(\\.[0-9]*)?[bcdeEfFgGinopsuxX%]",
-                    Pattern.CASE_INSENSITIVE);
+            .compile(RE_PRINTF_VARS);
     /**
      * Pattern for detecting the placeholders in a printf-function string. It
      * detects only simple placeholders, without SIGN-, PADDING-, ALIGNMENT- and
      * WIDTH specifier.
      * 
-     * @see PRINTF_VARS
+     * @see #PRINTF_VARS
      */
-    public static final Pattern SIMPLE_PRINTF_VARS = Pattern.compile(
-            "%([1-9]+\\$)?([0-9]*)(\\.[0-9]*)?[bcdeEfFgGinopsuxX%]", Pattern.CASE_INSENSITIVE);
+    public static final Pattern SIMPLE_PRINTF_VARS = Pattern.compile(RE_SIMPLE_PRINTF_VARS);
 
+    public static final Pattern SIMPLE_JAVA_MESSAGEFORMAT_PATTERN_VARS = Pattern.compile(RE_SIMPLE_JAVA_MESSAGEFORMAT_PATTERN_VARS);
+
+    /**
+     * Pattern for detecting OmegaT-tags and other placeholders (extended sprintf-variant) in texts
+     */
+    public static final Pattern SIMPLE_PLACEHOLDERS = Pattern.compile(RE_OMEGAT_TAG+"|"+RE_PRINTF_VARS);
+
+    /**
+     * combined pattern for all placeholder tags
+     */
+    private static Pattern PLACEHOLDERS;
+    /**
+     * pattern for text that should be removed from translation. Can be null!
+     */
+    private static Pattern REMOVE;
+    /**
+     * Pattern for text that should be considered a custom tag. Can be null!
+     */
+    private static Pattern CUSTOM_TAGS;
+
+    /**
+     * Returns the placeholder pattern (OmegaT tags, printf tags, java
+     * MessageFomat tags, custom tags, combined according to user configuration)
+     * 
+     * @return the pattern
+     * @see #updatePlaceholderPattern()
+     */
+    public static Pattern getPlaceholderPattern() {
+        if (PLACEHOLDERS == null) {
+            String regexp = RE_OMEGAT_TAG;
+            if ("true".equalsIgnoreCase(Preferences.getPreference(Preferences.CHECK_ALL_PRINTF_TAGS))) {
+                regexp += "|"+RE_PRINTF_VARS;
+            } else if ("true".equalsIgnoreCase(Preferences.getPreference(Preferences.CHECK_SIMPLE_PRINTF_TAGS))) {
+                regexp += "|"+RE_SIMPLE_PRINTF_VARS;
+            }
+            if ("true".equalsIgnoreCase(Preferences.getPreference(Preferences.CHECK_JAVA_PATTERN_TAGS))) {
+                regexp += "|"+RE_SIMPLE_JAVA_MESSAGEFORMAT_PATTERN_VARS;
+            }
+            //assume: customRegExp has already been validated.
+            String customRegExp = Preferences.getPreferenceDefaultAllowEmptyString(Preferences.CHECK_CUSTOM_PATTERN);
+            if (!"".equalsIgnoreCase(customRegExp)) {
+                regexp += "|"+customRegExp;
+            }
+            PLACEHOLDERS = Pattern.compile(regexp);
+        }
+        return PLACEHOLDERS;
+    }
+
+    /**
+     * Resets the placeholder pattern. Use it when the user has changed tagvalidation configuration.
+     */
+    public static void updatePlaceholderPattern() {
+        PLACEHOLDERS = null;
+    }
+
+    public static Pattern getRemovePattern() {
+        if (REMOVE == null) {
+            String removeRegExp = Preferences.getPreferenceDefaultAllowEmptyString(Preferences.CHECK_REMOVE_PATTERN);
+            if (!"".equalsIgnoreCase(removeRegExp)) {
+                REMOVE = Pattern.compile(removeRegExp);
+            }
+        }
+        return REMOVE;
+    }
+
+    /**
+     * Resets the remove pattern. Use it when the user has changed tagvalidation configuration.
+     */
+    public static void updateRemovePattern() {
+        REMOVE = null;
+    }
+
+    public static Pattern getCustomTagPattern() {
+        if (CUSTOM_TAGS == null) {
+            String customTagsRegex = Preferences.getPreferenceDefaultAllowEmptyString(Preferences.CHECK_CUSTOM_PATTERN);
+            if (!"".equalsIgnoreCase(customTagsRegex)) {
+                CUSTOM_TAGS = Pattern.compile(customTagsRegex);
+            }
+        }
+        return CUSTOM_TAGS;
+    }
+
+    /**
+     * Resets the remove pattern. Use it when the user has changed tagvalidation configuration.
+     */
+    public static void updateCustomTagPattern() {
+        CUSTOM_TAGS = null;
+    }
 }
