@@ -9,19 +9,20 @@
                Home page: http://www.omegat.org/
                Support center: http://groups.yahoo.com/group/OmegaT/
 
- This program is free software; you can redistribute it and/or modify
+ This file is part of OmegaT.
+
+ OmegaT is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
+ OmegaT is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
 package org.omegat.util;
@@ -39,7 +40,6 @@ import java.util.Stack;
 import java.util.regex.Matcher;
 
 import org.omegat.core.Core;
-import org.omegat.core.segmentation.Segmenter;
 import org.omegat.util.xml.XMLReader;
 import org.xml.sax.Attributes;
 import org.xml.sax.InputSource;
@@ -93,6 +93,7 @@ public class TMXReader extends org.xml.sax.helpers.DefaultHandler {
         this.sourceLanguage = sourceLanguage.getLanguage();
         this.targetLanguage = targetLanguage.getLanguage();
         this.isSegmentingEnabled = isSegmentingEnabled;
+        dateParser = new TMXDateParser();
     }
 
     /** Returns the source language */
@@ -113,7 +114,7 @@ public class TMXReader extends org.xml.sax.helpers.DefaultHandler {
     /** Returns an original text of a source segment #n */
     public String getSourceSegment(int n) {
         if (n < 0 || n >= numSegments())
-            return new String();
+            return "";
         else
             return m_srcList.get(n);
     }
@@ -121,7 +122,7 @@ public class TMXReader extends org.xml.sax.helpers.DefaultHandler {
     /** Returns a translation of a target segment #n */
     public String getTargetSegment(int n) {
         if (n < 0 || n >= numSegments())
-            return new String();
+            return "";
         else
             return m_tarList.get(n);
     }
@@ -254,7 +255,7 @@ public class TMXReader extends org.xml.sax.helpers.DefaultHandler {
     }
 
     /** Internal class for OmegaT tag */
-    class Tag {
+    static class Tag {
         /** is this an ending tag, e.g. &lt;/b4&gt; */
         public boolean end;
         /** name of the tag, e.g. "b" for &lt;/b4&gt; */
@@ -305,10 +306,10 @@ public class TMXReader extends org.xml.sax.helpers.DefaultHandler {
 
             int tagstart = matcher.start();
             int tagend = matcher.end();
-            boolean end = matcher.group(1).length() > 0;
+            boolean end = !matcher.group(1).isEmpty();
             String name = matcher.group(2);
             int num = Integer.parseInt(matcher.group(3));
-            boolean alone = matcher.group(4).length() > 0;
+            boolean alone = !matcher.group(4).isEmpty();
 
             if (num == 1)
                 num = 0;
@@ -334,9 +335,9 @@ public class TMXReader extends org.xml.sax.helpers.DefaultHandler {
             while (matcher.find()) {
                 tagstart = matcher.start();
                 tagend = matcher.end();
-                end = matcher.group(1).length() > 0;
+                end = !matcher.group(1).isEmpty();
                 name = matcher.group(2);
-                alone = matcher.group(4).length() > 0;
+                alone = !matcher.group(4).isEmpty();
                 tag = new Tag(end, name, num, alone);
 
                 if (end && unclosedTagsNames.containsKey(name)) {
@@ -381,12 +382,14 @@ public class TMXReader extends org.xml.sax.helpers.DefaultHandler {
 
     /** Collects a segment from TMX. Performs upgrades of a segment if needed. */
     private void storeSegment(String source, String translation, String changeId, long changeDate) {
+        source = StringUtil.normalizeUnicode(source);
         source = upgradeSegment(source);
+        translation = StringUtil.normalizeUnicode(translation);
         translation = upgradeSegment(translation);
 
         if (isUpgradeSentSeg()) {
-            List<String> srcSegments = Segmenter.segment(sourceLang, source, null, null);
-            List<String> tarSegments = Segmenter.segment(targetLang, translation, null, null);
+            List<String> srcSegments = Core.getSegmenter().segment(sourceLang, source, null, null);
+            List<String> tarSegments = Core.getSegmenter().segment(targetLang, translation, null, null);
 
             int n = srcSegments.size();
             if (n == tarSegments.size()) {
@@ -452,7 +455,7 @@ public class TMXReader extends org.xml.sax.helpers.DefaultHandler {
         // parse the TMX file
         try {
             // log the parsing attempt
-            Log.logRB("TMXR_INFO_READING_FILE", new Object[] { displayFilename });
+            Log.logRB("TMXR_INFO_READING_FILE", displayFilename);
 
             // create a new SAX parser factory
             javax.xml.parsers.SAXParserFactory parserFactory = javax.xml.parsers.SAXParserFactory
@@ -497,10 +500,9 @@ public class TMXReader extends org.xml.sax.helpers.DefaultHandler {
      * Receives notification of a parser warning. Called by SAX parser.
      */
     public void warning(SAXParseException exception) throws SAXException {
-        Log.logWarningRB(
-                "TMXR_WARNING_WHILE_PARSING",
-                new Object[] { String.valueOf(exception.getLineNumber()),
-                        String.valueOf(exception.getColumnNumber()) });
+        Log.logWarningRB("TMXR_WARNING_WHILE_PARSING",
+                exception.getLineNumber(),
+                exception.getColumnNumber());
         Log.log(exception);
     }
 
@@ -509,10 +511,9 @@ public class TMXReader extends org.xml.sax.helpers.DefaultHandler {
      * parser.
      */
     public void error(SAXParseException exception) throws SAXException {
-        Log.logErrorRB(
-                "TMXR_RECOVERABLE_ERROR_WHILE_PARSING",
-                new Object[] { String.valueOf(exception.getLineNumber()),
-                        String.valueOf(exception.getColumnNumber()) });
+        Log.logErrorRB("TMXR_RECOVERABLE_ERROR_WHILE_PARSING",
+                exception.getLineNumber(),
+                exception.getColumnNumber());
         Log.log(exception);
     }
 
@@ -520,10 +521,9 @@ public class TMXReader extends org.xml.sax.helpers.DefaultHandler {
      * Receives notification of a fatal XML parsing error. Called by SAX parser.
      */
     public void fatalError(SAXParseException exception) throws SAXException {
-        Log.logErrorRB(
-                "TMXR_FATAL_ERROR_WHILE_PARSING",
-                new Object[] { String.valueOf(exception.getLineNumber()),
-                        String.valueOf(exception.getColumnNumber()) });
+        Log.logErrorRB("TMXR_FATAL_ERROR_WHILE_PARSING",
+                exception.getLineNumber(),
+                exception.getColumnNumber());
         Log.log(exception);
     }
 
@@ -716,16 +716,16 @@ public class TMXReader extends org.xml.sax.helpers.DefaultHandler {
         tmxSourceLanguage = attributes.getValue(TMX_ATTR_SRCLANG);
 
         // log some details
-        Log.logRB("TMXR_INFO_CREATION_TOOL", new Object[] { creationtool });
-        Log.logRB("TMXR_INFO_CREATION_TOOL_VERSION", new Object[] { creationtoolversion });
-        Log.logRB("TMXR_INFO_SEG_TYPE", new Object[] { segtype });
-        Log.logRB("TMXR_INFO_SOURCE_LANG", new Object[] { tmxSourceLanguage });
+        Log.logRB("TMXR_INFO_CREATION_TOOL", creationtool);
+        Log.logRB("TMXR_INFO_CREATION_TOOL_VERSION", creationtoolversion);
+        Log.logRB("TMXR_INFO_SEG_TYPE", segtype);
+        Log.logRB("TMXR_INFO_SOURCE_LANG", tmxSourceLanguage);
 
         // give a warning if the TMX source language is
         // different from the project source language
         if (!tmxSourceLanguage.equalsIgnoreCase(sourceLanguage)) {
-            Log.logWarningRB("TMXR_WARNING_INCORRECT_SOURCE_LANG", new Object[] { tmxSourceLanguage,
-                    sourceLanguage });
+            Log.logWarningRB("TMXR_WARNING_INCORRECT_SOURCE_LANG", tmxSourceLanguage,
+                    sourceLanguage);
         }
 
         // give a warning that TMX file will be upgraded from 1.4.x
@@ -770,7 +770,7 @@ public class TMXReader extends org.xml.sax.helpers.DefaultHandler {
                 // add the language to the set, but trim off any region
                 // indicators
                 // for simplicity's sake, and convert it to all uppercase
-                if (language.length() > 0) {
+                if (!language.isEmpty()) {
                     m_variantLanguages.add(language.substring(0, 2).toUpperCase(Locale.ENGLISH));
                     if (languages.length() > 0)
                         languages.append(", ");
@@ -782,7 +782,7 @@ public class TMXReader extends org.xml.sax.helpers.DefaultHandler {
             } while (commaPos > 0);
 
             // Log presence of preferred variant languages
-            Log.logRB("TMXR_INFO_VARIANT_LANGUAGES_DISPLAYED", new Object[] { languages.toString() });
+            Log.logRB("TMXR_INFO_VARIANT_LANGUAGES_DISPLAYED", languages.toString());
         }
     }
 
@@ -939,11 +939,11 @@ public class TMXReader extends org.xml.sax.helpers.DefaultHandler {
         long changeDate = 0;
         if (target.changeDate != null) {
             try {
-                changeDate = TMXDateParser.parse(target.changeDate).getTime();
+                changeDate = dateParser.parse(target.changeDate).getTime();
             } catch (ParseException e) {
                 if (target.creationDate != null) {
                     try {
-                        changeDate = TMXDateParser.parse(target.creationDate).getTime();
+                        changeDate = dateParser.parse(target.creationDate).getTime();
                     } catch (ParseException e2) {
                     }
                 }
@@ -957,7 +957,7 @@ public class TMXReader extends org.xml.sax.helpers.DefaultHandler {
         // "pad" with empty strings, or omit segments if necessary
         // NOTE: this is not the most ideal solution, but the best possible
         for (int i = 0; i < source.subSegments.size(); i++) {
-            String starget = new String();
+            String starget = "";
             if (i < target.subSegments.size())
                 starget = target.subSegments.get(i).toString();
             storeSegment(source.subSegments.get(i).toString(), starget, changeId, changeDate);
@@ -1190,10 +1190,12 @@ public class TMXReader extends org.xml.sax.helpers.DefaultHandler {
                                     // (null if none)
     private boolean isSegmentingEnabled;
 
+    private TMXDateParser dateParser;
+
     /**
      * Internal class to represent translation unit variants
      */
-    private class TUV {
+    static private class TUV {
         /**
          * Language and (optional) country code: LL(C-CC)
          */
