@@ -4,23 +4,26 @@
           glossaries, and translation leveraging into updated projects.
 
  Copyright (C) 2000-2006 Keith Godfrey and Maxym Mykhalchuk
- Portions copyright 2007 - Zoltan Bartko - bartkozoltan@bartkozoltan.com
+               2006 Henry Pijffers (henry.pijffers@saxnot.com)
+               2007 Zoltan Bartko (bartkozoltan@bartkozoltan.com)
+               2015 Aaron Madlon-Kay
                Home page: http://www.omegat.org/
                Support center: http://groups.yahoo.com/group/OmegaT/
 
- This program is free software; you can redistribute it and/or modify
+ This file is part of OmegaT.
+
+ OmegaT is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
+ the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
 
- This program is distributed in the hope that it will be useful,
+ OmegaT is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
  **************************************************************************/
 
 package org.omegat.util;
@@ -36,15 +39,22 @@ import java.util.regex.Pattern;
  * 
  * @author Keith Godfrey
  * @author Maxym Mykhalchuk
+ * @author Henry Pijffers (henry.pijffers@saxnot.com)
+ * @author Zoltan Bartko
+ * @author Aaron Madlon-Kay
  */
 public class Token {
     /**
      * Two tokens are thought equal if their hash code is equal.
-     * 
-     * @author Henry Pijffers (henry.pijffers@saxnot.com)
      */
     public boolean equals(Object other) {
-        return ((this == other) || ((other instanceof Token) && (hash == ((Token) other).hash)));
+        if (this == other) {
+            return true;
+        }
+        if (other instanceof Token) {
+            return hash == ((Token) other).hash;
+        }
+        return false;
     }
 
     /**
@@ -113,4 +123,16 @@ public class Token {
     public String getTextFromString(String input) {
         return input.substring(offset, length + offset);
     }
+    
+    /**
+     * Get the strings represented by the provided tokens, from the original string
+     * they were produced from. For debugging purposes.
+     */
+    public static String[] getTextsFromString(Token[] tokens, String string) {
+        String[] result = new String[tokens.length];
+        for (int i = 0; i < tokens.length; i++) {
+            result[i] = tokens[i].getTextFromString(string);
+        }
+        return result;
+    }
 }
